@@ -3,28 +3,23 @@
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
 import {
-  getSelectedFrameId,
-  getSource,
-  getLocationSource,
   getSettledSourceTextContent,
   isMapScopesEnabled,
   getSelectedFrame,
-  getSelectedGeneratedScope,
-  getSelectedOriginalScope,
-  getThreadContext,
+  getGeneratedFrameScope,
+  getOriginalFrameScope,
   getFirstSourceActorForGeneratedSource,
-} from "../../selectors";
+} from "../../selectors/index";
 import {
   loadOriginalSourceText,
   loadGeneratedSourceText,
 } from "../sources/loadSourceText";
+import { validateSelectedFrame } from "../../utils/context";
 import { PROMISE } from "../utils/middleware/promise";
-import assert from "../../utils/assert";
 
 import { log } from "../../utils/log";
-import { isGenerated } from "../../utils/source";
 
-import { buildMappedScopes } from "../../utils/pause/mapScopes";
+import { buildMappedScopes } from "../../utils/pause/mapScopes/index";
 import { isFulfilled } from "../../utils/async-value";
 
 import { getMappedLocation } from "../../utils/source-maps";
@@ -32,16 +27,14 @@ import { getMappedLocation } from "../../utils/source-maps";
 const expressionRegex = /\bfp\(\)/g;
 
 export async function buildOriginalScopes(
-  frame,
+  selectedFrame,
   client,
-  cx,
-  frameId,
   generatedScopes
 ) {
-  if (!frame.originalVariables) {
+  if (!selectedFrame.originalVariables) {
     throw new TypeError("(frame.originalVariables: XScopeVariables)");
   }
-  const originalVariables = frame.originalVariables;
+  const originalVariables = selectedFrame.originalVariables;
   const frameBase = originalVariables.frameBase || "";
 
   const inputs = [];
@@ -55,7 +48,7 @@ export async function buildOriginalScopes(
   }
 
   const results = await client.evaluateExpressions(inputs, {
-    frameId,
+    frameId: selectedFrame.id,
   });
 
   const variables = {};
@@ -86,7 +79,7 @@ export async function buildOriginalScopes(
 }
 
 export function toggleMapScopes() {
-  return async function({ dispatch, getState }) {
+  return async function ({ dispatch, getState }) {
     if (isMapScopesEnabled(getState())) {
       dispatch({ type: "TOGGLE_MAP_SCOPES", mapScopes: false });
       return;
@@ -94,53 +87,63 @@ export function toggleMapScopes() {
 
     dispatch({ type: "TOGGLE_MAP_SCOPES", mapScopes: true });
 
-    const cx = getThreadContext(getState());
-
-    if (getSelectedOriginalScope(getState(), cx.thread)) {
+    // Ignore the call if there is no selected frame (we are not paused?)
+    const state = getState();
+    const selectedFrame = getSelectedFrame(state);
+    if (!selectedFrame) {
       return;
     }
 
-    const scopes = getSelectedGeneratedScope(getState(), cx.thread);
-    const frame = getSelectedFrame(getState(), cx.thread);
-    if (!scopes || !frame) {
+    if (getOriginalFrameScope(getState(), selectedFrame)) {
       return;
     }
 
-    dispatch(mapScopes(cx, Promise.resolve(scopes.scope), frame));
+    // Also ignore the call if we didn't fetch the scopes for the selected frame
+    const scopes = getGeneratedFrameScope(getState(), selectedFrame);
+    if (!scopes) {
+      return;
+    }
+
+    dispatch(mapScopes(selectedFrame, Promise.resolve(scopes.scope)));
   };
 }
 
-export function mapScopes(cx, scopes, frame) {
-  return async function(thunkArgs) {
-    const { dispatch, client, getState } = thunkArgs;
-    assert(cx.thread == frame.thread, "Thread mismatch");
+export function mapScopes(selectedFrame, scopes) {
+  return async function (thunkArgs) {
+    const { getState, dispatch, client } = thunkArgs;
 
     await dispatch({
       type: "MAP_SCOPES",
-      cx,
-      thread: cx.thread,
-      frame,
-      [PROMISE]: (async function() {
-        if (frame.isOriginal && frame.originalVariables) {
-          const frameId = getSelectedFrameId(getState(), cx.thread);
-          return buildOriginalScopes(frame, client, cx, frameId, scopes);
+      selectedFrame,
+      [PROMISE]: (async function () {
+        if (selectedFrame.isOriginal && selectedFrame.originalVariables) {
+          return buildOriginalScopes(selectedFrame, client, scopes);
         }
 
-        return dispatch(getMappedScopes(cx, scopes, frame));
+        // getMappedScopes is only specific to the sources where we map the variables
+        // in scope and so only need a thread context. Assert that we are on the same thread
+        // before retrieving a thread context.
+        validateSelectedFrame(getState(), selectedFrame);
+
+        return dispatch(getMappedScopes(scopes, selectedFrame));
       })(),
     });
   };
 }
 
-export function getMappedScopes(cx, scopes, frame) {
-  return async function(thunkArgs) {
+/**
+ * Get scopes mapped for a precise location.
+ *
+ * @param {Promise} scopes
+ *        Can be null. Result of Commands.js's client.getFrameScopes
+ * @param {Objects locations
+ *        Frame object, or custom object with 'location' and 'generatedLocation' attributes.
+ */
+export function getMappedScopes(scopes, locations) {
+  return async function (thunkArgs) {
     const { getState, dispatch } = thunkArgs;
-    const generatedSource = getLocationSource(
-      getState(),
-      frame.generatedLocation
-    );
-
-    const source = getLocationSource(getState(), frame.location);
+    const generatedSource = locations.generatedLocation.source;
+    const source = locations.location.source;
 
     if (
       !isMapScopesEnabled(getState()) ||
@@ -148,13 +151,13 @@ export function getMappedScopes(cx, scopes, frame) {
       !generatedSource ||
       generatedSource.isWasm ||
       source.isPrettyPrinted ||
-      isGenerated(source)
+      !source.isOriginal
     ) {
       return null;
     }
 
     // Load source text for the original source
-    await dispatch(loadOriginalSourceText({ cx, source }));
+    await dispatch(loadOriginalSourceText(source));
 
     const generatedSourceActor = getFirstSourceActorForGeneratedSource(
       getState(),
@@ -162,25 +165,19 @@ export function getMappedScopes(cx, scopes, frame) {
     );
 
     // Also load source text for its corresponding generated source
-    await dispatch(
-      loadGeneratedSourceText({
-        cx,
-        sourceActor: generatedSourceActor,
-      })
-    );
+    await dispatch(loadGeneratedSourceText(generatedSourceActor));
 
     try {
       const content =
-        getSource(getState(), source.id) &&
         // load original source text content
-        getSettledSourceTextContent(getState(), frame.location);
+        getSettledSourceTextContent(getState(), locations.location);
 
       return await buildMappedScopes(
         source,
         content && isFulfilled(content)
           ? content.value
           : { type: "text", value: "", contentType: undefined },
-        frame,
+        locations,
         await scopes,
         thunkArgs
       );
@@ -191,11 +188,13 @@ export function getMappedScopes(cx, scopes, frame) {
   };
 }
 
+/**
+ * Used to map variables used within conditional and log breakpoints.
+ */
 export function getMappedScopesForLocation(location) {
-  return async function(thunkArgs) {
-    const { dispatch, getState } = thunkArgs;
-    const cx = getThreadContext(getState());
+  return async function (thunkArgs) {
+    const { dispatch } = thunkArgs;
     const mappedLocation = await getMappedLocation(location, thunkArgs);
-    return dispatch(getMappedScopes(cx, null, mappedLocation));
+    return dispatch(getMappedScopes(null, mappedLocation));
   };
 }
