@@ -7,18 +7,18 @@
  * @module utils/source
  */
 
-const { getUnicodeUrl } = require("devtools/client/shared/unicode-url");
+const {
+  getUnicodeUrl,
+} = require("resource://devtools/client/shared/unicode-url.js");
 const {
   micromatch,
-} = require("devtools/client/shared/vendor/micromatch/micromatch.js");
+} = require("resource://devtools/client/shared/vendor/micromatch/micromatch.js");
 
 import { getRelativePath } from "../utils/sources-tree/utils";
 import { endTruncateStr } from "./utils";
 import { truncateMiddleText } from "../utils/text";
-import { parse as parseURL } from "../utils/url";
 import { memoizeLast } from "../utils/memoizeLast";
-import { renderWasmText } from "./wasm";
-import { toEditorLine } from "./editor";
+import { toWasmSourceLine, getEditor } from "./editor/index";
 export { isMinified } from "./isMinified";
 
 import { isFulfilled } from "./async-value";
@@ -161,14 +161,6 @@ export function isJavaScript(source, content) {
   );
 }
 
-/**
- * @memberof utils/source
- * @static
- */
-export function isPretty(source) {
-  return isPrettyURL(source.url);
-}
-
 export function isPrettyURL(url) {
   return url ? url.endsWith(":formatted") : false;
 }
@@ -208,37 +200,23 @@ function resolveFileURL(
 }
 
 export function getFormattedSourceId(id) {
+  if (typeof id != "string") {
+    console.error(
+      "Expected source id to be a string, got",
+      typeof id,
+      " | id:",
+      id
+    );
+    return "";
+  }
   return id.substring(id.lastIndexOf("/") + 1);
 }
 
 /**
- * Gets a readable filename from a source URL for display purposes.
- * If the source does not have a URL, the source ID will be returned instead.
- *
- * @memberof utils/source
- * @static
+ * Provides a middle-truncated filename displayed in Tab titles
  */
-export function getFilename(
-  source,
-  rawSourceURL = getRawSourceURL(source.url)
-) {
-  const { id } = source;
-  if (!rawSourceURL) {
-    return getFormattedSourceId(id);
-  }
-
-  const { filename } = source.displayURL;
-  return getRawSourceURL(filename);
-}
-
-/**
- * Provides a middle-trunated filename
- *
- * @memberof utils/source
- * @static
- */
-export function getTruncatedFileName(source, querystring = "", length = 30) {
-  return truncateMiddleText(`${getFilename(source)}${querystring}`, length);
+export function getTruncatedFileName(source) {
+  return truncateMiddleText(source.longName, 30);
 }
 
 /* Gets path for files with same filename for editor tabs, breakpoints, etc.
@@ -250,15 +228,13 @@ export function getTruncatedFileName(source, querystring = "", length = 30) {
 
 export function getDisplayPath(mySource, sources) {
   const rawSourceURL = getRawSourceURL(mySource.url);
-  const filename = getFilename(mySource, rawSourceURL);
+  const filename = mySource.shortName;
 
   // Find sources that have the same filename, but different paths
   // as the original source
   const similarSources = sources.filter(source => {
     const rawSource = getRawSourceURL(source.url);
-    return (
-      rawSourceURL != rawSource && filename == getFilename(source, rawSource)
-    );
+    return rawSourceURL != rawSource && filename == source.shortName;
   });
 
   if (!similarSources.length) {
@@ -312,41 +288,6 @@ export function getFileURL(source, truncate = true) {
   return resolveFileURL(url, getUnicodeUrl, truncate);
 }
 
-export function getSourcePath(url) {
-  if (!url) {
-    return "";
-  }
-
-  const { path, href } = parseURL(url);
-  // for URLs like "about:home" the path is null so we pass the full href
-  return path || href;
-}
-
-/**
- * Returns amount of lines in the source. If source is a WebAssembly binary,
- * the function returns amount of bytes.
- */
-export function getSourceLineCount(content) {
-  if (content.type === "wasm") {
-    const { binary } = content.value;
-    return binary.length;
-  }
-
-  let count = 0;
-
-  for (let i = 0; i < content.value.length; ++i) {
-    if (content.value[i] === "\n") {
-      ++count;
-    }
-  }
-
-  return count + 1;
-}
-
-export function isInlineScript(source) {
-  return source.introductionType === "scriptElement";
-}
-
 function getNthLine(str, lineNum) {
   let startIndex = -1;
 
@@ -375,9 +316,9 @@ export const getLineText = memoizeLast((sourceId, asyncContent, line) => {
   const content = asyncContent.value;
 
   if (content.type === "wasm") {
-    const editorLine = toEditorLine(sourceId, line);
-    const lines = renderWasmText(sourceId, content);
-    return lines[editorLine] || "";
+    const editor = getEditor();
+    const lines = editor.renderWasmText(content);
+    return lines[toWasmSourceLine(line)] || "";
   }
 
   const lineText = getNthLine(content.value, line - 1);
@@ -396,22 +337,13 @@ export function getTextAtPosition(sourceId, asyncContent, location) {
  *
  * @param {Object} source
  *        The reducer source object.
- * @param {Object} symbols
- *        The reducer symbol object for the given source.
  * @param {Boolean} isBlackBoxed
  *        To be set to true, when the given source is blackboxed.
- * @param {Boolean} hasPrettyTab
- *        To be set to true, if the given source isn't the pretty printed one,
  *        but another tab for that source is opened pretty printed.
  * @return String
  *        The classname to use.
  */
-export function getSourceClassnames(
-  source,
-  symbols,
-  isBlackBoxed,
-  hasPrettyTab = false
-) {
+export function getSourceClassnames(source, isBlackBoxed) {
   // Conditionals should be ordered by priority of icon!
   const defaultClassName = "file";
 
@@ -419,21 +351,10 @@ export function getSourceClassnames(
     return defaultClassName;
   }
 
-  // In the SourceTree, we don't show the pretty printed sources,
-  // but still want to show the pretty print icon when a pretty printed tab
-  // for the current source is opened.
-  if (isPretty(source) || hasPrettyTab) {
-    return "prettyPrint";
-  }
-
   if (isBlackBoxed) {
     return "blackBox";
   }
 
-  if (symbols && symbols.framework) {
-    return symbols.framework.toLowerCase();
-  }
-
   if (isUrlExtension(source.url)) {
     return "extension";
   }
@@ -452,55 +373,6 @@ export function getRelativeUrl(source, root) {
   return url.slice(url.indexOf(root) + root.length + 1);
 }
 
-/**
- * source.url doesn't include thread actor ID, so before calling underRoot(), the thread actor ID
- * must be removed from the root, which this function handles.
- * @param {string} root The root url to be cleaned
- * @param {Set<Thread>} threads The list of threads
- * @returns {string} The root url with thread actor IDs removed
- */
-export function removeThreadActorId(root, threads) {
-  threads.forEach(thread => {
-    if (root.includes(thread.actor)) {
-      root = root.slice(thread.actor.length + 1);
-    }
-  });
-  return root;
-}
-
-/**
- * Checks if the source is descendant of the root identified by the
- * root url specified. The root might likely be projectDirectoryRoot which
- * is a defined by a pref that allows users restrict the source tree to
- * a subset of sources.
- *
- * @param {Object} source
- *                  The source object
- * @param {String} rootUrlWithoutThreadActor
- *                 The url for the root node, without the thread actor ID. This can be obtained
- *                 by calling removeThreadActorId()
- */
-export function isDescendantOfRoot(source, rootUrlWithoutThreadActor) {
-  if (source.url && source.url.includes("chrome://")) {
-    const { group, path } = source.displayURL;
-    return (group + path).includes(rootUrlWithoutThreadActor);
-  }
-
-  return !!source.url && source.url.includes(rootUrlWithoutThreadActor);
-}
-
-export function isGenerated(source) {
-  return !source.isOriginal;
-}
-
-export function getSourceQueryString(source) {
-  if (!source) {
-    return "";
-  }
-
-  return parseURL(getRawSourceURL(source.url)).search;
-}
-
 export function isUrlExtension(url) {
   return url.includes("moz-extension:") || url.includes("chrome-extension");
 }
