@@ -19,6 +19,9 @@ const EXAMPLE_URL =
 const EXAMPLE_REMOTE_URL =
   "https://example.org/browser/devtools/client/debugger/test/mochitest/examples/";
 
+const EXAMPLE_URL_WITH_PORT =
+  "http://mochi.test:8888/browser/devtools/client/debugger/test/mochitest/examples/";
+
 // shared-head.js handles imports, constants, and utility functions
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/shared/test/shared-head.js",
@@ -35,6 +38,13 @@ Services.scriptloader.loadSubScript(
   this
 );
 
+// Cleanup preferences set by the tracer.
+registerCleanupFunction(() => {
+  for (const pref of ["logging.console", "logging.PageMessages"]) {
+    Services.prefs.clearUserPref(pref);
+  }
+});
+
 /**
  * Helper function for `_loadAllIntegrationTests`.
  *
@@ -116,21 +126,20 @@ const INTEGRATION_TEST_PAGE_SOURCES = [
   "replaced-bundle.js",
   "removed-original.js",
   "named-eval.js",
+  "react-component-module.js",
   // This is the JS file with encoded characters and custom protocol
   "文字コード.js",
   // Webpack generated some extra sources:
   "bootstrap 3b1a221408fdde86aa49",
   "bootstrap a1ecee2f86e1d0ea3fb5",
-  "bootstrap 6fda1f7ea9ecbc1a2d5b",
+  "bootstrap d343aa81956b90d9f67e",
   // There is 3 occurences, one per target (main thread, worker and iframe).
   // But there is even more source actors (named evals and duplicated script tags).
   "same-url.sjs",
   "same-url.sjs",
+  "log-worker.js",
+  "same-url.sjs",
 ];
-// The iframe one is only available when fission is enabled, or EFT
-if (isFissionEnabled() || isEveryFrameTargetEnabled()) {
-  INTEGRATION_TEST_PAGE_SOURCES.push("same-url.sjs");
-}
 
 /**
  * Install a Web Extension which will run a content script against any test page
@@ -220,12 +229,17 @@ async function assertBreakableLines(
 ) {
   await selectSource(dbg, source);
   is(
-    getCM(dbg).lineCount(),
+    getLineCount(dbg),
     numberOfLines,
     `We show the expected number of lines in CodeMirror for ${source}`
   );
   for (let line = 1; line <= numberOfLines; line++) {
-    assertLineIsBreakable(dbg, source, line, breakableLines.includes(line));
+    await assertLineIsBreakable(
+      dbg,
+      source,
+      line,
+      breakableLines.includes(line)
+    );
   }
 }
 
@@ -244,3 +258,51 @@ function getRange(start, end) {
   }
   return range;
 }
+
+/**
+ * Get the currently selected line number displayed in the editor's footer.
+ */
+function assertCursorPosition(dbg, expectedLine, expectedColumn, message) {
+  const cursorPosition = findElementWithSelector(dbg, ".cursor-position");
+  if (!cursorPosition) {
+    ok(false, message + " (no cursor displayed in footer)");
+  }
+  // Cursor position text has the following shape: (L, C)
+  // where L is the line number, and C the column number
+  const match = cursorPosition.innerText.match(/\((\d+), (\d+)\)/);
+  if (!match) {
+    ok(
+      false,
+      message +
+        ` (wrong cursor content in footer : '${cursorPosition.innerText}')`
+    );
+  }
+  const [_, line, column] = match;
+  is(parseInt(line, 10), expectedLine, message + " (footer line)");
+  is(parseInt(column, 10), expectedColumn, message + " (footer column)");
+  const cursor = getCMEditor(dbg).getSelectionCursor();
+  is(cursor.from.line, expectedLine, message + " (actual cursor line)");
+  // CodeMirror column is 0-based while the location mentioned in test 1-based.
+  is(cursor.from.ch + 1, expectedColumn, message + " (actual cursor column)");
+}
+
+async function waitForCursorPosition(dbg, expectedLine) {
+  return waitFor(() => {
+    const cursorPosition = findElementWithSelector(dbg, ".cursor-position");
+    if (!cursorPosition) {
+      return false;
+    }
+    const { innerText } = cursorPosition;
+    // Cursor position text has the following shape: (L, C)
+    // where L is the line number, and C the column number
+    const line = innerText.substring(1, innerText.indexOf(","));
+    return parseInt(line, 10) == expectedLine;
+  });
+}
+
+/**
+ * @see selectDebuggerContextMenuItem in debugger/test/mochitest/shared-head.js
+ */
+function selectContextMenuItem(dbg, selector) {
+  return selectDebuggerContextMenuItem(dbg, selector);
+}
