@@ -1,5 +1,3 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
@@ -55,7 +53,12 @@ add_task(async function checkMenuEntryStates() {
   }
 
   const onContextMenuHidden = toolbox.once("menu-close");
-  EventUtils.sendKey("ESCAPE", toolbox.win);
+  if (Services.prefs.getBoolPref("widget.macos.native-context-menus", false)) {
+    info("Using hidePopup semantics because of macOS native context menus.");
+    textboxContextMenu.hidePopup();
+  } else {
+    EventUtils.sendKey("ESCAPE", toolbox.win);
+  }
   await onContextMenuHidden;
 });
 
@@ -65,11 +68,10 @@ add_task(async function automaticallyBindTexbox() {
   );
   gDevTools.registerTool({
     id: textboxToolId,
-    isTargetSupported: () => true,
-    url: `data:text/html;charset=utf8,<input /><input type='text' />
-            <input type='search' /><textarea></textarea><input type='radio' />`,
+    isToolSupported: () => true,
+    url: CHROME_URL_ROOT + "doc_textbox_tool.html",
     label: "Context menu works without tool intervention",
-    build: function(iframeWindow, toolbox) {
+    build(iframeWindow, toolbox) {
       this.panel = createTestPanel(iframeWindow, toolbox);
       return this.panel.open();
     },
@@ -122,7 +124,12 @@ async function checkTextBox(textBox, toolbox) {
 
   info("Closing the menu");
   const onContextMenuHidden = toolbox.once("menu-close");
-  EventUtils.sendKey("ESCAPE", toolbox.win);
+  if (Services.prefs.getBoolPref("widget.macos.native-context-menus", false)) {
+    info("Using hidePopup semantics because of macOS native context menus.");
+    textboxContextMenu.hidePopup();
+  } else {
+    EventUtils.sendKey("ESCAPE", toolbox.win);
+  }
   await onContextMenuHidden;
 
   textboxContextMenu = toolbox.getTextBoxContextMenu();
