@@ -48,13 +48,20 @@ class StyleRuleFront extends FrontClassWithSpec(styleRuleSpec) {
    * trait is true; otherwise a RuleModificationList will be
    * returned.
    *
+   * @param {Window} win
+   *                 This is needed by the RuleRewriter.
    * @param {CssPropertiesFront} cssProperties
    *                             This is needed by the RuleRewriter.
    * @return {RuleModificationList}
    */
-  startModifyingProperties(cssProperties) {
+  startModifyingProperties(win, cssProperties) {
     if (this.canSetRuleText) {
-      return new RuleRewriter(cssProperties.isKnown, this, this.authoredText);
+      return new RuleRewriter(
+        win,
+        cssProperties.isKnown,
+        this,
+        this.authoredText
+      );
     }
     return new RuleModificationList(this);
   }
@@ -71,6 +78,9 @@ class StyleRuleFront extends FrontClassWithSpec(styleRuleSpec) {
   get cssText() {
     return this._form.cssText;
   }
+  get isNestedDeclarations() {
+    return !!this._form.isNestedDeclarations;
+  }
   get authoredText() {
     return typeof this._form.authoredText === "string"
       ? this._form.authoredText
@@ -88,26 +98,42 @@ class StyleRuleFront extends FrontClassWithSpec(styleRuleSpec) {
   get selectors() {
     return this._form.selectors;
   }
+  get selectorsSpecificity() {
+    return this._form.selectorsSpecificity;
+  }
 
   /**
-   * When a rule is nested in another non-at-rule (aka CSS Nesting), this will return
-   * the "full" selectors, which includes ancestor rules selectors.
-   * To compute it, the parent selector (&) is recursively replaced by the parent
-   * rule selector wrapped in `:is()`.
-   * For example, with the following nested rule: `body { & > main {} }`,
-   * the desugared selectors will be [`:is(body) > main`],
-   * while the "regular" selectors will only be [`main`].
+   * Returns a concatenation of the rule's selector and all its ancestor "selectors".
+   * This is different from a "desugared" selector as what's returned is not an
+   * actual selector, but some kind of key that represent the rule selectors.
+   * This is used for the selector highlighter, where we need to know what's
+   * being highlighted.
    *
-   * See https://www.w3.org/TR/css-nesting-1/#nest-selector for more information.
-   *
-   * @returns {Array<String>} An array of the desugared selectors for this rule.
-   *                          This falls back to the regular list of selectors
-   *                          when desugared selectors are not sent by the server.
+   * @returns {String}
    */
-  get desugaredSelectors() {
-    // We don't send the desugaredSelectors for top-level selectors, so fall back to
-    // the regular selectors in that case.
-    return this._form.desugaredSelectors || this._form.selectors;
+  get computedSelector() {
+    let selector = "";
+    for (const ancestor of this.ancestorData) {
+      let ancestorSelector;
+      if (ancestor.selectors) {
+        ancestorSelector = ancestor.selectors.join(",");
+      } else if (ancestor.type === "container") {
+        ancestorSelector =
+          ancestor.containerName + " " + ancestor.containerQuery;
+      } else if (ancestor.type === "supports") {
+        ancestorSelector = ancestor.conditionText;
+      } else if (ancestor.value) {
+        ancestorSelector = ancestor.value;
+      }
+      selector +=
+        "/" + (ancestor.type ? ancestor.type + " " : "") + ancestorSelector;
+    }
+
+    return (selector ? selector + "/" : "") + this._form.selectors.join(",");
+  }
+
+  get selectorWarnings() {
+    return this._form.selectorWarnings;
   }
 
   get parentStyleSheet() {
@@ -152,6 +178,10 @@ class StyleRuleFront extends FrontClassWithSpec(styleRuleSpec) {
     return this._form.ancestorData;
   }
 
+  get userAdded() {
+    return this._form.userAdded;
+  }
+
   async modifySelector(node, value) {
     const response = await super.modifySelector(
       node,
