@@ -6,9 +6,9 @@
 
 const {
   PureComponent,
-} = require("resource://devtools/client/shared/vendor/react.js");
+} = require("resource://devtools/client/shared/vendor/react.mjs");
 const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
-const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
 
 class EndDelaySign extends PureComponent {
   static get propTypes() {
@@ -20,11 +20,8 @@ class EndDelaySign extends PureComponent {
 
   render() {
     const { animation, timeScale } = this.props;
-    const {
-      endDelay,
-      endTime,
-      isEndDelayFilled,
-    } = animation.state.absoluteValues;
+    const { endDelay, endTime, isEndDelayFilled } =
+      animation.state.absoluteValues;
 
     const toPercentage = v => (v / timeScale.getDuration()) * 100;
     const absEndDelay = Math.abs(endDelay);
