@@ -10,11 +10,6 @@ Services.scriptloader.loadSubScript(
   this
 );
 
-Services.prefs.setIntPref("devtools.toolbox.footer.height", 350);
-registerCleanupFunction(() => {
-  Services.prefs.clearUserPref("devtools.toolbox.footer.height");
-});
-
 /**
  * Is the given node visible in the page (rendered in the frame tree).
  * @param {DOMNode}
@@ -70,21 +65,22 @@ function waitForMarkupLoaded(inspector) {
 }
 
 function getStyle(browser, selector, propertyName) {
-  return SpecialPowers.spawn(browser, [selector, propertyName], async function(
-    _selector,
-    _propertyName
-  ) {
-    return content.document
-      .querySelector(_selector)
-      .style.getPropertyValue(_propertyName);
-  });
+  return SpecialPowers.spawn(
+    browser,
+    [selector, propertyName],
+    async function (_selector, _propertyName) {
+      return content.document
+        .querySelector(_selector)
+        .style.getPropertyValue(_propertyName);
+    }
+  );
 }
 
 function setStyle(browser, selector, propertyName, value) {
   return SpecialPowers.spawn(
     browser,
     [selector, propertyName, value],
-    async function(_selector, _propertyName, _value) {
+    async function (_selector, _propertyName, _value) {
       content.document.querySelector(_selector).style[_propertyName] = _value;
     }
   );
@@ -96,7 +92,7 @@ function setStyle(browser, selector, propertyName, value) {
  * finished updating. We also need to wait for the "boxmodel-view-updated" event.
  */
 var _selectNode = selectNode;
-selectNode = async function(node, inspector, reason) {
+selectNode = async function (node, inspector, reason) {
   const onUpdate = waitForUpdate(inspector, true);
   await _selectNode(node, inspector, reason);
   await onUpdate;
