@@ -16,9 +16,9 @@ const TEST_URI = `
   </div>
 `;
 
-add_task(async function() {
+add_task(async function () {
   await addTab("data:text/html;charset=utf-8," + encodeURIComponent(TEST_URI));
-  const { inspector, flexboxInspector, testActor } = await openLayoutView();
+  const { inspector, flexboxInspector } = await openLayoutView();
   const { document: doc } = flexboxInspector;
 
   const onFlexItemSizingRendered = waitForDOM(doc, "ul.flex-item-sizing");
@@ -26,14 +26,23 @@ add_task(async function() {
   const [flexItemSizingContainer] = await onFlexItemSizingRendered;
 
   info("Checking the initial state of the flex item list.");
-  is(flexItemSizingContainer.querySelectorAll("li").length, 2,
-    "Got the correct number of flex item sizing properties in the list.");
+  is(
+    flexItemSizingContainer.querySelectorAll("li").length,
+    2,
+    "Got the correct number of flex item sizing properties in the list."
+  );
 
   info("Changing the flexbox in the page.");
-  const onFlexItemSizingChanged = waitForDOM(doc, "ul.flex-item-sizing > li", 3);
-  testActor.eval(`
-    document.getElementById("item").style.minWidth = "100px";
-  `);
+  const onFlexItemSizingChanged = waitForDOM(
+    doc,
+    "ul.flex-item-sizing > li",
+    3
+  );
+  SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [],
+    () => (content.document.getElementById("item").style.minWidth = "100px")
+  );
   const elements = await onFlexItemSizingChanged;
 
   info("Checking the flex item sizing info is correct.");
