@@ -4,14 +4,13 @@
 
 "use strict";
 
-const Services = require("Services");
-const { PureComponent } = require("devtools/client/shared/vendor/react");
-const dom = require("devtools/client/shared/vendor/react-dom-factories");
-const PropTypes = require("devtools/client/shared/vendor/react-prop-types");
+const {
+  PureComponent,
+} = require("resource://devtools/client/shared/vendor/react.mjs");
+const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
 
-const Types = require("../types");
-
-const FONT_HIGHLIGHTER_PREF = "devtools.inspector.fonthighlighter.enabled";
+const Types = require("resource://devtools/client/inspector/fonts/types.js");
 
 class FontName extends PureComponent {
   static get propTypes() {
@@ -28,34 +27,26 @@ class FontName extends PureComponent {
   }
 
   onNameMouseOver() {
-    const {
-      font,
-      onToggleFontHighlight,
-    } = this.props;
+    const { font, onToggleFontHighlight } = this.props;
 
     onToggleFontHighlight(font, true);
   }
 
   onNameMouseOut() {
-    const {
-      font,
-      onToggleFontHighlight,
-    } = this.props;
+    const { font, onToggleFontHighlight } = this.props;
 
     onToggleFontHighlight(font, false);
   }
 
   render() {
-    const options = {
-      className: "font-name"
-    };
-
-    if (Services.prefs.getBoolPref(FONT_HIGHLIGHTER_PREF)) {
-      options.onMouseOver = this.onNameMouseOver;
-      options.onMouseOut = this.onNameMouseOut;
-    }
-
-    return dom.span(options, this.props.font.name);
+    return dom.span(
+      {
+        className: "font-name",
+        onMouseOver: this.onNameMouseOver,
+        onMouseOut: this.onNameMouseOut,
+      },
+      this.props.font.name
+    );
   }
 }
 
