@@ -4,12 +4,17 @@
 
 "use strict";
 
-const { createRef, PureComponent } = require("devtools/client/shared/vendor/react");
-const dom = require("devtools/client/shared/vendor/react-dom-factories");
-const PropTypes = require("devtools/client/shared/vendor/react-prop-types");
+const {
+  createRef,
+  PureComponent,
+} = require("resource://devtools/client/shared/vendor/react.mjs");
+const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
 
-const Types = require("../types");
-const { getStr } = require("../utils/l10n");
+const Types = require("resource://devtools/client/inspector/fonts/types.js");
+const {
+  getStr,
+} = require("resource://devtools/client/inspector/fonts/utils/l10n.js");
 
 const PREVIEW_TEXT_MAX_LENGTH = 30;
 
@@ -37,7 +42,7 @@ class FontPreviewInput extends PureComponent {
     const value = e.target.value;
     this.props.onPreviewTextChange(value);
 
-    this.setState((prevState) => {
+    this.setState(prevState => {
       return { ...prevState, value };
     });
   }
@@ -59,7 +64,7 @@ class FontPreviewInput extends PureComponent {
         className: "devtools-searchinput",
         onChange: this.onChange,
         onFocus: this.onFocus,
-        maxlength: PREVIEW_TEXT_MAX_LENGTH,
+        maxLength: PREVIEW_TEXT_MAX_LENGTH,
         placeholder: getStr("fontinspector.previewTextPlaceholder"),
         ref: this.inputRef,
         type: "text",
