@@ -5,7 +5,7 @@
 "use strict";
 
 const {
-  style: { ELEMENT_STYLE },
+  style: { ELEMENT_STYLE, PRES_HINTS },
 } = require("resource://devtools/shared/constants.js");
 const CssLogic = require("resource://devtools/shared/inspector/css-logic.js");
 const TextProperty = require("resource://devtools/client/inspector/rules/models/text-property.js");
@@ -53,11 +53,11 @@ class Rule {
     this.compatibilityIssues = null;
 
     this.matchedSelectorIndexes = options.matchedSelectorIndexes || [];
-    this.pseudoElement = options.pseudoElement || "";
     this.isSystem = options.isSystem;
     this.isUnmatched = options.isUnmatched || false;
     this.darkColorScheme = options.darkColorScheme;
     this.inherited = options.inherited || null;
+    this.pseudoElement = options.pseudoElement || "";
     this.keyframes = options.keyframes || null;
     this.userAdded = options.rule.userAdded;
 
@@ -90,17 +90,6 @@ class Rule {
     return this.textProps;
   }
 
-  get inheritance() {
-    if (!this.inherited) {
-      return null;
-    }
-
-    return {
-      inherited: this.inherited,
-      inheritedSource: this.inheritedSource,
-    };
-  }
-
   get selector() {
     return {
       getUniqueSelector: this.getUniqueSelector,
@@ -125,22 +114,25 @@ class Rule {
     return title;
   }
 
-  get inheritedSource() {
-    if (this._inheritedSource) {
-      return this._inheritedSource;
+  get inheritedSectionLabel() {
+    if (this._inheritedSectionLabel) {
+      return this._inheritedSectionLabel;
     }
-    this._inheritedSource = "";
+    this._inheritedSectionLabel = "";
     if (this.inherited) {
       let eltText = this.inherited.displayName;
       if (this.inherited.id) {
         eltText += "#" + this.inherited.id;
       }
-      this._inheritedSource = STYLE_INSPECTOR_L10N.getFormatStr(
+      if (CssLogic.ELEMENT_BACKED_PSEUDO_ELEMENTS.has(this.pseudoElement)) {
+        eltText += this.pseudoElement;
+      }
+      this._inheritedSectionLabel = STYLE_INSPECTOR_L10N.getFormatStr(
         "rule.inheritedFrom",
         eltText
       );
     }
-    return this._inheritedSource;
+    return this._inheritedSectionLabel;
   }
 
   get keyframesName() {
@@ -169,9 +161,15 @@ class Rule {
   }
 
   get selectorText() {
-    return this.domRule.selectors
-      ? this.domRule.selectors.join(", ")
-      : CssLogic.l10n("rule.sourceElement");
+    if (Array.isArray(this.domRule.selectors)) {
+      return this.domRule.selectors.join(", ");
+    }
+
+    if (this.domRule.type === PRES_HINTS) {
+      return CssLogic.l10n("rule.sourceElementAttributesStyle");
+    }
+
+    return CssLogic.l10n("rule.sourceElement");
   }
 
   /**
@@ -283,7 +281,13 @@ class Rule {
    *        Optional, property next to which the new property will be added.
    */
   createProperty(name, value, priority, enabled, siblingProp) {
-    const prop = new TextProperty(this, name, value, priority, enabled);
+    const prop = new TextProperty({
+      rule: this,
+      name,
+      value,
+      priority,
+      enabled,
+    });
 
     let ind;
     if (siblingProp) {
@@ -592,14 +596,15 @@ class Rule {
         name,
         prop.value
       );
-      const textProp = new TextProperty(
-        this,
+
+      const textProp = new TextProperty({
+        rule: this,
         name,
         value,
-        prop.priority,
-        !("commentOffsets" in prop),
-        invisible
-      );
+        priority: prop.priority,
+        enabled: !("commentOffsets" in prop),
+        invisible,
+      });
       textProps.push(textProp);
     }
 
@@ -626,7 +631,12 @@ class Rule {
         prop.name,
         prop.value
       );
-      const textProp = new TextProperty(this, prop.name, value, prop.priority);
+      const textProp = new TextProperty({
+        rule: this,
+        name: prop.name,
+        value,
+        priority: prop.priority,
+      });
       textProp.enabled = false;
       textProps.push(textProp);
     }
@@ -884,6 +894,13 @@ class Rule {
     );
   }
 
+  /**
+   * @returns {Boolean} Whether or not the rule can be edited
+   */
+  isEditable() {
+    return !this.isSystem && this.domRule.type !== PRES_HINTS;
+  }
+
   /**
    * See whether this rule has any non-invisible properties.
    * @return {Boolean} true if there is any visible property, or false
