@@ -1,4 +1,3 @@
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
  http://creativecommons.org/publicdomain/zero/1.0/ */
 
@@ -19,9 +18,9 @@ const TEST_URI = `
   <div id="testid2">Styled Node</div>
 `;
 
-add_task(async function() {
+add_task(async function () {
   await addTab("data:text/html;charset=utf-8," + encodeURIComponent(TEST_URI));
-  const {inspector, view} = await openRuleView();
+  const { inspector, view } = await openRuleView();
   await selectNode("#testid", inspector);
 
   await testEditProperty(inspector, view);
@@ -31,23 +30,30 @@ add_task(async function() {
 
 async function testEditProperty(inspector, ruleView) {
   const idRule = getRuleViewRuleEditor(ruleView, 1).rule;
-  const prop = idRule.textProps[0];
+  const prop = getTextProperty(ruleView, 1, { "background-color": "blue" });
 
   let editor = await focusEditableField(ruleView, prop.editor.nameSpan);
   let input = editor.input;
-  is(inplaceEditor(prop.editor.nameSpan), editor,
-    "Next focused editor should be the name editor.");
+  is(
+    inplaceEditor(prop.editor.nameSpan),
+    editor,
+    "Next focused editor should be the name editor."
+  );
 
-  ok(input.selectionStart === 0 && input.selectionEnd === input.value.length,
-    "Editor contents are selected.");
+  ok(
+    input.selectionStart === 0 && input.selectionEnd === input.value.length,
+    "Editor contents are selected."
+  );
 
   // Try clicking on the editor's input again, shouldn't cause trouble
   // (see bug 761665).
   EventUtils.synthesizeMouse(input, 1, 1, {}, ruleView.styleWindow);
   input.select();
 
-  info("Entering property name \"border-color\" followed by a colon to " +
-    "focus the value");
+  info(
+    'Entering property name "border-color" followed by a colon to ' +
+      "focus the value"
+  );
   const onNameDone = ruleView.once("ruleview-changed");
   const onFocus = once(idRule.editor.element, "focus", true);
   EventUtils.sendString("border-color:", ruleView.styleWindow);
@@ -57,10 +63,15 @@ async function testEditProperty(inspector, ruleView) {
   info("Verifying that the focused field is the valueSpan");
   editor = inplaceEditor(ruleView.styleDocument.activeElement);
   input = editor.input;
-  is(inplaceEditor(prop.editor.valueSpan), editor,
-    "Focus should have moved to the value.");
-  ok(input.selectionStart === 0 && input.selectionEnd === input.value.length,
-    "Editor contents are selected.");
+  is(
+    inplaceEditor(prop.editor.valueSpan),
+    editor,
+    "Focus should have moved to the value."
+  );
+  ok(
+    input.selectionStart === 0 && input.selectionEnd === input.value.length,
+    "Editor contents are selected."
+  );
 
   info("Entering a value following by a semi-colon to commit it");
   const onBlur = once(editor.input, "blur");
@@ -71,50 +82,42 @@ async function testEditProperty(inspector, ruleView) {
     EventUtils.sendChar(ch, ruleView.styleWindow);
     ruleView.debounce.flush();
     await onPreviewDone;
-    is(prop.editor.warning.hidden, true,
-      "warning triangle is hidden or shown as appropriate");
+    ok(!prop.editor.warning, "warning triangle is hidden as expected");
   }
   await onBlur;
 
-  const newValue = await executeInContent("Test:GetRulePropertyValue", {
-    styleSheetIndex: 0,
-    ruleIndex: 0,
-    name: "border-color"
-  });
+  const newValue = await getRulePropertyValue(0, 0, "border-color");
   is(newValue, "red", "border-color should have been set.");
 
   ruleView.styleDocument.activeElement.blur();
-  await addProperty(ruleView, 1, "color", "red", ";");
+  await addProperty(ruleView, 1, "color", "red", { commitValueWith: ";" });
 
   const props = ruleView.element.querySelectorAll(".ruleview-property");
   for (let i = 0; i < props.length; i++) {
-    is(props[i].hasAttribute("dirty"), i <= 1,
-      "props[" + i + "] marked dirty as appropriate");
+    is(
+      props[i].hasAttribute("dirty"),
+      i <= 1,
+      "props[" + i + "] marked dirty as appropriate"
+    );
   }
 }
 
 async function testDisableProperty(inspector, ruleView) {
-  const idRule = getRuleViewRuleEditor(ruleView, 1).rule;
-  const prop = idRule.textProps[0];
+  const prop = getTextProperty(ruleView, 1, {
+    "border-color": "red",
+    color: "red",
+  });
 
   info("Disabling a property");
   await togglePropStatus(ruleView, prop);
 
-  let newValue = await executeInContent("Test:GetRulePropertyValue", {
-    styleSheetIndex: 0,
-    ruleIndex: 0,
-    name: "border-color"
-  });
+  let newValue = await getRulePropertyValue(0, 0, "border-color");
   is(newValue, "", "Border-color should have been unset.");
 
   info("Enabling the property again");
   await togglePropStatus(ruleView, prop);
 
-  newValue = await executeInContent("Test:GetRulePropertyValue", {
-    styleSheetIndex: 0,
-    ruleIndex: 0,
-    name: "border-color"
-  });
+  newValue = await getRulePropertyValue(0, 0, "border-color");
   is(newValue, "red", "Border-color should have been reset.");
 }
 
@@ -127,7 +130,10 @@ async function testPropertyStillMarkedDirty(inspector, ruleView) {
 
   const props = ruleView.element.querySelectorAll(".ruleview-property");
   for (let i = 0; i < props.length; i++) {
-    is(props[i].hasAttribute("dirty"), i <= 1,
-      "props[" + i + "] marked dirty as appropriate");
+    is(
+      props[i].hasAttribute("dirty"),
+      i <= 1,
+      "props[" + i + "] marked dirty as appropriate"
+    );
   }
 }
