@@ -12,6 +12,7 @@
 const flags = require("resource://devtools/shared/flags.js");
 const {
   VIEW_NODE_CSS_QUERY_CONTAINER,
+  VIEW_NODE_CSS_SELECTOR_WARNINGS,
   VIEW_NODE_FONT_TYPE,
   VIEW_NODE_IMAGE_URL_TYPE,
   VIEW_NODE_INACTIVE_CSS,
@@ -21,7 +22,7 @@ const {
 
 loader.lazyRequireGetter(
   this,
-  "getColor",
+  "getCssVariableColor",
   "resource://devtools/client/shared/theme.js",
   true
 );
@@ -61,20 +62,24 @@ loader.lazyRequireGetter(
   "resource://devtools/client/shared/widgets/tooltip/css-query-container-tooltip-helper.js",
   false
 );
+loader.lazyRequireGetter(
+  this,
+  "CssSelectorWarningsTooltipHelper",
+  "resource://devtools/client/shared/widgets/tooltip/css-selector-warnings-tooltip-helper.js",
+  false
+);
 
 const PREF_IMAGE_TOOLTIP_SIZE = "devtools.inspector.imagePreviewTooltipSize";
 
 // Types of existing tooltips
 const TOOLTIP_CSS_COMPATIBILITY = "css-compatibility";
 const TOOLTIP_CSS_QUERY_CONTAINER = "css-query-info";
+const TOOLTIP_CSS_SELECTOR_WARNINGS = "css-selector-warnings";
 const TOOLTIP_FONTFAMILY_TYPE = "font-family";
 const TOOLTIP_IMAGE_TYPE = "image";
 const TOOLTIP_INACTIVE_CSS = "inactive-css";
 const TOOLTIP_VARIABLE_TYPE = "variable";
 
-// Telemetry
-const TOOLTIP_SHOWN_SCALAR = "devtools.tooltip.shown";
-
 /**
  * Manages all tooltips in the style-inspector.
  *
@@ -115,6 +120,8 @@ TooltipsOverlay.prototype = {
     this.inactiveCssTooltipHelper = new InactiveCssTooltipHelper();
     this.compatibilityTooltipHelper = new CssCompatibilityTooltipHelper();
     this.cssQueryContainerTooltipHelper = new CssQueryContainerTooltipHelper();
+    this.cssSelectorWarningsTooltipHelper =
+      new CssSelectorWarningsTooltipHelper();
 
     // Instantiate the interactiveTooltip and preview tooltip when the
     // rule/computed view is hovered over in order to call
@@ -258,9 +265,25 @@ TooltipsOverlay.prototype = {
       tooltipType = TOOLTIP_CSS_QUERY_CONTAINER;
     }
 
+    // Selector warnings info tooltip
+    if (type === VIEW_NODE_CSS_SELECTOR_WARNINGS) {
+      tooltipType = TOOLTIP_CSS_SELECTOR_WARNINGS;
+    }
+
     return tooltipType;
   },
 
+  _removePreviousInstances() {
+    for (const tooltip of this._instances.values()) {
+      if (tooltip.isVisible()) {
+        if (tooltip.revert) {
+          tooltip.revert();
+        }
+        tooltip.hide();
+      }
+    }
+  },
+
   /**
    * Executed by the tooltip when the pointer hovers over an element of the
    * view. Used to decide whether the tooltip should be shown or not and to
@@ -283,12 +306,7 @@ TooltipsOverlay.prototype = {
       return false;
     }
 
-    for (const [, tooltip] of this._instances) {
-      if (tooltip.isVisible()) {
-        tooltip.revert();
-        tooltip.hide();
-      }
-    }
+    this._removePreviousInstances();
 
     const inspector = this.view.inspector;
 
@@ -326,8 +344,24 @@ TooltipsOverlay.prototype = {
       type === TOOLTIP_VARIABLE_TYPE &&
       nodeInfo.value.value.startsWith("--")
     ) {
-      const variable = nodeInfo.value.variable;
-      await this._setVariablePreviewTooltip(variable);
+      const {
+        variable,
+        registeredProperty,
+        startingStyleVariable,
+        variableComputed,
+        outputParserOptions,
+        cssProperties,
+        value,
+      } = nodeInfo.value;
+      await this._setVariablePreviewTooltip({
+        topSectionText: variable,
+        computed: variableComputed,
+        registeredProperty,
+        startingStyle: startingStyleVariable,
+        outputParserOptions,
+        cssProperties,
+        variableName: value,
+      });
 
       this.sendOpenScalarToTelemetry(type);
 
@@ -350,9 +384,8 @@ TooltipsOverlay.prototype = {
    */
   async onInteractiveTooltipTargetHover(target) {
     if (target.classList.contains("ruleview-compatibility-warning")) {
-      const nodeCompatibilityInfo = await this.view.getNodeCompatibilityInfo(
-        target
-      );
+      const nodeCompatibilityInfo =
+        await this.view.getNodeCompatibilityInfo(target);
 
       await this.compatibilityTooltipHelper.setContent(
         nodeCompatibilityInfo,
@@ -375,19 +408,11 @@ TooltipsOverlay.prototype = {
       return false;
     }
 
-    // Remove previous tooltip instances.
-    for (const [, tooltip] of this._instances) {
-      if (tooltip.isVisible()) {
-        if (tooltip.revert) {
-          tooltip.revert();
-        }
-        tooltip.hide();
-      }
-    }
+    this._removePreviousInstances();
 
     if (type === TOOLTIP_INACTIVE_CSS) {
       // Ensure this is the correct node and not a parent.
-      if (!target.classList.contains("ruleview-unused-warning")) {
+      if (!target.classList.contains("ruleview-inactive-css-warning")) {
         return false;
       }
 
@@ -417,6 +442,17 @@ TooltipsOverlay.prototype = {
       return true;
     }
 
+    if (type === TOOLTIP_CSS_SELECTOR_WARNINGS) {
+      await this.cssSelectorWarningsTooltipHelper.setContent(
+        nodeInfo.value,
+        this.getTooltip("interactiveTooltip")
+      );
+
+      this.sendOpenScalarToTelemetry(type);
+
+      return true;
+    }
+
     return false;
   },
 
@@ -427,7 +463,7 @@ TooltipsOverlay.prototype = {
    *        The node type from `devtools/client/inspector/shared/node-types` or the Tooltip type.
    */
   sendOpenScalarToTelemetry(type) {
-    this.view.inspector.telemetry.keyedScalarAdd(TOOLTIP_SHOWN_SCALAR, type, 1);
+    Glean.devtoolsTooltip.shown[type].add(1);
   },
 
   /**
@@ -490,7 +526,10 @@ TooltipsOverlay.prototype = {
     font = font.replace("!important", "");
     font = font.trim();
 
-    const fillStyle = getColor("body-color");
+    const fillStyle = getCssVariableColor(
+      "--theme-body-color",
+      this.view.inspector.panelWin
+    );
     const { data, size: maxDim } = await nodeFront.getFontFamilyDataURL(
       font,
       fillStyle
@@ -515,13 +554,17 @@ TooltipsOverlay.prototype = {
   /**
    * Set the content of the preview tooltip to display a variable preview.
    *
-   * @param {String} text
-   *        The text to display for the variable tooltip
+   * @param {Object} tooltipParams
+   *        See VariableTooltipHelper#setVariableTooltip `params`.
    * @return {Promise} A promise that resolves when the preview tooltip content is ready
    */
-  async _setVariablePreviewTooltip(text) {
+  async _setVariablePreviewTooltip(tooltipParams) {
     const doc = this.view.inspector.panelDoc;
-    await setVariableTooltip(this.getTooltip("previewTooltip"), doc, text);
+    await setVariableTooltip(
+      this.getTooltip("previewTooltip"),
+      doc,
+      tooltipParams
+    );
   },
 
   _onNewSelection() {
