@@ -3,22 +3,32 @@
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const { immutableUpdate, reportException, assert } = require("devtools/shared/DevToolsUtils");
-const { snapshotState: states, actions} = require("../constants");
-const { L10N, openFilePicker, createSnapshot } = require("../utils");
-const { OS } = require("resource://gre/modules/osfile.jsm");
+const {
+  immutableUpdate,
+  reportException,
+  assert,
+} = require("resource://devtools/shared/DevToolsUtils.js");
+const {
+  snapshotState: states,
+  actions,
+} = require("resource://devtools/client/memory/constants.js");
+const {
+  L10N,
+  openFilePicker,
+  createSnapshot,
+} = require("resource://devtools/client/memory/utils.js");
 const {
   selectSnapshot,
   computeSnapshotData,
   readSnapshot,
-} = require("./snapshot");
+} = require("resource://devtools/client/memory/actions/snapshot.js");
 const VALID_EXPORT_STATES = [states.SAVED, states.READ];
 
-exports.pickFileAndExportSnapshot = function(snapshot) {
-  return async function(dispatch, getState) {
+exports.pickFileAndExportSnapshot = function (snapshot) {
+  return async function ({ dispatch }) {
     const outputFile = await openFilePicker({
       title: L10N.getFormatStr("snapshot.io.save.window"),
-      defaultName: OS.Path.basename(snapshot.path),
+      defaultName: PathUtils.filename(snapshot.path),
       filters: [[L10N.getFormatStr("snapshot.io.filter"), "*.fxsnapshot"]],
       mode: "save",
     });
@@ -31,15 +41,17 @@ exports.pickFileAndExportSnapshot = function(snapshot) {
   };
 };
 
-const exportSnapshot = exports.exportSnapshot = function(snapshot, dest) {
-  return async function(dispatch, getState) {
+const exportSnapshot = (exports.exportSnapshot = function (snapshot, dest) {
+  return async function ({ dispatch }) {
     dispatch({ type: actions.EXPORT_SNAPSHOT_START, snapshot });
 
-    assert(VALID_EXPORT_STATES.includes(snapshot.state),
-      `Snapshot is in invalid state for exporting: ${snapshot.state}`);
+    assert(
+      VALID_EXPORT_STATES.includes(snapshot.state),
+      `Snapshot is in invalid state for exporting: ${snapshot.state}`
+    );
 
     try {
-      await OS.File.copy(snapshot.path, dest);
+      await IOUtils.copy(snapshot.path, dest);
     } catch (error) {
       reportException("exportSnapshot", error);
       dispatch({ type: actions.EXPORT_SNAPSHOT_ERROR, snapshot, error });
@@ -47,10 +59,10 @@ const exportSnapshot = exports.exportSnapshot = function(snapshot, dest) {
 
     dispatch({ type: actions.EXPORT_SNAPSHOT_END, snapshot });
   };
-};
+});
 
-exports.pickFileAndImportSnapshotAndCensus = function(heapWorker) {
-  return async function(dispatch, getState) {
+exports.pickFileAndImportSnapshotAndCensus = function (heapWorker) {
+  return async function ({ dispatch }) {
     const input = await openFilePicker({
       title: L10N.getFormatStr("snapshot.io.import.window"),
       filters: [[L10N.getFormatStr("snapshot.io.filter"), "*.fxsnapshot"]],
@@ -65,8 +77,8 @@ exports.pickFileAndImportSnapshotAndCensus = function(heapWorker) {
   };
 };
 
-const importSnapshotAndCensus = function(heapWorker, path) {
-  return async function(dispatch, getState) {
+const importSnapshotAndCensus = function (heapWorker, path) {
+  return async function ({ dispatch, getState }) {
     const snapshot = immutableUpdate(createSnapshot(getState()), {
       path,
       state: states.IMPORTING,
