@@ -21,7 +21,7 @@ Services.prefs.setBoolPref("devtools.memory.enabled", true);
 /**
  * Open the memory panel for the given tab.
  */
-this.openMemoryPanel = async function(tab) {
+this.openMemoryPanel = async function (tab) {
   info("Opening memory panel.");
   const toolbox = await gDevTools.showToolboxForTab(tab, { toolId: "memory" });
   info("Memory panel shown successfully.");
@@ -32,9 +32,9 @@ this.openMemoryPanel = async function(tab) {
 /**
  * Close the memory panel for the given tab.
  */
-this.closeMemoryPanel = async function(tab) {
+this.closeMemoryPanel = async function (tab) {
   info("Closing memory panel.");
-  const toolbox = await gDevTools.getToolboxForTab(tab);
+  const toolbox = gDevTools.getToolboxForTab(tab);
   await toolbox.destroy();
   info("Closed memory panel successfully.");
 };
@@ -51,7 +51,7 @@ this.closeMemoryPanel = async function(tab) {
  *     });
  */
 function makeMemoryTest(url, generator) {
-  return async function() {
+  return async function () {
     waitForExplicitFinish();
 
     // It can take a long time to save a snapshot to disk, read the snapshots
@@ -237,7 +237,7 @@ function createRAFMock() {
   let queuedFns = [];
   const mock = { timesCalled: 0 };
 
-  mock.nextFrame = function() {
+  mock.nextFrame = function () {
     const thisQueue = queuedFns;
     queuedFns = [];
     for (let i = 0; i < thisQueue.length; i++) {
@@ -245,7 +245,7 @@ function createRAFMock() {
     }
   };
 
-  mock.raf = function(fn) {
+  mock.raf = function (fn) {
     mock.timesCalled++;
     queuedFns.push(fn);
   };
