@@ -4,37 +4,44 @@
 "use strict";
 
 /**
- * Tests if focus modifiers work for the SideMenuWidget.
+ * Tests if focus modifiers work for the Side Menu.
  */
 
 add_task(async function () {
-  let { tab, monitor } = await initNetMonitor(CUSTOM_GET_URL);
+  const { tab, monitor } = await initNetMonitor(CUSTOM_GET_URL, {
+    requestCount: 1,
+  });
   info("Starting test... ");
 
   // It seems that this test may be slow on Ubuntu builds running on ec2.
   requestLongerTimeout(2);
 
-  let { document, store, windowRequire } = monitor.panelWin;
-  let Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
+  const { document, store, windowRequire } = monitor.panelWin;
+  const Actions = windowRequire("devtools/client/netmonitor/src/actions/index");
 
   store.dispatch(Actions.batchEnable(false));
 
   let count = 0;
   function check(selectedIndex, panelVisibility) {
-    info("Performing check " + (count++) + ".");
+    info("Performing check " + count++ + ".");
 
-    let requestItems = Array.from(document.querySelectorAll(".request-list-item"));
-    is(requestItems.findIndex((item) => item.matches(".selected")), selectedIndex,
-      "The selected item in the requests menu was incorrect.");
-    is(!!document.querySelector(".network-details-panel"), panelVisibility,
-      "The network details panel should render correctly.");
+    const requestItems = Array.from(
+      document.querySelectorAll(".request-list-item")
+    );
+    is(
+      requestItems.findIndex(item => item.matches(".selected")),
+      selectedIndex,
+      "The selected item in the requests menu was incorrect."
+    );
+    is(
+      !!document.querySelector(".network-details-bar"),
+      panelVisibility,
+      "The network details panel should render correctly."
+    );
   }
 
-  let wait = waitForNetworkEvents(monitor, 2);
-  await ContentTask.spawn(tab.linkedBrowser, {}, async function () {
-    content.wrappedJSObject.performRequests(2);
-  });
-  await wait;
+  // Execute requests.
+  await performRequests(monitor, tab, 2);
 
   check(-1, false);
 
@@ -53,11 +60,8 @@ add_task(async function () {
   store.dispatch(Actions.selectDelta(-10));
   check(0, true);
 
-  wait = waitForNetworkEvents(monitor, 18);
-  await ContentTask.spawn(tab.linkedBrowser, {}, async function () {
-    content.wrappedJSObject.performRequests(18);
-  });
-  await wait;
+  // Execute requests.
+  await performRequests(monitor, tab, 18);
 
   store.dispatch(Actions.selectDelta(+Infinity));
   check(19, true);
