@@ -7,6 +7,7 @@
  * Tests that the request for a domain that is not found shows
  * correctly.
  */
+
 add_task(async function () {
   const URL = "https://not-existed.com/";
   const { monitor } = await initNetMonitor(URL, {
@@ -31,10 +32,7 @@ add_task(async function () {
     const value = firstItem.querySelector(
       ".requests-list-transferred"
     ).innerText;
-    if (value == "") {
-      return false;
-    }
-    return value;
+    return value.includes("NS_ERROR") ? value : false;
   });
 
   is(
