@@ -8,7 +8,7 @@
  * cloned request retains the same cause type.
  */
 
-add_task(async function() {
+add_task(async function () {
   if (
     Services.prefs.getBoolPref(
       "devtools.netmonitor.features.newEditAndResend",
@@ -60,8 +60,9 @@ add_task(async function() {
   const cloned = getSelectedRequest(store.getState());
 
   // Compares if the requests have the same cause type (XHR)
-  ok(
-    original.cause.type === cloned.cause.type,
+  Assert.strictEqual(
+    original.cause.type,
+    cloned.cause.type,
     "Both requests retain the same cause type"
   );
 
@@ -73,7 +74,7 @@ add_task(async function() {
  * new request retains the same cause type.
  */
 
-add_task(async function() {
+add_task(async function () {
   if (
     Services.prefs.getBoolPref(
       "devtools.netmonitor.features.newEditAndResend",
@@ -125,11 +126,95 @@ add_task(async function() {
     EventUtils.sendMouseEvent({ type: "mousedown" }, newRequest);
     const request = getSelectedRequest(store.getState());
 
-    ok(
-      original.cause.type === request.cause.type,
+    Assert.strictEqual(
+      original.cause.type,
+      request.cause.type,
       "Both requests retain the same cause type"
     );
 
     await teardown(monitor);
   }
 });
+
+/**
+ * Tests that resending a XHR request uses the same security deatils as the
+ * original request.
+ */
+
+add_task(async function () {
+  if (
+    Services.prefs.getBoolPref(
+      "devtools.netmonitor.features.newEditAndResend",
+      true
+    )
+  ) {
+    const { tab, monitor } = await initNetMonitor(HTTPS_CORS_URL, {
+      requestCount: 1,
+    });
+
+    const { document, store, windowRequire, connector } = monitor.panelWin;
+    const Actions = windowRequire(
+      "devtools/client/netmonitor/src/actions/index"
+    );
+    store.dispatch(Actions.batchEnable(false));
+
+    info("Performing a CORS request");
+    const requestUrl = "https://example.com" + CORS_SJS_PATH;
+
+    const wait = waitForNetworkEvents(monitor, 1);
+    await SpecialPowers.spawn(
+      tab.linkedBrowser,
+      [requestUrl],
+      async function (url) {
+        content.wrappedJSObject.performRequests(url);
+      }
+    );
+
+    info("Waiting until the requests appear in netmonitor");
+    await wait;
+
+    const { getSelectedRequest } = windowRequire(
+      "devtools/client/netmonitor/src/selectors/index"
+    );
+
+    info("Select XHR request");
+    const xhrRequest = document.querySelectorAll(".request-list-item")[0];
+    EventUtils.sendMouseEvent({ type: "mousedown" }, xhrRequest);
+
+    info("Fetch the Headers for the original XHR request");
+    let originalRequest = getSelectedRequest(store.getState());
+    await connector.requestData(originalRequest.id, "requestHeaders");
+    await waitForRequestData(store, ["requestHeaders"]);
+    originalRequest = getSelectedRequest(store.getState());
+
+    info("Resend the XHR request");
+    const waitForResentRequest = waitForNetworkEvents(monitor, 1);
+    EventUtils.sendMouseEvent({ type: "contextmenu" }, xhrRequest);
+    await selectContextMenuItem(monitor, "request-list-context-resend-only");
+    await waitForResentRequest;
+
+    info("Fetch the Headers for the resent XHR request");
+    let resentRequest = getSelectedRequest(store.getState());
+    await connector.requestData(resentRequest.id, "requestHeaders");
+    await waitForRequestData(store, ["requestHeaders"]);
+    resentRequest = getSelectedRequest(store.getState());
+
+    const originalRequestSecFetchModeHeader =
+      originalRequest.requestHeaders.headers.find(
+        header => header.name == "Sec-Fetch-Mode"
+      );
+    const resentRequestSecFetchModeHeader =
+      resentRequest.requestHeaders.headers.find(
+        header => header.name == "Sec-Fetch-Mode"
+      );
+
+    info("Assert the security mode for the original amd resent request");
+    Assert.strictEqual(
+      originalRequestSecFetchModeHeader.value,
+      resentRequestSecFetchModeHeader.value,
+      "Both requests retain the same security mode"
+    );
+
+    await teardown(monitor);
+  }
+});
