@@ -6,7 +6,7 @@
 /**
  * Tests if the POST requests display the correct information in the UI.
  */
-add_task(async function() {
+add_task(async function () {
   const {
     L10N,
   } = require("resource://devtools/client/netmonitor/src/utils/l10n.js");
@@ -36,14 +36,16 @@ add_task(async function() {
     const requestsListStatus = requestItem.querySelector(".status-code");
     EventUtils.sendMouseEvent({ type: "mouseover" }, requestsListStatus);
     await waitUntil(() => requestsListStatus.title);
+    await waitForDOMIfNeeded(requestItem, ".requests-list-timings-total");
   }
 
-  verifyRequestItemTarget(
+  await verifyRequestItemTarget(
     document,
     getDisplayedRequests(store.getState()),
     getSortedRequests(store.getState())[0],
     "POST",
-    SIMPLE_SJS + "?foo=bar&baz=42&type=urlencoded",
+    SIMPLE_SJS +
+      "?foo=bar&baz=42&valueWithEqualSign=hijk=123=mnop&type=urlencoded",
     {
       status: 200,
       statusText: "Och Aye",
@@ -53,7 +55,7 @@ add_task(async function() {
       time: true,
     }
   );
-  verifyRequestItemTarget(
+  await verifyRequestItemTarget(
     document,
     getDisplayedRequests(store.getState()),
     getSortedRequests(store.getState())[1],
@@ -134,8 +136,8 @@ add_task(async function() {
       checkVisibility("request");
       is(
         labels.length,
-        2,
-        "There should be 2 param values displayed in this tabpanel."
+        4,
+        "There should be 4 param values displayed in this tabpanel."
       );
       is(
         labels[0].textContent,
@@ -157,6 +159,26 @@ add_task(async function() {
         `"123"`,
         "The second post param value was incorrect."
       );
+      is(
+        labels[2].textContent,
+        "valueWithEqualSign",
+        "The third post param name was incorrect."
+      );
+      is(
+        values[2].textContent,
+        `"xyz=abc=123"`,
+        "The third post param value was incorrect."
+      );
+      is(
+        labels[3].textContent,
+        "valueWithAmpersand",
+        "The fourth post param name was incorrect."
+      );
+      is(
+        values[3].textContent,
+        `"abcd&1234"`,
+        "The fourth post param value was incorrect."
+      );
     } else {
       checkVisibility("request editor");
 
