@@ -10,7 +10,7 @@ const TEST_URI =
   "http://example.com/browser/devtools/client/webconsole/" +
   "test/browser/test-eval-error.html";
 
-add_task(async function() {
+add_task(async function () {
   const hud = await openNewTabAndConsole(TEST_URI);
 
   execute(hud, "throwErrorObject()");
@@ -53,16 +53,7 @@ add_task(async function() {
   ok(true, "The object was rendered in an ObjectInspector");
 
   info("Expanding the object");
-  const onOiExpanded = waitFor(() => {
-    return oi.querySelectorAll(".node").length === 3;
-  });
-  oi.querySelector(".arrow").click();
-  await onOiExpanded;
-
-  ok(
-    oi.querySelector(".arrow").classList.contains("expanded"),
-    "Object expanded"
-  );
+  await expandObjectInspectorNode(oi.querySelector(".tree-node"));
 
   // The object inspector now looks like:
   // {...}
