@@ -7,7 +7,7 @@
 "use strict";
 
 const {
-  logEvent,
+  evalAndLogEvent,
   getThrownMessage,
 } = require("resource://devtools/server/actors/utils/logEvent.js");
 
@@ -90,7 +90,7 @@ class BreakpointActor {
   /**
    * Called on changes to this breakpoint's script offsets or options.
    */
-  _newOffsetsOrOptions(script, offsets, oldOptions) {
+  _newOffsetsOrOptions(script, offsets) {
     // Clear any existing handler first in case this is called multiple times
     // after options change.
     for (const offset of offsets) {
@@ -209,11 +209,12 @@ class BreakpointActor {
     }
 
     if (logValue) {
-      return logEvent({
+      return evalAndLogEvent({
         threadActor: this.threadActor,
         frame,
         level: "logPoint",
         expression: `[${logValue}]`,
+        showStacktrace: this.options.showStacktrace,
       });
     }
 
