@@ -75,6 +75,12 @@ class AutoRefreshHighlighter extends EventEmitter {
 
     this.highlighterEnv = highlighterEnv;
 
+    this._updateSimpleHighlighters = this._updateSimpleHighlighters.bind(this);
+    this.highlighterEnv.on(
+      "use-simple-highlighters-updated",
+      this._updateSimpleHighlighters
+    );
+
     this.currentNode = null;
     this.currentQuads = {};
 
@@ -102,6 +108,10 @@ class AutoRefreshHighlighter extends EventEmitter {
     return this.win;
   }
 
+  get supportsSimpleHighlighters() {
+    return false;
+  }
+
   /**
    * Show the highlighter on a given node
    * @param {DOMNode} node
@@ -120,6 +130,14 @@ class AutoRefreshHighlighter extends EventEmitter {
 
     this._stopRefreshLoop();
     this.currentNode = node;
+
+    // For offset-path, the highlighter needs to be computed from the containing block
+    // of the node, not the node itself.
+    this.useContainingBlock = this.options.mode === "cssOffsetPath";
+    this.drawingNode = this.useContainingBlock
+      ? InspectorUtils.containingBlockOf(this.currentNode)
+      : this.currentNode;
+
     this._updateAdjustedQuads();
     this._startRefreshLoop();
 
@@ -188,10 +206,19 @@ class AutoRefreshHighlighter extends EventEmitter {
   _updateAdjustedQuads() {
     this.currentQuads = {};
 
+    // If we need to use the containing block, and if it is the <html> element,
+    // we need to use the viewport quads.
+    const useViewport =
+      this.useContainingBlock &&
+      this.drawingNode === this.currentNode.ownerDocument.documentElement;
+    const node = useViewport
+      ? this.drawingNode.ownerDocument
+      : this.drawingNode;
+
     for (const region of BOX_MODEL_REGIONS) {
       this.currentQuads[region] = getAdjustedQuads(
         this.contentWindow,
-        this.currentNode,
+        node,
         region,
         { ignoreScroll: this._ignoreScroll, ignoreZoom: this._ignoreZoom }
       );
@@ -307,9 +334,31 @@ class AutoRefreshHighlighter extends EventEmitter {
     this.rafID = this.rafWin = null;
   }
 
+  _updateSimpleHighlighters() {
+    if (!this.supportsSimpleHighlighters) {
+      return;
+    }
+
+    if (!this.rootEl) {
+      // Highlighters which support simple highlighters are expected to use a root element.
+      return;
+    }
+
+    // Add/remove the `user-simple-highlighters` class based on the current
+    // toolbox configuration.
+    this.rootEl.classList.toggle(
+      "use-simple-highlighters",
+      this.highlighterEnv.useSimpleHighlightersForReducedMotion
+    );
+  }
+
   destroy() {
     this.hide();
 
+    this.highlighterEnv.off(
+      "use-simple-highlighters-updated",
+      this._updateSimpleHighlighters
+    );
     this.highlighterEnv = null;
     this.currentNode = null;
   }
