@@ -17,6 +17,12 @@ const Debugger = require("Debugger");
 const {
   EXCLUDED_LISTENER,
 } = require("resource://devtools/server/actors/inspector/constants.js");
+loader.lazyRequireGetter(
+  this,
+  "isUserDefinedEventName",
+  "resource://devtools/server/actors/events/events.js",
+  true
+);
 
 // eslint-disable-next-line
 const JQUERY_LIVE_REGEX =
@@ -923,6 +929,7 @@ class EventCollector {
    *             enabled: true
    *             sourceActor: "sourceActor.1234",
    *             nsIEventListenerInfo: nsIEventListenerInfo {…},
+   *             isUserDefined: false,
    *           }
    */
   // eslint-disable-next-line complexity
@@ -948,7 +955,7 @@ class EventCollector {
       const hide = listener.hide || {};
       const override = listener.override || {};
       const tags = listener.tags || "";
-      const type = listener.type || "";
+      const type = override.type || listener.type || "";
       const enabled = !!listener.enabled;
       let functionSource = handler.toString();
       let line = 0;
@@ -1051,7 +1058,7 @@ class EventCollector {
       }
 
       eventObj = {
-        type: override.type || type,
+        type,
         handler: override.handler || functionSource.trim(),
         origin: override.origin || origin,
         tags: override.tags || tags,
@@ -1064,6 +1071,7 @@ class EventCollector {
         sourceActor,
         nsIEventListenerInfo: listener.nsIEventListenerInfo,
         enabled,
+        isUserDefined: isUserDefinedEventName(type),
       };
 
       // Hide the debugger icon for DOM0 and native listeners. DOM0 listeners are
