@@ -16,7 +16,6 @@ const {
 const {
   getDebuggerSourceURL,
 } = require("resource://devtools/server/actors/utils/source-url.js");
-
 loader.lazyRequireGetter(
   this,
   "ArrayBufferActor",
@@ -36,22 +35,31 @@ loader.lazyRequireGetter(
   "resource://devtools/shared/DevToolsUtils.js"
 );
 
+ChromeUtils.defineESModuleGetters(
+  this,
+  {
+    ExtensionUtils: "resource://gre/modules/ExtensionUtils.sys.mjs",
+  },
+  { global: "contextual" }
+);
+
 const windowsDrive = /^([a-zA-Z]:)/;
 
 function resolveSourceURL(sourceURL, targetActor) {
   if (sourceURL) {
-    try {
-      let baseURL;
-      if (targetActor.window) {
-        baseURL = targetActor.window.location?.href;
-      }
-      // For worker, we don't have easy access to location,
-      // so pull extra information directly from the target actor.
-      if (targetActor.workerUrl) {
-        baseURL = targetActor.workerUrl;
-      }
-      return new URL(sourceURL, baseURL || undefined).href;
-    } catch (err) {}
+    let baseURL;
+    if (targetActor.window) {
+      baseURL = targetActor.window.location?.href;
+    }
+    // For worker, we don't have easy access to location,
+    // so pull extra information directly from the target actor.
+    if (targetActor.workerUrl) {
+      baseURL = targetActor.workerUrl;
+    }
+    const parsedURL = URL.parse(sourceURL, baseURL);
+    if (parsedURL) {
+      return parsedURL.href;
+    }
   }
 
   return null;
@@ -128,7 +136,7 @@ class SourceActor extends Actor {
       // because we can't easily fetch the full html content of the srcdoc attribute.
       this.__isInlineSource =
         source.introductionType === "inlineScript" &&
-        !resolveSourceURL(source.displayURL, this.threadActor._parent) &&
+        !resolveSourceURL(source.displayURL, this.threadActor.targetActor) &&
         !this.url.startsWith("about:srcdoc");
     }
     return this.__isInlineSource;
@@ -148,7 +156,7 @@ class SourceActor extends Actor {
   }
   get url() {
     if (this._url === undefined) {
-      this._url = getSourceURL(this._source, this.threadActor._parent);
+      this._url = getSourceURL(this._source, this.threadActor.targetActor);
     }
     return this._url;
   }
@@ -159,7 +167,7 @@ class SourceActor extends Actor {
 
       // Cu is not available for workers and so we are not able to get a
       // WebExtensionPolicy object
-      if (!isWorker && this.url?.startsWith("moz-extension:")) {
+      if (!isWorker && ExtensionUtils.isExtensionUrl(this.url)) {
         try {
           const extURI = Services.io.newURI(this.url);
           const policy = WebExtensionPolicy.getByURI(extURI);
@@ -193,6 +201,11 @@ class SourceActor extends Actor {
       introductionType = "scriptElement";
     }
 
+    // NOTE: Debugger.Source.prototype.startColumn is 1-based.
+    //       Convert to 0-based, while keeping the wasm's column (1) as is.
+    //       (bug 1863878)
+    const columnBase = source.introductionType === "wasm" ? 0 : 1;
+
     return {
       actor: this.actorID,
       extensionName: this.extensionName,
@@ -200,13 +213,13 @@ class SourceActor extends Actor {
       isBlackBoxed: this.sourcesManager.isBlackBoxed(this.url),
       sourceMapBaseURL: getSourcemapBaseURL(
         this.url,
-        this.threadActor._parent.window
+        this.threadActor.targetActor.window
       ),
       sourceMapURL: source.sourceMapURL,
       introductionType,
       isInlineSource: this._isInlineSource,
       sourceStartLine: source.startLine,
-      sourceStartColumn: source.startColumn,
+      sourceStartColumn: source.startColumn - columnBase,
       sourceLength: source.text?.length,
     };
   }
@@ -411,10 +424,15 @@ class SourceActor extends Actor {
         return false;
       }
 
+      // NOTE: Debugger.Script.prototype.startColumn is 1-based.
+      //       Convert to 0-based, while keeping the wasm's column (1) as is.
+      //       (bug 1863878)
+      const columnBase = script.format === "wasm" ? 0 : 1;
       if (
         script.startLine > endLine ||
         script.startLine + lineCount <= startLine ||
-        (script.startLine == endLine && script.startColumn > endColumn)
+        (script.startLine == endLine &&
+          script.startColumn - columnBase > endColumn)
       ) {
         return false;
       }
@@ -422,7 +440,7 @@ class SourceActor extends Actor {
       if (
         lineCount == 1 &&
         script.startLine == startLine &&
-        script.startColumn + script.sourceLength <= startColumn
+        script.startColumn - columnBase + script.sourceLength <= startColumn
       ) {
         return false;
       }
@@ -469,20 +487,25 @@ class SourceActor extends Actor {
       end: { line: endLine = Infinity, column: endColumn = Infinity } = {},
     } = query || {};
 
+    // NOTE: Debugger.Script.prototype.startColumn is 1-based.
+    //       Convert to 0-based, while keeping the wasm's column (1) as is.
+    //       (bug 1863878)
+    const columnBase = script.format === "wasm" ? 0 : 1;
+
     const offsets = script.getPossibleBreakpoints();
     for (const { lineNumber, columnNumber } of offsets) {
       if (
         lineNumber < startLine ||
-        (lineNumber === startLine && columnNumber < startColumn) ||
+        (lineNumber === startLine && columnNumber - columnBase < startColumn) ||
         lineNumber > endLine ||
-        (lineNumber === endLine && columnNumber >= endColumn)
+        (lineNumber === endLine && columnNumber - columnBase >= endColumn)
       ) {
         continue;
       }
 
       positions.push({
         line: lineNumber,
-        column: columnNumber,
+        column: columnNumber - columnBase,
       });
     }
   }
@@ -610,6 +633,11 @@ class SourceActor extends Actor {
         script => !actor.hasScript(script)
       );
 
+      // NOTE: Debugger.Script.prototype.getPossibleBreakpoints returns
+      //       columnNumber in 1-based.
+      //       The following code uses columnNumber only for comparing against
+      //       other columnNumber, and we don't need to convert to 0-based.
+
       // This is a line breakpoint, so we add a breakpoint on the first
       // breakpoint on the line.
       const lineMatches = [];
@@ -644,13 +672,19 @@ class SourceActor extends Actor {
       );
 
       for (const script of scripts) {
+        // NOTE: getPossibleBreakpoints's minColumn/maxColumn parameters are
+        //       1-based.
+        //       Convert to 1-based, while keeping the wasm's column (1) as is.
+        //       (bug 1863878)
+        const columnBase = script.format === "wasm" ? 0 : 1;
+
         // Check to see if the script contains a breakpoint position at
         // this line and column.
         const possibleBreakpoint = script
           .getPossibleBreakpoints({
             line,
-            minColumn: column,
-            maxColumn: column + 1,
+            minColumn: column + columnBase,
+            maxColumn: column + columnBase + 1,
           })
           .pop();
 
