@@ -10,9 +10,9 @@
 const Resources = require("resource://devtools/server/actors/resources/index.js");
 
 add_task(
-  threadFrontTest(async ({ threadActor, threadFront, debuggee, client }) => {
+  threadFrontTest(async ({ threadActor, threadFront, debuggee }) => {
     let lastMessage, lastExpression;
-    const targetActor = threadActor._parent;
+    const { targetActor } = threadActor;
     // Only Workers are evaluating through the WebConsoleActor.
     // Tabs will be evaluating directly via the frame object.
     targetActor._consoleActor = {
@@ -24,11 +24,14 @@ add_task(
     // And then listen for resource RDP event.
     // Bug 1646677: But we should probably migrate this test to ResourceCommand so that
     // we don't have to hack the server side via Resource.watchResources call.
-    targetActor.on("resource-available-form", resources => {
-      if (resources[0].resourceType == Resources.TYPES.CONSOLE_MESSAGE) {
-        lastMessage = resources[0].message;
+    targetActor.on(
+      "resources-available-array",
+      ([[resourceType, resources]]) => {
+        if (resourceType == Resources.TYPES.CONSOLE_MESSAGE) {
+          lastMessage = resources[0];
+        }
       }
-    });
+    );
 
     // But both tabs and processes will be going through the ConsoleMessages module
     // We force watching for console message first,
@@ -59,6 +62,7 @@ add_task(
     if (lastMessage) {
       Assert.equal(lastMessage.level, "logPointError");
       Assert.equal(lastMessage.arguments[0], "c is not defined");
+      Assert.ok(/\d+\.\d+/.test(lastMessage.timeStamp));
     } else {
       Assert.equal(lastExpression.text, "console.log(...[c])");
       Assert.equal(lastExpression.lineNumber, 3);
@@ -70,7 +74,7 @@ function evalCode(debuggee) {
   /* eslint-disable */
   Cu.evalInSandbox(
     "debugger;\n" + // 1
-    "var a = 'three';\n" + // 2
+      "var a = 'three';\n" + // 2
       "var b = 2;\n", // 3
     debuggee,
     "1.8",
