@@ -31,37 +31,19 @@
  * is then available to scanning devices.
  */
 
-const { Cu, CC, Cc, Ci } = require("chrome");
-const EventEmitter = require("devtools/shared/event-emitter");
-const Services = require("Services");
+const EventEmitter = require("resource://devtools/shared/event-emitter.js");
 
-const UDPSocket = CC("@mozilla.org/network/udp-socket;1",
-                     "nsIUDPSocket",
-                     "init");
+const UDPSocket = Components.Constructor(
+  "@mozilla.org/network/udp-socket;1",
+  "nsIUDPSocket",
+  "init"
+);
 
 const SCAN_PORT = 50624;
 const UPDATE_PORT = 50625;
 const ADDRESS = "224.0.0.115";
 const REPLY_TIMEOUT = 5000;
 
-const { XPCOMUtils } = Cu.import("resource://gre/modules/XPCOMUtils.jsm", {});
-
-XPCOMUtils.defineLazyGetter(this, "converter", () => {
-  let conv = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
-             .createInstance(Ci.nsIScriptableUnicodeConverter);
-  conv.charset = "utf8";
-  return conv;
-});
-
-XPCOMUtils.defineLazyGetter(this, "sysInfo", () => {
-  return Cc["@mozilla.org/system-info;1"].getService(Ci.nsIPropertyBag2);
-});
-
-XPCOMUtils.defineLazyGetter(this, "libcutils", function () {
-  let { libcutils } = Cu.import("resource://gre/modules/systemlibs.js", {});
-  return libcutils;
-});
-
 var logging = Services.prefs.getBoolPref("devtools.discovery.log");
 function log(msg) {
   if (logging) {
@@ -77,8 +59,11 @@ function log(msg) {
 function Transport(port) {
   EventEmitter.decorate(this);
   try {
-    this.socket = new UDPSocket(port, false,
-                                Services.scriptSecurityManager.getSystemPrincipal());
+    this.socket = new UDPSocket(
+      port,
+      false,
+      Services.scriptSecurityManager.getSystemPrincipal()
+    );
     this.socket.joinMulticast(ADDRESS);
     this.socket.asyncListen(this);
   } catch (e) {
@@ -87,7 +72,6 @@ function Transport(port) {
 }
 
 Transport.prototype = {
-
   /**
    * Send a object to some UDP port.
    * @param object object
@@ -95,12 +79,12 @@ Transport.prototype = {
    * @param port integer
    *        UDP port to send the message to
    */
-  send: function (object, port) {
+  send(object, port) {
     if (logging) {
       log("Send to " + port + ":\n" + JSON.stringify(object, null, 2));
     }
-    let message = JSON.stringify(object);
-    let rawMessage = converter.convertToByteArray(message);
+    const message = JSON.stringify(object);
+    const rawMessage = Uint8Array.from(message, x => x.charCodeAt(0));
     try {
       this.socket.send(ADDRESS, port, rawMessage, rawMessage.length);
     } catch (e) {
@@ -108,30 +92,30 @@ Transport.prototype = {
     }
   },
 
-  destroy: function () {
+  destroy() {
     this.socket.close();
   },
 
   // nsIUDPSocketListener
 
-  onPacketReceived: function (socket, message) {
-    let messageData = message.data;
-    let object = JSON.parse(messageData);
+  onPacketReceived(socket, message) {
+    const messageData = message.data;
+    const object = JSON.parse(messageData);
     object.from = message.fromAddr.address;
-    let port = message.fromAddr.port;
+    const port = message.fromAddr.port;
     if (port == this.socket.port) {
       log("Ignoring looped message");
       return;
     }
     if (logging) {
-      log("Recv on " + this.socket.port + ":\n" +
-          JSON.stringify(object, null, 2));
+      log(
+        "Recv on " + this.socket.port + ":\n" + JSON.stringify(object, null, 2)
+      );
     }
     this.emit("message", object);
   },
 
-  onStopListening: function () {}
-
+  onStopListening() {},
 };
 
 /**
@@ -148,8 +132,7 @@ function LocalDevice() {
 LocalDevice.UNKNOWN = "unknown";
 
 LocalDevice.prototype = {
-
-  _get: function () {
+  _get() {
     // Without Settings API, just generate a name and stop, since the value
     // can't be persisted.
     this._generate();
@@ -159,24 +142,13 @@ LocalDevice.prototype = {
    * Generate a new device name from various platform-specific properties.
    * Triggers the |name| setter to persist if needed.
    */
-  _generate: function () {
-    if (Services.appinfo.widgetToolkit == "gonk") {
-      // For Firefox OS devices, create one from the device name plus a little
-      // randomness.  The goal is just to distinguish devices in an office
-      // environment where many people may have the same device model for
-      // testing purposes (which would otherwise all report the same name).
-      let name = libcutils.property_get("ro.product.device");
-      // Pick a random number from [0, 2^32)
-      let randomID = Math.floor(Math.random() * Math.pow(2, 32));
-      // To hex and zero pad
-      randomID = ("00000000" + randomID.toString(16)).slice(-8);
-      this.name = name + "-" + randomID;
-    } else if (Services.appinfo.widgetToolkit == "android") {
+  _generate() {
+    if (Services.appinfo.widgetToolkit == "android") {
       // For Firefox for Android, use the device's model name.
       // TODO: Bug 1180997: Find the right way to expose an editable name
-      this.name = sysInfo.get("device");
+      this.name = Services.sysinfo.get("device");
     } else {
-      this.name = sysInfo.get("host");
+      this.name = Services.dns.myHostName;
     }
   },
 
@@ -187,8 +159,7 @@ LocalDevice.prototype = {
   set name(name) {
     this._name = name;
     log("Device: " + this._name);
-  }
-
+  },
 };
 
 function Discovery() {
@@ -200,14 +171,14 @@ function Discovery() {
   this.replyTimeout = REPLY_TIMEOUT;
 
   // Defaulted to Transport, but can be altered by tests
-  this._factories = { Transport: Transport };
+  this._factories = { Transport };
 
   this._transports = {
     scan: null,
-    update: null
+    update: null,
   };
   this._expectingReplies = {
-    from: new Set()
+    from: new Set(),
   };
 
   this._onRemoteScan = this._onRemoteScan.bind(this);
@@ -216,7 +187,6 @@ function Discovery() {
 }
 
 Discovery.prototype = {
-
   /**
    * Add a new service offered by this device.
    * @param service string
@@ -224,7 +194,7 @@ Discovery.prototype = {
    * @param info object
    *        Arbitrary data about the service to announce to scanning devices
    */
-  addService: function (service, info) {
+  addService(service, info) {
     log("ADDING LOCAL SERVICE");
     if (Object.keys(this.localServices).length === 0) {
       this._startListeningForScan();
@@ -237,7 +207,7 @@ Discovery.prototype = {
    * @param service string
    *        Name of the service
    */
-  removeService: function (service) {
+  removeService(service) {
     delete this.localServices[service];
     if (Object.keys(this.localServices).length === 0) {
       this._stopListeningForScan();
@@ -247,7 +217,7 @@ Discovery.prototype = {
   /**
    * Scan for service updates from other devices.
    */
-  scan: function () {
+  scan() {
     this._startListeningForUpdate();
     this._waitForReplies();
     // TODO Bug 1027457: Use timer to debounce
@@ -255,12 +225,12 @@ Discovery.prototype = {
   },
 
   /**
-   * Get a list of all remote devices currently offering some service.:w
+   * Get a list of all remote devices currently offering some service.
    */
-  getRemoteDevices: function () {
-    let devices = new Set();
-    for (let service in this.remoteServices) {
-      for (let device in this.remoteServices[service]) {
+  getRemoteDevices() {
+    const devices = new Set();
+    for (const service in this.remoteServices) {
+      for (const device in this.remoteServices[service]) {
         devices.add(device);
       }
     }
@@ -270,8 +240,8 @@ Discovery.prototype = {
   /**
    * Get a list of all remote devices currently offering a particular service.
    */
-  getRemoteDevicesWithService: function (service) {
-    let devicesWithService = this.remoteServices[service] || {};
+  getRemoteDevicesWithService(service) {
+    const devicesWithService = this.remoteServices[service] || {};
     return Object.keys(devicesWithService);
   },
 
@@ -279,23 +249,25 @@ Discovery.prototype = {
    * Get service info (any details registered by the remote device) for a given
    * service on a device.
    */
-  getRemoteService: function (service, device) {
-    let devicesWithService = this.remoteServices[service] || {};
+  getRemoteService(service, device) {
+    const devicesWithService = this.remoteServices[service] || {};
     return devicesWithService[device];
   },
 
-  _waitForReplies: function () {
+  _waitForReplies() {
     clearTimeout(this._expectingReplies.timer);
     this._expectingReplies.from = new Set(this.getRemoteDevices());
-    this._expectingReplies.timer =
-      setTimeout(this._purgeMissingDevices, this.replyTimeout);
+    this._expectingReplies.timer = setTimeout(
+      this._purgeMissingDevices,
+      this.replyTimeout
+    );
   },
 
   get Transport() {
     return this._factories.Transport;
   },
 
-  _startListeningForScan: function () {
+  _startListeningForScan() {
     if (this._transports.scan) {
       // Already listening
       return;
@@ -305,7 +277,7 @@ Discovery.prototype = {
     this._transports.scan.on("message", this._onRemoteScan);
   },
 
-  _stopListeningForScan: function () {
+  _stopListeningForScan() {
     if (!this._transports.scan) {
       // Not listening
       return;
@@ -315,7 +287,7 @@ Discovery.prototype = {
     this._transports.scan = null;
   },
 
-  _startListeningForUpdate: function () {
+  _startListeningForUpdate() {
     if (this._transports.update) {
       // Already listening
       return;
@@ -325,7 +297,7 @@ Discovery.prototype = {
     this._transports.update.on("message", this._onRemoteUpdate);
   },
 
-  _stopListeningForUpdate: function () {
+  _stopListeningForUpdate() {
     if (!this._transports.update) {
       // Not listening
       return;
@@ -335,7 +307,7 @@ Discovery.prototype = {
     this._transports.update = null;
   },
 
-  _restartListening: function () {
+  _restartListening() {
     if (this._transports.scan) {
       this._stopListeningForScan();
       this._startListeningForScan();
@@ -360,34 +332,34 @@ Discovery.prototype = {
     return null;
   },
 
-  _sendStatusTo: function (port) {
-    let status = {
+  _sendStatusTo(port) {
+    const status = {
       device: this.device.name,
-      services: this.localServices
+      services: this.localServices,
     };
     this._outgoingTransport.send(status, port);
   },
 
-  _onRemoteScan: function () {
+  _onRemoteScan() {
     // Send my own status in response
     log("GOT SCAN REQUEST");
     this._sendStatusTo(UPDATE_PORT);
   },
 
-  _onRemoteUpdate: function (e, update) {
+  _onRemoteUpdate(update) {
     log("GOT REMOTE UPDATE");
 
-    let remoteDevice = update.device;
-    let remoteHost = update.from;
+    const remoteDevice = update.device;
+    const remoteHost = update.from;
 
     // Record the reply as received so it won't be purged as missing
     this._expectingReplies.from.delete(remoteDevice);
 
     // First, loop over the known services
-    for (let service in this.remoteServices) {
-      let devicesWithService = this.remoteServices[service];
-      let hadServiceForDevice = !!devicesWithService[remoteDevice];
-      let haveServiceForDevice = service in update.services;
+    for (const service in this.remoteServices) {
+      const devicesWithService = this.remoteServices[service];
+      const hadServiceForDevice = !!devicesWithService[remoteDevice];
+      const haveServiceForDevice = service in update.services;
       // If the remote device used to have service, but doesn't any longer, then
       // it was deleted, so we remove it here.
       if (hadServiceForDevice && !haveServiceForDevice) {
@@ -398,18 +370,19 @@ Discovery.prototype = {
     }
 
     // Second, loop over the services in the received update
-    for (let service in update.services) {
+    for (const service in update.services) {
       // Detect if this is a new device for this service
-      let newDevice = !this.remoteServices[service] ||
-                      !this.remoteServices[service][remoteDevice];
+      const newDevice =
+        !this.remoteServices[service] ||
+        !this.remoteServices[service][remoteDevice];
 
       // Look up the service info we may have received previously from the same
       // remote device
-      let devicesWithService = this.remoteServices[service] || {};
-      let oldDeviceInfo = devicesWithService[remoteDevice];
+      const devicesWithService = this.remoteServices[service] || {};
+      const oldDeviceInfo = devicesWithService[remoteDevice];
 
       // Store the service info from the remote device
-      let newDeviceInfo = Cu.cloneInto(update.services[service], {});
+      const newDeviceInfo = Cu.cloneInto(update.services[service], {});
       newDeviceInfo.host = remoteHost;
       devicesWithService[remoteDevice] = newDeviceInfo;
       this.remoteServices[service] = devicesWithService;
@@ -422,19 +395,21 @@ Discovery.prototype = {
 
       // If we've seen this service from the remote device, but the details have
       // changed, announce the update
-      if (!newDevice &&
-          JSON.stringify(oldDeviceInfo) != JSON.stringify(newDeviceInfo)) {
+      if (
+        !newDevice &&
+        JSON.stringify(oldDeviceInfo) != JSON.stringify(newDeviceInfo)
+      ) {
         log("UPDATED " + service + ", DEVICE " + remoteDevice);
         this.emit(service + "-device-updated", remoteDevice, newDeviceInfo);
       }
     }
   },
 
-  _purgeMissingDevices: function () {
+  _purgeMissingDevices() {
     log("PURGING MISSING DEVICES");
-    for (let service in this.remoteServices) {
-      let devicesWithService = this.remoteServices[service];
-      for (let remoteDevice in devicesWithService) {
+    for (const service in this.remoteServices) {
+      const devicesWithService = this.remoteServices[service];
+      for (const remoteDevice in devicesWithService) {
         // If we're still expecting a reply from a remote device when it's time
         // to purge, then the service is removed.
         if (this._expectingReplies.from.has(remoteDevice)) {
@@ -444,8 +419,7 @@ Discovery.prototype = {
         }
       }
     }
-  }
-
+  },
 };
 
 var discovery = new Discovery();
