@@ -4,6 +4,7 @@
 
 "use strict";
 
+const LINE_BREAK_RE = /\r\n?|\n|\u2028|\u2029/;
 const MAX_DATA_URL_LENGTH = 40;
 /**
  * Provide access to the style information in a page.
@@ -258,7 +259,7 @@ function prettifyCSS(text, ruleCount) {
   }
 
   // Stylesheets may start and end with HTML comment tags (possibly with whitespaces
-  // before and after). Remove those first. Don't do anything there aren't any.
+  // before and after). Remove those first. Don't do anything if there aren't any.
   const trimmed = text.trim();
   if (trimmed.startsWith("<!--")) {
     text = trimmed.replace(/^<!--/, "").replace(/-->$/, "").trim();
@@ -332,6 +333,8 @@ function prettifyCSS(text, ruleCount) {
   let anyNonWS;
   // True if the terminating token is "}".
   let isCloseBrace;
+  // True if the terminating token is a new line character.
+  let isNewLine;
   // True if the token just before the terminating token was
   // whitespace.
   let lastWasWS;
@@ -394,7 +397,16 @@ function prettifyCSS(text, ruleCount) {
         break;
       }
 
-      if (token.tokenType !== "WhiteSpace") {
+      if (token.tokenType === "WhiteSpace") {
+        if (LINE_BREAK_RE.test(token.text)) {
+          // If we encounter a new line after a significant token, we can
+          // move on to the next significant token.
+          // This avoids messing with declarations with no semi-colon preceding
+          // a closing brace, eg `{\n  color: red\n  }`
+          isNewLine = true;
+          break;
+        }
+      } else {
         anyNonWS = true;
       }
 
@@ -433,6 +445,7 @@ function prettifyCSS(text, ruleCount) {
     endIndex = undefined;
     anyNonWS = false;
     isCloseBrace = false;
+    isNewLine = false;
     lastWasWS = false;
 
     // Read tokens until we see a reason to insert a newline.
@@ -445,6 +458,9 @@ function prettifyCSS(text, ruleCount) {
         // need anything here.
       } else {
         result = result + indent + text.substring(startIndex, endIndex);
+        if (isNewLine) {
+          lineOffset = lineOffset - 1;
+        }
         if (isCloseBrace) {
           result += prettifyCSS.LINE_SEPARATOR;
           lineOffset = lineOffset + 1;
@@ -570,7 +586,7 @@ exports.getMatchingCSSRules = getMatchingCSSRules;
  * Returns true if the given node has visited state.
  */
 function hasVisitedState(node) {
-  if (!node) {
+  if (!Element.isInstance(node)) {
     return false;
   }
 
@@ -839,3 +855,23 @@ function isCssVariable(input) {
   return !!input.match(IS_VARIABLE_TOKEN);
 }
 exports.isCssVariable = isCssVariable;
+
+/**
+ * This is a list of all the element backed pseudo elements.
+ *
+ * From https://drafts.csswg.org/css-pseudo-4/#element-backed :
+ * > The element-backed pseudo-elements, interact with most CSS and other platform features
+ * > as if they were real elements (and, in fact, often are real elements that are
+ * > not otherwise selectable).
+ *
+ * Those pseudo elements are not displayed in the markup view, but declarations in rules
+ * targetting them can then be inherited by their "children", and so we need to retrieve
+ * those rules to surface them in the Inspector (e.g. in "Inherited" sections in the Rules
+ * view, in the matched selectors section in the Computed panel, …).
+ *
+ * Any new element-backed pseudo elements should be added into this Set.
+ */
+exports.ELEMENT_BACKED_PSEUDO_ELEMENTS = new Set([
+  "::details-content",
+  "::file-selector-button",
+]);
