@@ -4,7 +4,7 @@
 
 "use strict";
 
-const protocol = require("devtools/shared/protocol");
+const protocol = require("resource://devtools/shared/protocol.js");
 const { Arg, generateActorSpec, RetVal, types } = protocol;
 
 types.addActorType("accessible");
@@ -55,7 +55,6 @@ const accessibleSpec = generateActorSpec({
       type: "nameChange",
       name: Arg(0, "string"),
       parent: Arg(1, "nullable:accessible"),
-      walker: Arg(2, "nullable:accessiblewalker"),
     },
     "value-change": {
       type: "valueChange",
@@ -80,11 +79,9 @@ const accessibleSpec = generateActorSpec({
     reorder: {
       type: "reorder",
       childCount: Arg(0, "number"),
-      walker: Arg(1, "nullable:accessiblewalker"),
     },
     "text-change": {
       type: "textChange",
-      walker: Arg(0, "nullable:accessiblewalker"),
     },
     "index-in-parent-change": {
       type: "indexInParentChange",
@@ -134,10 +131,6 @@ const accessibleWalkerSpec = generateActorSpec({
   typeName: "accessiblewalker",
 
   events: {
-    "accessible-destroy": {
-      type: "accessibleDestroy",
-      accessible: Arg(0, "accessible"),
-    },
     "document-ready": {
       type: "documentReady",
     },
@@ -176,7 +169,7 @@ const accessibleWalkerSpec = generateActorSpec({
     getAccessibleFor: {
       request: { node: Arg(0, "domnode") },
       response: {
-        accessible: RetVal("accessible"),
+        accessible: RetVal("nullable:accessible"),
       },
     },
     getAncestry: {
@@ -203,6 +196,29 @@ const accessibleWalkerSpec = generateActorSpec({
     pick: {},
     pickAndFocus: {},
     cancelPick: {},
+    showTabbingOrder: {
+      request: {
+        elm: Arg(0, "domnode"),
+        index: Arg(1, "number"),
+      },
+      response: {
+        tabbingOrderInfo: RetVal("json"),
+      },
+    },
+    hideTabbingOrder() {},
+  },
+});
+
+const simulatorSpec = generateActorSpec({
+  typeName: "simulator",
+
+  methods: {
+    simulate: {
+      request: { options: Arg(0, "nullable:json") },
+      response: {
+        value: RetVal("boolean"),
+      },
+    },
   },
 });
 
@@ -216,6 +232,38 @@ const accessibilitySpec = generateActorSpec({
     shutdown: {
       type: "shutdown",
     },
+  },
+
+  methods: {
+    getTraits: {
+      request: {},
+      response: { traits: RetVal("json") },
+    },
+    bootstrap: {
+      request: {},
+      response: {
+        state: RetVal("json"),
+      },
+    },
+    getWalker: {
+      request: {},
+      response: {
+        walker: RetVal("accessiblewalker"),
+      },
+    },
+    getSimulator: {
+      request: {},
+      response: {
+        simulator: RetVal("nullable:simulator"),
+      },
+    },
+  },
+});
+
+const parentAccessibilitySpec = generateActorSpec({
+  typeName: "parentaccessibility",
+
+  events: {
     "can-be-disabled-change": {
       type: "canBeDisabledChange",
       canBeDisabled: Arg(0, "boolean"),
@@ -233,12 +281,6 @@ const accessibilitySpec = generateActorSpec({
         state: RetVal("json"),
       },
     },
-    getWalker: {
-      request: {},
-      response: {
-        walker: RetVal("accessiblewalker"),
-      },
-    },
     enable: {
       request: {},
       response: {},
@@ -253,3 +295,5 @@ const accessibilitySpec = generateActorSpec({
 exports.accessibleSpec = accessibleSpec;
 exports.accessibleWalkerSpec = accessibleWalkerSpec;
 exports.accessibilitySpec = accessibilitySpec;
+exports.parentAccessibilitySpec = parentAccessibilitySpec;
+exports.simulatorSpec = simulatorSpec;
