@@ -3,7 +3,13 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const { Arg, Option, RetVal, generateActorSpec } = require("devtools/shared/protocol");
+const {
+  Arg,
+  Option,
+  RetVal,
+  generateActorSpec,
+  BULK_RESPONSE,
+} = require("resource://devtools/shared/protocol.js");
 
 const perfDescription = {
   typeName: "perf",
@@ -14,22 +20,25 @@ const perfDescription = {
       entries: Arg(0, "number"),
       interval: Arg(1, "number"),
       features: Arg(2, "number"),
+      duration: Arg(3, "nullable:number"),
+      // The `activeTabID` option passed to `profiler_start` is used to
+      // determine the active tab when user starts the profiler.
+      // This is a parameter that is generated on the
+      // server, that's why we don't need to pass anything on `startProfiler`
+      // actor method. But we return this in "profiler-started" event because
+      // client may want to use that value.
+      activeTabID: Arg(4, "number"),
     },
     "profiler-stopped": {
       type: "profiler-stopped",
     },
-    "profile-locked-by-private-browsing": {
-      type: "profile-locked-by-private-browsing",
-    },
-    "profile-unlocked-from-private-browsing": {
-      type: "profile-unlocked-from-private-browsing",
-    },
   },
 
   methods: {
     startProfiler: {
       request: {
         entries: Option(0, "number"),
+        duration: Option(0, "nullable:number"),
         interval: Option(0, "number"),
         features: Option(0, "array:string"),
         threads: Option(0, "array:string"),
@@ -37,14 +46,34 @@ const perfDescription = {
       response: { value: RetVal("boolean") },
     },
 
-    /**
-     * Returns null when unable to return the profile.
-     */
+    /* @backward-compat { version 143 }
+     * Version 140 introduced getProfileAndStopProfilerBulk below, a more
+     * efficient version of getProfileAndStopProfiler. getProfileAndStopProfiler
+     * needs to stay to support older versions of Firefox. */
     getProfileAndStopProfiler: {
       request: {},
       response: RetVal("nullable:json"),
     },
 
+    startCaptureAndStopProfiler: {
+      request: {},
+      response: { value: RetVal("number") },
+    },
+
+    getPreviouslyCapturedProfileDataBulk: {
+      request: {
+        handle: Arg(0, "number"),
+      },
+      response: BULK_RESPONSE,
+    },
+
+    getPreviouslyRetrievedAdditionalInformation: {
+      request: {
+        handle: Arg(0, "number"),
+      },
+      response: { value: RetVal("nullable:json") },
+    },
+
     stopProfilerAndDiscardProfile: {
       request: {},
       response: {},
@@ -68,9 +97,9 @@ const perfDescription = {
       response: { value: RetVal("boolean") },
     },
 
-    isLockedForPrivateBrowsing: {
+    getSupportedFeatures: {
       request: {},
-      response: { value: RetVal("boolean") },
+      response: { value: RetVal("array:string") },
     },
   },
 };
