@@ -9,40 +9,63 @@ const {
   RetVal,
   generateActorSpec,
   types,
-} = require("devtools/shared/protocol");
+} = require("resource://devtools/shared/protocol.js");
 
 types.addDictType("available-breakpoint-group", {
   name: "string",
   events: "array:available-breakpoint-event",
 });
+
 types.addDictType("available-breakpoint-event", {
   id: "string",
   name: "string",
 });
 
+types.addDictType("thread.frames", {
+  frames: "array:frame",
+});
+
+types.addDictType("thread.breakpoint-options", {
+  condition: "nullable:string",
+  logValue: "nullable:string",
+  showStacktrace: "nullable:boolean",
+});
+
+types.addDictType("paused-reason", {
+  type: "string",
+
+  // Used for any pause type that wants to describe why the pause happened.
+  message: "nullable:string",
+
+  // Used for the stepping pause types.
+  frameFinished: "nullable:json",
+
+  // Used for the "exception" pause type.
+  exception: "nullable:json",
+
+  // Used for the "interrupted" pause type.
+  onNext: "nullable:boolean",
+
+  // Used for the "eventBreakpoint" pause type.
+  breakpoint: "nullable:json",
+
+  // Used for the "mutationBreakpoint" pause type.
+  mutationType: "nullable:string",
+});
+
 const threadSpec = generateActorSpec({
-  typeName: "context",
+  typeName: "thread",
 
   events: {
     paused: {
       actor: Option(0, "nullable:string"),
-      frame: Option(0, "nullable:json"),
-      why: Option(0, "nullable:json"),
-      poppedFrames: Option(0, "nullable:json"),
-      error: Option(0, "nullable:json"),
-      recordingEndpoint: Option(0, "nullable:json"),
-      executionPoint: Option(0, "nullable:json"),
+      frame: Option(0, "frame"),
+      why: Option(0, "paused-reason"),
     },
     resumed: {},
-    detached: {},
-    willInterrupt: {},
     newSource: {
       source: Option(0, "json"),
     },
-    progress: {
-      recording: Option(0, "json"),
-      executionPoint: Option(0, "json"),
-    },
   },
 
   methods: {
@@ -50,10 +73,6 @@ const threadSpec = generateActorSpec({
       request: {
         options: Arg(0, "json"),
       },
-      response: RetVal("nullable:json"),
-    },
-    detach: {
-      request: {},
       response: {},
     },
     reconfigure: {
@@ -65,7 +84,7 @@ const threadSpec = generateActorSpec({
     resume: {
       request: {
         resumeLimit: Arg(0, "nullable:json"),
-        rewind: Arg(1, "boolean"),
+        frameActorID: Arg(1, "nullable:string"),
       },
       response: RetVal("nullable:json"),
     },
@@ -74,7 +93,7 @@ const threadSpec = generateActorSpec({
         start: Arg(0, "number"),
         count: Arg(1, "number"),
       },
-      response: RetVal("json"),
+      response: RetVal("thread.frames"),
     },
     interrupt: {
       request: {
@@ -82,7 +101,9 @@ const threadSpec = generateActorSpec({
       },
     },
     sources: {
-      response: RetVal("array:json"),
+      response: {
+        sources: RetVal("array:json"),
+      },
     },
     skipBreakpoints: {
       request: {
@@ -92,20 +113,18 @@ const threadSpec = generateActorSpec({
         skip: Arg(0, "json"),
       },
     },
-    threadGrips: {
-      request: {
-        actors: Arg(0, "array:string"),
-      },
+    dumpThread: {
+      request: {},
       response: RetVal("json"),
     },
-    dumpThread: {
+    dumpPools: {
       request: {},
       response: RetVal("json"),
     },
     setBreakpoint: {
       request: {
         location: Arg(0, "json"),
-        options: Arg(1, "json"),
+        options: Arg(1, "thread.breakpoint-options"),
       },
     },
     removeBreakpoint: {
@@ -152,6 +171,19 @@ const threadSpec = generateActorSpec({
         ignoreCaughtExceptions: Arg(1, "string"),
       },
     },
+
+    toggleEventLogging: {
+      request: {
+        logEventBreakpoints: Arg(0, "string"),
+      },
+    },
+
+    isAttached: {
+      request: {},
+      response: {
+        value: RetVal("boolean"),
+      },
+    },
   },
 });
 
