@@ -7,11 +7,11 @@ const StreamUtils = require("resource://devtools/shared/transport/stream-utils.j
 const StringInputStream = Components.Constructor(
   "@mozilla.org/io/string-input-stream;1",
   "nsIStringInputStream",
-  "setData"
+  "setByteStringData"
 );
 
 function run_test() {
-  add_task(async function() {
+  add_task(async function () {
     await test_delimited_read("0123:", "0123:");
     await test_delimited_read("0123:4567:", "0123:");
     await test_delimited_read("012345678901:", "0123456789");
@@ -24,7 +24,7 @@ function run_test() {
 /** * Tests ***/
 
 function test_delimited_read(input, expected) {
-  input = new StringInputStream(input, input.length);
+  input = new StringInputStream(input);
   const result = StreamUtils.delimitedRead(input, ":", 10);
   Assert.equal(result, expected);
 }
