@@ -12,11 +12,12 @@
 #include "mozilla/Maybe.h"
 #include "mozilla/NotNull.h"
 #include "mozilla/gfx/2D.h"
+#include "mozilla/image/ImageUtils.h"
 #include "nsCOMPtr.h"
+#include "Orientation.h"
 #include "SurfaceFlags.h"
 
-namespace mozilla {
-namespace image {
+namespace mozilla::image {
 
 class Decoder;
 class IDecodingTask;
@@ -25,27 +26,14 @@ class RasterImage;
 class SourceBuffer;
 class SourceBufferIterator;
 
-/**
- * The type of decoder; this is usually determined from a MIME type using
- * DecoderFactory::GetDecoderType().
- */
-enum class DecoderType {
-  PNG,
-  GIF,
-  JPEG,
-  BMP,
-  BMP_CLIPBOARD,
-  ICO,
-  ICON,
-  WEBP,
-  UNKNOWN
-};
-
 class DecoderFactory {
  public:
   /// @return the type of decoder which is appropriate for @aMimeType.
   static DecoderType GetDecoderType(const char* aMimeType);
 
+  /// @return the default flags to use when creating a decoder of @aType.
+  static DecoderFlags GetDefaultDecoderFlagsForType(DecoderType aType);
+
   /**
    * Creates and initializes a decoder for non-animated images of type @aType.
    * (If the image *is* animated, only the first frame will be decoded.) The
@@ -113,6 +101,15 @@ class DecoderFactory {
    */
   static already_AddRefed<Decoder> CloneAnimationDecoder(Decoder* aDecoder);
 
+  /**
+   * Creates and initializes a metadata decoder for an anonymous image, cloned
+   * from the given decoder.
+   *
+   * @param aDecoder Decoder to clone.
+   */
+  static already_AddRefed<Decoder> CloneAnonymousMetadataDecoder(
+      Decoder* aDecoder, const Maybe<DecoderFlags>& aDecoderFlags = Nothing());
+
   /**
    * Creates and initializes a metadata decoder of type @aType. This decoder
    * will only decode the image's header, extracting metadata like the size of
@@ -126,7 +123,7 @@ class DecoderFactory {
    *                      from.
    */
   static already_AddRefed<IDecodingTask> CreateMetadataDecoder(
-      DecoderType aType, NotNull<RasterImage*> aImage,
+      DecoderType aType, NotNull<RasterImage*> aImage, DecoderFlags aFlags,
       NotNull<SourceBuffer*> aSourceBuffer);
 
   /**
@@ -153,7 +150,7 @@ class DecoderFactory {
   static already_AddRefed<Decoder> CreateDecoderForICOResource(
       DecoderType aType, SourceBufferIterator&& aIterator,
       NotNull<nsICODecoder*> aICODecoder, bool aIsMetadataDecode,
-      const Maybe<gfx::IntSize>& aExpectedSize,
+      const Maybe<OrientedIntSize>& aExpectedSize,
       const Maybe<uint32_t>& aDataOffset = Nothing());
 
   /**
@@ -185,9 +182,11 @@ class DecoderFactory {
    * @param aType Which type of decoder to create - JPEG, PNG, etc.
    * @param aSourceBuffer The SourceBuffer which the decoder will read its data
    *                      from.
+   * @param aDecoderFlags Flags specifying the behavior of this decoder.
    */
   static already_AddRefed<Decoder> CreateAnonymousMetadataDecoder(
-      DecoderType aType, NotNull<SourceBuffer*> aSourceBuffer);
+      DecoderType aType, NotNull<SourceBuffer*> aSourceBuffer,
+      DecoderFlags aDecoderFlags);
 
  private:
   virtual ~DecoderFactory() = 0;
@@ -200,7 +199,6 @@ class DecoderFactory {
                                               bool aIsRedecode);
 };
 
-}  // namespace image
-}  // namespace mozilla
+}  // namespace mozilla::image
 
 #endif  // mozilla_image_DecoderFactory_h
