@@ -9,42 +9,40 @@
 
 #include "imgIRequest.h"
 
-#include "nsILoadGroup.h"
+#include "nsIPrincipal.h"
 #include "nsISupportsPriority.h"
 #include "nsITimedChannel.h"
 #include "nsCOMPtr.h"
 #include "nsThreadUtils.h"
+#include "mozilla/PreloaderBase.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/gfx/Rect.h"
 
-#include "imgRequest.h"
 #include "IProgressObserver.h"
 
-#define NS_IMGREQUESTPROXY_CID                       \
-  { /* 20557898-1dd2-11b2-8f65-9c462ee2bc95 */       \
-    0x20557898, 0x1dd2, 0x11b2, {                    \
-      0x8f, 0x65, 0x9c, 0x46, 0x2e, 0xe2, 0xbc, 0x95 \
-    }                                                \
-  }
+#define NS_IMGREQUESTPROXY_CID                \
+  {/* 20557898-1dd2-11b2-8f65-9c462ee2bc95 */ \
+   0x20557898,                                \
+   0x1dd2,                                    \
+   0x11b2,                                    \
+   {0x8f, 0x65, 0x9c, 0x46, 0x2e, 0xe2, 0xbc, 0x95}}
 
 class imgCacheValidator;
 class imgINotificationObserver;
+class imgRequest;
 class imgStatusNotifyRunnable;
 class ProxyBehaviour;
 
 namespace mozilla {
-namespace dom {
-class TabGroup;
-}
-
 namespace image {
 class Image;
 class ProgressTracker;
 }  // namespace image
 }  // namespace mozilla
 
-class imgRequestProxy : public imgIRequest,
+class imgRequestProxy : public mozilla::PreloaderBase,
+                        public imgIRequest,
                         public mozilla::image::IProgressObserver,
                         public nsISupportsPriority,
                         public nsITimedChannel {
@@ -52,9 +50,11 @@ class imgRequestProxy : public imgIRequest,
   virtual ~imgRequestProxy();
 
  public:
+  typedef mozilla::dom::Document Document;
   typedef mozilla::image::Image Image;
   typedef mozilla::image::ProgressTracker ProgressTracker;
 
+  NS_INLINE_DECL_STATIC_IID(NS_IMGREQUESTPROXY_CID)
   MOZ_DECLARE_REFCOUNTED_TYPENAME(imgRequestProxy)
   NS_DECL_ISUPPORTS
   NS_DECL_IMGIREQUEST
@@ -66,8 +66,7 @@ class imgRequestProxy : public imgIRequest,
 
   // Callers to Init or ChangeOwner are required to call NotifyListener after
   // (although not immediately after) doing so.
-  nsresult Init(imgRequest* aOwner, nsILoadGroup* aLoadGroup,
-                nsIDocument* aLoadingDocument, nsIURI* aURI,
+  nsresult Init(imgRequest* aOwner, nsILoadGroup* aLoadGroup, nsIURI* aURI,
                 imgINotificationObserver* aObserver);
 
   nsresult ChangeOwner(imgRequest* aNewOwner);  // this will change mOwner.
@@ -111,8 +110,9 @@ class imgRequestProxy : public imgIRequest,
   void MarkValidating();
   void ClearValidating();
 
-  bool IsOnEventTarget() const;
-  already_AddRefed<nsIEventTarget> GetEventTarget() const override;
+  // Flags this image load as not cancelable temporarily. This is needed so that
+  // stylesheets can be shared across documents properly, see bug 1800979.
+  void SetCancelable(bool);
 
   // Removes all animation consumers that were created with
   // IncrementAnimationConsumers. This is necessary since we need
@@ -121,11 +121,35 @@ class imgRequestProxy : public imgIRequest,
   void ClearAnimationConsumers();
 
   nsresult SyncClone(imgINotificationObserver* aObserver,
-                     nsIDocument* aLoadingDocument, imgRequestProxy** aClone);
+                     Document* aLoadingDocument, imgRequestProxy** aClone);
   nsresult Clone(imgINotificationObserver* aObserver,
-                 nsIDocument* aLoadingDocument, imgRequestProxy** aClone);
-  nsresult GetStaticRequest(nsIDocument* aLoadingDocument,
-                            imgRequestProxy** aReturn);
+                 Document* aLoadingDocument, imgRequestProxy** aClone);
+  already_AddRefed<imgRequestProxy> GetStaticRequest(
+      Document* aLoadingDocument);
+
+  imgRequest* GetOwner() const;
+
+  struct LCPTimings {
+    bool AreSet() const { return mLoadTime.isSome() && mRenderTime.isSome(); }
+
+    void Reset() {
+      mLoadTime = mozilla::Nothing();
+      mRenderTime = mozilla::Nothing();
+    }
+
+    mozilla::Maybe<mozilla::TimeStamp> mLoadTime;
+    mozilla::Maybe<mozilla::TimeStamp> mRenderTime;
+
+    void Set(const mozilla::TimeStamp& aLoadTime,
+             const mozilla::TimeStamp& aRenderTime) {
+      mLoadTime = Some(aLoadTime);
+      mRenderTime = Some(aRenderTime);
+    }
+  };
+
+  LCPTimings& GetLCPTimings() { return mLCPTimings; }
+
+  const LCPTimings& GetLCPTimings() const { return mLCPTimings; }
 
  protected:
   friend class mozilla::image::ProgressTracker;
@@ -152,7 +176,7 @@ class imgRequestProxy : public imgIRequest,
   /* Remove from and forget the load group. */
   void RemoveFromLoadGroup();
 
-  /* Remove from the load group and readd as a background request. */
+  /* Remove from the load group and re-add as a background request. */
   void MoveToBackgroundInLoadGroup();
 
   /* Finish up canceling ourselves */
@@ -167,20 +191,14 @@ class imgRequestProxy : public imgIRequest,
   //   (b) whether mOwner has instantiated its image yet
   already_AddRefed<ProgressTracker> GetProgressTracker() const;
 
-  nsITimedChannel* TimedChannel() {
-    if (!GetOwner()) {
-      return nullptr;
-    }
-    return GetOwner()->GetTimedChannel();
-  }
+  nsITimedChannel* TimedChannel();
 
   already_AddRefed<Image> GetImage() const;
   bool HasImage() const;
-  imgRequest* GetOwner() const;
   imgCacheValidator* GetValidator() const;
 
   nsresult PerformClone(imgINotificationObserver* aObserver,
-                        nsIDocument* aLoadingDocument, bool aSyncNotify,
+                        Document* aLoadingDocument, bool aSyncNotify,
                         imgRequestProxy** aClone);
 
   virtual imgRequestProxy* NewClonedProxy();
@@ -194,15 +212,15 @@ class imgRequestProxy : public imgIRequest,
  private:
   friend class imgCacheValidator;
 
-  void AddToOwner(nsIDocument* aLoadingDocument);
+  void AddToOwner();
   void RemoveFromOwner(nsresult aStatus);
 
   nsresult DispatchWithTargetIfAvailable(already_AddRefed<nsIRunnable> aEvent);
-  void DispatchWithTarget(already_AddRefed<nsIRunnable> aEvent);
 
   // The URI of our request.
   nsCOMPtr<nsIURI> mURI;
 
+  LCPTimings mLCPTimings;
   // mListener is only promised to be a weak ref (see imgILoader.idl),
   // but we actually keep a strong ref to it until we've seen our
   // first OnStopRequest.
@@ -212,12 +230,11 @@ class imgRequestProxy : public imgIRequest,
       "they are destroyed") mListener;
 
   nsCOMPtr<nsILoadGroup> mLoadGroup;
-  RefPtr<mozilla::dom::TabGroup> mTabGroup;
-  nsCOMPtr<nsIEventTarget> mEventTarget;
 
   nsLoadFlags mLoadFlags;
   uint32_t mLockCount;
   uint32_t mAnimationConsumers;
+  bool mCancelable : 1;
   bool mCanceled : 1;
   bool mIsInLoadGroup : 1;
   bool mForceDispatchLoadGroup : 1;
@@ -228,24 +245,34 @@ class imgRequestProxy : public imgIRequest,
   // interfaces as image loads proceed.
   bool mPendingNotify : 1;
   bool mValidating : 1;
-  bool mHadListener : 1;
-  bool mHadDispatch : 1;
 };
 
+inline nsISupports* ToSupports(imgRequestProxy* p) {
+  return NS_ISUPPORTS_CAST(imgIRequest*, p);
+}
+
 // Used for static image proxies for which no requests are available, so
 // certain behaviours must be overridden to compensate.
 class imgRequestProxyStatic : public imgRequestProxy {
  public:
-  imgRequestProxyStatic(Image* aImage, nsIPrincipal* aPrincipal);
+  imgRequestProxyStatic(Image* aImage, nsIPrincipal* aImagePrincipal,
+                        nsIPrincipal* aTriggeringPrincipal,
+                        bool hadCrossOriginRedirects);
 
   NS_IMETHOD GetImagePrincipal(nsIPrincipal** aPrincipal) override;
+  NS_IMETHOD GetTriggeringPrincipal(nsIPrincipal** aPrincipal) override;
+
+  NS_IMETHOD GetHadCrossOriginRedirects(
+      bool* aHadCrossOriginRedirects) override;
 
  protected:
   imgRequestProxy* NewClonedProxy() override;
 
   // Our principal. We have to cache it, rather than accessing the underlying
   // request on-demand, because static proxies don't have an underlying request.
-  nsCOMPtr<nsIPrincipal> mPrincipal;
+  const nsCOMPtr<nsIPrincipal> mImagePrincipal;
+  const nsCOMPtr<nsIPrincipal> mTriggeringPrincipal;
+  const bool mHadCrossOriginRedirects;
 };
 
 #endif  // mozilla_image_imgRequestProxy_h
