@@ -20,41 +20,36 @@ using namespace mozilla::image;
 namespace mozilla {
 namespace image {
 
-class TestSurfacePipeFactory
-{
-public:
-  static SurfacePipe SimpleSurfacePipe()
-  {
+class TestSurfacePipeFactory {
+ public:
+  static SurfacePipe SimpleSurfacePipe() {
     SurfacePipe pipe;
-    return Move(pipe);
+    return pipe;
   }
 
   template <typename T>
-  static SurfacePipe SurfacePipeFromPipeline(T&& aPipeline)
-  {
-    return SurfacePipe { Move(aPipeline) };
+  static SurfacePipe SurfacePipeFromPipeline(T&& aPipeline) {
+    return SurfacePipe{std::move(aPipeline)};
   }
 
-private:
-  TestSurfacePipeFactory() { }
+ private:
+  TestSurfacePipeFactory() {}
 };
 
-} // namespace image
-} // namespace mozilla
+}  // namespace image
+}  // namespace mozilla
 
-void
-CheckSurfacePipeMethodResults(SurfacePipe* aPipe,
-                              Decoder* aDecoder,
-                              const IntRect& aRect = IntRect(0, 0, 100, 100))
-{
+void CheckSurfacePipeMethodResults(SurfacePipe* aPipe, image::Decoder* aDecoder,
+                                   const IntRect& aRect = IntRect(0, 0, 100,
+                                                                  100)) {
   // Check that the pipeline ended up in the state we expect.  Note that we're
   // explicitly testing the SurfacePipe versions of these methods, so we don't
   // want to use AssertCorrectPipelineFinalState() here.
   EXPECT_TRUE(aPipe->IsSurfaceFinished());
   Maybe<SurfaceInvalidRect> invalidRect = aPipe->TakeInvalidRect();
   EXPECT_TRUE(invalidRect.isSome());
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mInputSpaceRect);
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mOutputSpaceRect);
+  EXPECT_EQ(OrientedIntRect(0, 0, 100, 100), invalidRect->mInputSpaceRect);
+  EXPECT_EQ(OrientedIntRect(0, 0, 100, 100), invalidRect->mOutputSpaceRect);
 
   // Check the generated image.
   CheckGeneratedImage(aDecoder, aRect);
@@ -76,8 +71,8 @@ CheckSurfacePipeMethodResults(SurfacePipe* aPipe,
   EXPECT_TRUE(aPipe->IsSurfaceFinished());
   invalidRect = aPipe->TakeInvalidRect();
   EXPECT_TRUE(invalidRect.isSome());
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mInputSpaceRect);
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mOutputSpaceRect);
+  EXPECT_EQ(OrientedIntRect(0, 0, 100, 100), invalidRect->mInputSpaceRect);
+  EXPECT_EQ(OrientedIntRect(0, 0, 100, 100), invalidRect->mOutputSpaceRect);
 
   aPipe->ResetToFirstRow();
   EXPECT_FALSE(aPipe->IsSurfaceFinished());
@@ -85,58 +80,12 @@ CheckSurfacePipeMethodResults(SurfacePipe* aPipe,
   EXPECT_TRUE(invalidRect.isNothing());
 }
 
-void
-CheckPalettedSurfacePipeMethodResults(SurfacePipe* aPipe,
-                                      Decoder* aDecoder,
-                                      const IntRect& aRect
-                                        = IntRect(0, 0, 100, 100))
-{
-  // Check that the pipeline ended up in the state we expect.  Note that we're
-  // explicitly testing the SurfacePipe versions of these methods, so we don't
-  // want to use AssertCorrectPipelineFinalState() here.
-  EXPECT_TRUE(aPipe->IsSurfaceFinished());
-  Maybe<SurfaceInvalidRect> invalidRect = aPipe->TakeInvalidRect();
-  EXPECT_TRUE(invalidRect.isSome());
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mInputSpaceRect);
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mOutputSpaceRect);
-
-  // Check the generated image.
-  CheckGeneratedPalettedImage(aDecoder, aRect);
-
-  // Reset and clear the image before the next test.
-  aPipe->ResetToFirstRow();
-  EXPECT_FALSE(aPipe->IsSurfaceFinished());
-  invalidRect = aPipe->TakeInvalidRect();
-  EXPECT_TRUE(invalidRect.isNothing());
-
-  uint32_t count = 0;
-  auto result = aPipe->WritePixels<uint8_t>([&]() {
-    ++count;
-    return AsVariant(uint8_t(0));
-  });
-  EXPECT_EQ(WriteState::FINISHED, result);
-  EXPECT_EQ(100u * 100u, count);
-
-  EXPECT_TRUE(aPipe->IsSurfaceFinished());
-  invalidRect = aPipe->TakeInvalidRect();
-  EXPECT_TRUE(invalidRect.isSome());
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mInputSpaceRect);
-  EXPECT_EQ(IntRect(0, 0, 100, 100), invalidRect->mOutputSpaceRect);
-
-  aPipe->ResetToFirstRow();
-  EXPECT_FALSE(aPipe->IsSurfaceFinished());
-  invalidRect = aPipe->TakeInvalidRect();
-  EXPECT_TRUE(invalidRect.isNothing());
-}
-
-class ImageSurfacePipeIntegration : public ::testing::Test
-{
-protected:
+class ImageSurfacePipeIntegration : public ::testing::Test {
+ protected:
   AutoInitializeImageLib mInit;
 };
 
-TEST_F(ImageSurfacePipeIntegration, SurfacePipe)
-{
+TEST_F(ImageSurfacePipeIntegration, SurfacePipe) {
   // Test that SurfacePipe objects can be initialized and move constructed.
   SurfacePipe pipe = TestSurfacePipeFactory::SimpleSurfacePipe();
 
@@ -144,14 +93,13 @@ TEST_F(ImageSurfacePipeIntegration, SurfacePipe)
   pipe = TestSurfacePipeFactory::SimpleSurfacePipe();
 
   // Test that SurfacePipe objects can be initialized with a pipeline.
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
   auto sink = MakeUnique<SurfaceSink>();
-  nsresult rv =
-    sink->Configure(SurfaceConfig { decoder, 0, IntSize(100, 100),
-                                    SurfaceFormat::B8G8R8A8, false });
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
+  nsresult rv = sink->Configure(
+      SurfaceConfig{decoder, IntSize(100, 100), SurfaceFormat::OS_RGBA, false});
+  ASSERT_NS_SUCCEEDED(rv);
 
   pipe = TestSurfacePipeFactory::SurfacePipeFromPipeline(sink);
 
@@ -201,6 +149,23 @@ TEST_F(ImageSurfacePipeIntegration, SurfacePipe)
     CheckSurfacePipeMethodResults(&pipe, decoder);
   }
 
+  // Test that WritePixelBlocks() gets passed through to the underlying
+  // pipeline.
+  {
+    uint32_t count = 0;
+    WriteState result = pipe.WritePixelBlocks<uint32_t>(
+        [&](uint32_t* aBlockStart, int32_t aLength) {
+          ++count;
+          EXPECT_EQ(int32_t(100), aLength);
+          memcpy(aBlockStart, buffer, 100 * sizeof(uint32_t));
+          return std::make_tuple(int32_t(100), Maybe<WriteState>());
+        });
+
+    EXPECT_EQ(WriteState::FINISHED, result);
+    EXPECT_EQ(100u, count);
+    CheckSurfacePipeMethodResults(&pipe, decoder);
+  }
+
   // Test that WriteEmptyRow() gets passed through to the underlying pipeline.
   {
     uint32_t count = 0;
@@ -219,106 +184,24 @@ TEST_F(ImageSurfacePipeIntegration, SurfacePipe)
   currentFrame->Finish();
 }
 
-TEST_F(ImageSurfacePipeIntegration, PalettedSurfacePipe)
-{
-  // Create a SurfacePipe containing a PalettedSurfaceSink.
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+TEST_F(ImageSurfacePipeIntegration, DeinterlaceDownscaleWritePixels) {
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
-  auto sink = MakeUnique<PalettedSurfaceSink>();
-  nsresult rv =
-    sink->Configure(PalettedSurfaceConfig { decoder, 0, IntSize(100, 100),
-                                            IntRect(0, 0, 100, 100),
-                                            SurfaceFormat::B8G8R8A8,
-                                            8, false });
-  ASSERT_TRUE(NS_SUCCEEDED(rv));
-
-  SurfacePipe pipe = TestSurfacePipeFactory::SurfacePipeFromPipeline(sink);
-
-  // Test that WritePixels() gets passed through to the underlying pipeline.
-  {
-    uint32_t count = 0;
-    auto result = pipe.WritePixels<uint8_t>([&]() {
-      ++count;
-      return AsVariant(uint8_t(255));
-    });
-    EXPECT_EQ(WriteState::FINISHED, result);
-    EXPECT_EQ(100u * 100u, count);
-    CheckPalettedSurfacePipeMethodResults(&pipe, decoder);
-  }
-
-  // Create a buffer the same size as one row of the surface, containing all
-  // 255 pixels. We'll use this for the WriteBuffer() tests.
-  uint8_t buffer[100];
-  for (int i = 0; i < 100; ++i) {
-    buffer[i] = 255;
-  }
-
-  // Test that WriteBuffer() gets passed through to the underlying pipeline.
-  {
-    uint32_t count = 0;
-    WriteState result = WriteState::NEED_MORE_DATA;
-    while (result == WriteState::NEED_MORE_DATA) {
-      result = pipe.WriteBuffer(buffer);
-      ++count;
-    }
-    EXPECT_EQ(WriteState::FINISHED, result);
-    EXPECT_EQ(100u, count);
-    CheckPalettedSurfacePipeMethodResults(&pipe, decoder);
-  }
-
-  // Test that the 3 argument version of WriteBuffer() gets passed through to
-  // the underlying pipeline.
-  {
-    uint32_t count = 0;
-    WriteState result = WriteState::NEED_MORE_DATA;
-    while (result == WriteState::NEED_MORE_DATA) {
-      result = pipe.WriteBuffer(buffer, 0, 100);
-      ++count;
-    }
-    EXPECT_EQ(WriteState::FINISHED, result);
-    EXPECT_EQ(100u, count);
-    CheckPalettedSurfacePipeMethodResults(&pipe, decoder);
-  }
-
-  // Test that WriteEmptyRow() gets passed through to the underlying pipeline.
-  {
-    uint32_t count = 0;
-    WriteState result = WriteState::NEED_MORE_DATA;
-    while (result == WriteState::NEED_MORE_DATA) {
-      result = pipe.WriteEmptyRow();
-      ++count;
-    }
-    EXPECT_EQ(WriteState::FINISHED, result);
-    EXPECT_EQ(100u, count);
-    CheckPalettedSurfacePipeMethodResults(&pipe, decoder, IntRect(0, 0, 0, 0));
-  }
-
-  // Mark the frame as finished so we don't get an assertion.
-  RawAccessFrameRef currentFrame = decoder->GetCurrentFrameRef();
-  currentFrame->Finish();
-}
-
-TEST_F(ImageSurfacePipeIntegration, DeinterlaceDownscaleWritePixels)
-{
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
-  ASSERT_TRUE(decoder != nullptr);
-
-  auto test = [](Decoder* aDecoder, SurfaceFilter* aFilter) {
+  auto test = [](image::Decoder* aDecoder, SurfaceFilter* aFilter) {
     CheckWritePixels(aDecoder, aFilter,
                      /* aOutputRect = */ Some(IntRect(0, 0, 25, 25)));
   };
 
-  WithFilterPipeline(decoder, test,
-                     DeinterlacingConfig<uint32_t> { /* mProgressiveDisplay = */ true },
-                     DownscalingConfig { IntSize(100, 100),
-                                         SurfaceFormat::B8G8R8A8 },
-                     SurfaceConfig { decoder, 0, IntSize(25, 25),
-                                     SurfaceFormat::B8G8R8A8, false });
+  WithFilterPipeline(
+      decoder, test,
+      DeinterlacingConfig<uint32_t>{/* mProgressiveDisplay = */ true},
+      DownscalingConfig{IntSize(100, 100), SurfaceFormat::OS_RGBA},
+      SurfaceConfig{decoder, IntSize(25, 25), SurfaceFormat::OS_RGBA, false});
 }
 
-TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectBottomRightDownscaleWritePixels)
-{
+TEST_F(ImageSurfacePipeIntegration,
+       RemoveFrameRectBottomRightDownscaleWritePixels) {
   // This test case uses a frame rect that extends beyond the borders of the
   // image to the bottom and to the right. It looks roughly like this (with the
   // box made of '#'s representing the frame rect):
@@ -331,7 +214,7 @@ TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectBottomRightDownscaleWritePixe
   //        +############+
   //        +------------+
 
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
   // Note that aInputWriteRect is 100x50 because RemoveFrameRectFilter ignores
@@ -354,9 +237,10 @@ TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectBottomRightDownscaleWritePixe
   // (3) The surface owned by SurfaceSink logically has only a 10x10 region
   //     region in it that's non-blank; this is the downscaled version of the
   //     50x50 region discussed in (1). (aOutputWriteRect captures this fact.)
-  //     Some fuzz, as usual, is necessary when dealing with Lanczos downscaling.
+  //     Some fuzz, as usual, is necessary when dealing with Lanczos
+  //     downscaling.
 
-  auto test = [](Decoder* aDecoder, SurfaceFilter* aFilter) {
+  auto test = [](image::Decoder* aDecoder, SurfaceFilter* aFilter) {
     CheckWritePixels(aDecoder, aFilter,
                      /* aOutputRect = */ Some(IntRect(0, 0, 20, 20)),
                      /* aInputRect = */ Some(IntRect(0, 0, 100, 100)),
@@ -365,16 +249,14 @@ TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectBottomRightDownscaleWritePixe
                      /* aFuzz = */ 0x33);
   };
 
-  WithFilterPipeline(decoder, test,
-                     RemoveFrameRectConfig { IntRect(50, 50, 100, 100) },
-                     DownscalingConfig { IntSize(100, 100),
-                                         SurfaceFormat::B8G8R8A8 },
-                     SurfaceConfig { decoder, 0, IntSize(20, 20),
-                                     SurfaceFormat::B8G8R8A8, false });
+  WithFilterPipeline(
+      decoder, test, RemoveFrameRectConfig{IntRect(50, 50, 100, 100)},
+      DownscalingConfig{IntSize(100, 100), SurfaceFormat::OS_RGBA},
+      SurfaceConfig{decoder, IntSize(20, 20), SurfaceFormat::OS_RGBA, false});
 }
 
-TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectTopLeftDownscaleWritePixels)
-{
+TEST_F(ImageSurfacePipeIntegration,
+       RemoveFrameRectTopLeftDownscaleWritePixels) {
   // This test case uses a frame rect that extends beyond the borders of the
   // image to the top and to the left. It looks roughly like this (with the
   // box made of '#'s representing the frame rect):
@@ -387,10 +269,10 @@ TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectTopLeftDownscaleWritePixels)
   //        +            +
   //        +------------+
 
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
-  auto test = [](Decoder* aDecoder, SurfaceFilter* aFilter) {
+  auto test = [](image::Decoder* aDecoder, SurfaceFilter* aFilter) {
     CheckWritePixels(aDecoder, aFilter,
                      /* aOutputRect = */ Some(IntRect(0, 0, 20, 20)),
                      /* aInputRect = */ Some(IntRect(0, 0, 100, 100)),
@@ -399,24 +281,21 @@ TEST_F(ImageSurfacePipeIntegration, RemoveFrameRectTopLeftDownscaleWritePixels)
                      /* aFuzz = */ 0x21);
   };
 
-  WithFilterPipeline(decoder, test,
-                     RemoveFrameRectConfig { IntRect(-50, -50, 100, 100) },
-                     DownscalingConfig { IntSize(100, 100),
-                                         SurfaceFormat::B8G8R8A8 },
-                     SurfaceConfig { decoder, 0, IntSize(20, 20),
-                                     SurfaceFormat::B8G8R8A8, false });
+  WithFilterPipeline(
+      decoder, test, RemoveFrameRectConfig{IntRect(-50, -50, 100, 100)},
+      DownscalingConfig{IntSize(100, 100), SurfaceFormat::OS_RGBA},
+      SurfaceConfig{decoder, IntSize(20, 20), SurfaceFormat::OS_RGBA, false});
 }
 
-TEST_F(ImageSurfacePipeIntegration, DeinterlaceRemoveFrameRectWritePixels)
-{
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+TEST_F(ImageSurfacePipeIntegration, DeinterlaceRemoveFrameRectWritePixels) {
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
   // Note that aInputRect is the full 100x100 size even though
   // RemoveFrameRectFilter is part of this pipeline, because deinterlacing
   // requires reading every row.
 
-  auto test = [](Decoder* aDecoder, SurfaceFilter* aFilter) {
+  auto test = [](image::Decoder* aDecoder, SurfaceFilter* aFilter) {
     CheckWritePixels(aDecoder, aFilter,
                      /* aOutputRect = */ Some(IntRect(0, 0, 100, 100)),
                      /* aInputRect = */ Some(IntRect(0, 0, 100, 100)),
@@ -424,19 +303,19 @@ TEST_F(ImageSurfacePipeIntegration, DeinterlaceRemoveFrameRectWritePixels)
                      /* aOutputWriteRect = */ Some(IntRect(50, 50, 50, 50)));
   };
 
-  WithFilterPipeline(decoder, test,
-                     DeinterlacingConfig<uint32_t> { /* mProgressiveDisplay = */ true },
-                     RemoveFrameRectConfig { IntRect(50, 50, 100, 100) },
-                     SurfaceConfig { decoder, 0, IntSize(100, 100),
-                                     SurfaceFormat::B8G8R8A8, false });
+  WithFilterPipeline(
+      decoder, test,
+      DeinterlacingConfig<uint32_t>{/* mProgressiveDisplay = */ true},
+      RemoveFrameRectConfig{IntRect(50, 50, 100, 100)},
+      SurfaceConfig{decoder, IntSize(100, 100), SurfaceFormat::OS_RGBA, false});
 }
 
-TEST_F(ImageSurfacePipeIntegration, DeinterlaceRemoveFrameRectDownscaleWritePixels)
-{
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+TEST_F(ImageSurfacePipeIntegration,
+       DeinterlaceRemoveFrameRectDownscaleWritePixels) {
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
-  auto test = [](Decoder* aDecoder, SurfaceFilter* aFilter) {
+  auto test = [](image::Decoder* aDecoder, SurfaceFilter* aFilter) {
     CheckWritePixels(aDecoder, aFilter,
                      /* aOutputRect = */ Some(IntRect(0, 0, 20, 20)),
                      /* aInputRect = */ Some(IntRect(0, 0, 100, 100)),
@@ -445,52 +324,16 @@ TEST_F(ImageSurfacePipeIntegration, DeinterlaceRemoveFrameRectDownscaleWritePixe
                      /* aFuzz = */ 33);
   };
 
-  WithFilterPipeline(decoder, test,
-                     DeinterlacingConfig<uint32_t> { /* mProgressiveDisplay = */ true },
-                     RemoveFrameRectConfig { IntRect(50, 50, 100, 100) },
-                     DownscalingConfig { IntSize(100, 100),
-                                         SurfaceFormat::B8G8R8A8 },
-                     SurfaceConfig { decoder, 0, IntSize(20, 20),
-                                     SurfaceFormat::B8G8R8A8, false });
+  WithFilterPipeline(
+      decoder, test,
+      DeinterlacingConfig<uint32_t>{/* mProgressiveDisplay = */ true},
+      RemoveFrameRectConfig{IntRect(50, 50, 100, 100)},
+      DownscalingConfig{IntSize(100, 100), SurfaceFormat::OS_RGBA},
+      SurfaceConfig{decoder, IntSize(20, 20), SurfaceFormat::OS_RGBA, false});
 }
 
-TEST_F(ImageSurfacePipeIntegration, ConfiguringPalettedRemoveFrameRectDownscaleFails)
-{
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
-  ASSERT_TRUE(decoder != nullptr);
-
-  // This is an invalid pipeline for paletted images, so configuration should
-  // fail.
-  AssertConfiguringPipelineFails(decoder,
-                                 RemoveFrameRectConfig { IntRect(0, 0, 50, 50) },
-                                 DownscalingConfig { IntSize(100, 100),
-                                                     SurfaceFormat::B8G8R8A8 },
-                                 PalettedSurfaceConfig { decoder, 0, IntSize(100, 100),
-                                                         IntRect(0, 0, 50, 50),
-                                                         SurfaceFormat::B8G8R8A8, 8,
-                                                         false });
-}
-
-TEST_F(ImageSurfacePipeIntegration, ConfiguringPalettedDeinterlaceDownscaleFails)
-{
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
-  ASSERT_TRUE(decoder != nullptr);
-
-  // This is an invalid pipeline for paletted images, so configuration should
-  // fail.
-  AssertConfiguringPipelineFails(decoder,
-                                 DeinterlacingConfig<uint8_t> { /* mProgressiveDisplay = */ true},
-                                 DownscalingConfig { IntSize(100, 100),
-                                                     SurfaceFormat::B8G8R8A8 },
-                                 PalettedSurfaceConfig { decoder, 0, IntSize(100, 100),
-                                                         IntRect(0, 0, 20, 20),
-                                                         SurfaceFormat::B8G8R8A8, 8,
-                                                         false });
-}
-
-TEST_F(ImageSurfacePipeIntegration, ConfiguringHugeDeinterlacingBufferFails)
-{
-  RefPtr<Decoder> decoder = CreateTrivialDecoder();
+TEST_F(ImageSurfacePipeIntegration, ConfiguringHugeDeinterlacingBufferFails) {
+  RefPtr<image::Decoder> decoder = CreateTrivialDecoder();
   ASSERT_TRUE(decoder != nullptr);
 
   // When DownscalingFilter is used, we may succeed in allocating an output
@@ -499,10 +342,8 @@ TEST_F(ImageSurfacePipeIntegration, ConfiguringHugeDeinterlacingBufferFails)
   // DeinterlacingFilter needs to allocate a buffer as large as the size of the
   // input. This can cause OOMs on operating systems that allow overcommit. This
   // test makes sure that we reject such allocations.
-  AssertConfiguringPipelineFails(decoder,
-                                 DeinterlacingConfig<uint32_t> { /* mProgressiveDisplay = */ true},
-                                 DownscalingConfig { IntSize(60000, 60000),
-                                                     SurfaceFormat::B8G8R8A8 },
-                                 SurfaceConfig { decoder, 0, IntSize(600, 600),
-                                                 SurfaceFormat::B8G8R8A8, false });
+  AssertConfiguringPipelineFails(
+      decoder, DeinterlacingConfig<uint32_t>{/* mProgressiveDisplay = */ true},
+      DownscalingConfig{IntSize(60000, 60000), SurfaceFormat::OS_RGBA},
+      SurfaceConfig{decoder, IntSize(600, 600), SurfaceFormat::OS_RGBA, false});
 }
