@@ -1,17 +1,17 @@
-// |reftest| skip error:SyntaxError -- BigInt is not supported
+// |reftest| error:SyntaxError
 // Copyright (C) 2017 Robin Templeton. All rights reserved.
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-description: > 
+description: >
   It is a Syntax Error if the NumericLiteralBase contains an ExponentPart.
 esid: sec-numeric-literal-static-semantics-early-errors
 features: [BigInt]
 negative:
-  phase: early
+  phase: parse
   type: SyntaxError
 ---*/
 
-throw "Test262: This statement should not be evaluated.";
+$DONOTEVALUATE();
 
 0e0n;
