@@ -21,7 +21,8 @@
 
 #include "builtin/ModuleObject.h"  // ModuleObject, Handle<ModuleObject*>
 #include "frontend/ParserAtom.h"   // frontend::TaggedParserAtomIndex
-#include "gc/Barrier.h"            // HeapPtr
+#include "gc/Allocator.h"          // HandleBuffer
+#include "gc/Barrier.h"            // GCPtr
 #include "gc/Cell.h"               // TenuredCellWithNonGCPointer
 #include "js/GCPolicyAPI.h"        // GCPolicy, IgnoreGCPolicy
 #include "js/HeapAPI.h"            // CellFlagBitsReservedForGC
@@ -29,7 +30,6 @@
 #include "js/TraceKind.h"          // JS::TraceKind
 #include "js/TypeDecls.h"          // HandleFunction
 #include "js/UbiNode.h"            // ubi::*
-#include "js/UniquePtr.h"          // UniquePtr
 #include "util/Poison.h"  // AlwaysPoison, JS_SCOPE_DATA_TRAILING_NAMES_PATTERN, MemCheckKind
 #include "vm/JSFunction.h"  // JSFunction
 #include "vm/ScopeKind.h"   // ScopeKind
@@ -43,7 +43,7 @@ struct JSContext;
 
 namespace js {
 
-class GenericPrinter;
+class JS_PUBLIC_API GenericPrinter;
 
 namespace frontend {
 class ScopeStencil;
@@ -332,10 +332,10 @@ class Scope : public gc::TenuredCellWithNonGCPointer<BaseScopeData> {
 
   // If there are any aliased bindings, the shape for the
   // EnvironmentObject. Otherwise nullptr.
-  const HeapPtr<SharedShape*> environmentShape_;
+  const GCPtr<SharedShape*> environmentShape_;
 
   // The enclosing scope or nullptr.
-  HeapPtr<Scope*> enclosingScope_;
+  GCPtr<Scope*> enclosingScope_;
 
   Scope(ScopeKind kind, Scope* enclosing, SharedShape* environmentShape)
       : TenuredCellWithNonGCPointer(nullptr),
@@ -347,8 +347,7 @@ class Scope : public gc::TenuredCellWithNonGCPointer<BaseScopeData> {
                        Handle<SharedShape*> envShape);
 
   template <typename ConcreteScope>
-  void initData(
-      MutableHandle<UniquePtr<typename ConcreteScope::RuntimeData>> data);
+  void initData(HandleBuffer<typename ConcreteScope::RuntimeData> data);
 
   template <typename F>
   void applyScopeDataTyped(F&& f);
@@ -361,7 +360,7 @@ class Scope : public gc::TenuredCellWithNonGCPointer<BaseScopeData> {
   static ConcreteScope* create(
       JSContext* cx, ScopeKind kind, Handle<Scope*> enclosing,
       Handle<SharedShape*> envShape,
-      MutableHandle<UniquePtr<typename ConcreteScope::RuntimeData>> data);
+      HandleBuffer<typename ConcreteScope::RuntimeData> data);
 
   static const JS::TraceKind TraceKind = JS::TraceKind::Scope;
 
@@ -434,9 +433,8 @@ class Scope : public gc::TenuredCellWithNonGCPointer<BaseScopeData> {
   }
 
   void traceChildren(JSTracer* trc);
-  void finalize(JS::GCContext* gcx);
 
-  size_t sizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const;
+  size_t sizeOfExcludingThis() const;
 
   void dump();
 #if defined(DEBUG) || defined(JS_JITSPEW)
@@ -525,6 +523,11 @@ class LexicalScope : public Scope {
     //   lets - [0, constStart)
     // consts - [constStart, length)
     uint32_t constStart = 0;
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+    // consts - [constStart, usingStart)
+    // usings - [usingStart, length)
+    uint32_t usingStart = 0;
+#endif
   };
 
   using RuntimeData = RuntimeScopeData<SlotInfo>;
@@ -710,12 +713,12 @@ class FunctionScope : public Scope {
     // The canonical function of the scope, as during a scope walk we
     // often query properties of the JSFunction (e.g., is the function an
     // arrow).
-    HeapPtr<JSFunction*> canonicalFunction = {};
+    GCPtr<JSFunction*> canonicalFunction = {};
 
     explicit RuntimeData(size_t length) { PoisonNames(this, length); }
     RuntimeData() = delete;
 
-    void trace(JSTracer* trc);
+    inline void trace(JSTracer* trc);
   };
 
   using ParserData = ParserScopeData<SlotInfo>;
@@ -860,9 +863,8 @@ class GlobalScope : public Scope {
   static GlobalScope* createEmpty(JSContext* cx, ScopeKind kind);
 
  private:
-  static GlobalScope* createWithData(
-      JSContext* cx, ScopeKind kind,
-      MutableHandle<UniquePtr<RuntimeData>> data);
+  static GlobalScope* createWithData(JSContext* cx, ScopeKind kind,
+                                     HandleBuffer<RuntimeData> data);
 
   RuntimeData& data() { return *static_cast<RuntimeData*>(rawData()); }
 
@@ -1001,18 +1003,23 @@ class ModuleScope : public Scope {
     uint32_t varStart = 0;
     uint32_t letStart = 0;
     uint32_t constStart = 0;
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+    // consts - [constStart, usingStart)
+    // usings - [usingStart, length)
+    uint32_t usingStart = 0;
+#endif
   };
 
   struct alignas(ScopeDataAlignBytes) RuntimeData
       : public AbstractBaseScopeData<JSAtom> {
     SlotInfo slotInfo;
     // The module of the scope.
-    HeapPtr<ModuleObject*> module = {};
+    GCPtr<ModuleObject*> module = {};
 
     explicit RuntimeData(size_t length);
     RuntimeData() = delete;
 
-    void trace(JSTracer* trc);
+    inline void trace(JSTracer* trc);
   };
 
   using ParserData = ParserScopeData<SlotInfo>;
@@ -1060,6 +1067,7 @@ class WasmInstanceScope : public Scope {
     //
     // memories - [0, globalsStart)
     //  globals - [globalsStart, length)
+    uint32_t memoriesStart = 0;
     uint32_t globalsStart = 0;
   };
 
@@ -1067,12 +1075,12 @@ class WasmInstanceScope : public Scope {
       : public AbstractBaseScopeData<JSAtom> {
     SlotInfo slotInfo;
     // The wasm instance of the scope.
-    HeapPtr<WasmInstanceObject*> instance = {};
+    GCPtr<WasmInstanceObject*> instance = {};
 
     explicit RuntimeData(size_t length);
     RuntimeData() = delete;
 
-    void trace(JSTracer* trc);
+    inline void trace(JSTracer* trc);
   };
 
   using ParserData = ParserScopeData<SlotInfo>;
@@ -1082,7 +1090,8 @@ class WasmInstanceScope : public Scope {
       typename std::conditional_t<std::is_same<NameT, JSAtom>::value,
                                   RuntimeData, ParserData>;
 
-  static WasmInstanceScope* create(JSContext* cx, WasmInstanceObject* instance);
+  static WasmInstanceScope* create(JSContext* cx,
+                                   Handle<WasmInstanceObject*> instance);
 
  private:
   RuntimeData& data() { return *static_cast<RuntimeData*>(rawData()); }
@@ -1094,7 +1103,7 @@ class WasmInstanceScope : public Scope {
  public:
   WasmInstanceObject* instance() const { return data().instance; }
 
-  uint32_t memoriesStart() const { return 0; }
+  uint32_t memoriesStart() const { return data().slotInfo.memoriesStart; }
 
   uint32_t globalsStart() const { return data().slotInfo.globalsStart; }
 
@@ -1215,6 +1224,11 @@ class BaseAbstractBindingIter {
   //          synthetic - [syntheticStart, privateMethodStart)
   //    private methods = [privateMethodStart, length)
   //
+  // If ENABLE_EXPLICIT_RESOURCE_MANAGEMENT is set, the consts range is split
+  // into the following:
+  //             consts - [constStart, usingStart)
+  //             usings - [usingStart, syntheticStart)
+  //
   // Access method when not closed over:
   //
   //            imports - name
@@ -1241,6 +1255,9 @@ class BaseAbstractBindingIter {
   MOZ_INIT_OUTSIDE_CTOR uint32_t varStart_;
   MOZ_INIT_OUTSIDE_CTOR uint32_t letStart_;
   MOZ_INIT_OUTSIDE_CTOR uint32_t constStart_;
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+  MOZ_INIT_OUTSIDE_CTOR uint32_t usingStart_;
+#endif
   MOZ_INIT_OUTSIDE_CTOR uint32_t syntheticStart_;
   MOZ_INIT_OUTSIDE_CTOR uint32_t privateMethodStart_;
   MOZ_INIT_OUTSIDE_CTOR uint32_t length_;
@@ -1272,6 +1289,9 @@ class BaseAbstractBindingIter {
 
   void init(uint32_t positionalFormalStart, uint32_t nonPositionalFormalStart,
             uint32_t varStart, uint32_t letStart, uint32_t constStart,
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+            uint32_t usingStart,
+#endif
             uint32_t syntheticStart, uint32_t privateMethodStart, uint8_t flags,
             uint32_t firstFrameSlot, uint32_t firstEnvironmentSlot,
             mozilla::Span<AbstractBindingName<NameT>> names) {
@@ -1280,6 +1300,9 @@ class BaseAbstractBindingIter {
     varStart_ = varStart;
     letStart_ = letStart;
     constStart_ = constStart;
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+    usingStart_ = usingStart;
+#endif
     syntheticStart_ = syntheticStart;
     privateMethodStart_ = privateMethodStart;
     length_ = names.size();
@@ -1470,10 +1493,20 @@ class BaseAbstractBindingIter {
     if (index_ < constStart_) {
       return BindingKind::Let;
     }
+#ifdef ENABLE_EXPLICIT_RESOURCE_MANAGEMENT
+    if (index_ < usingStart_) {
+      return isNamedLambda() ? BindingKind::NamedLambdaCallee
+                             : BindingKind::Const;
+    }
+    if (index_ < syntheticStart_) {
+      return BindingKind::Using;
+    }
+#else
     if (index_ < syntheticStart_) {
       return isNamedLambda() ? BindingKind::NamedLambdaCallee
                              : BindingKind::Const;
     }
+#endif
     if (index_ < privateMethodStart_) {
       return BindingKind::Synthetic;
     }
@@ -1764,6 +1797,10 @@ SharedShape* CreateEnvironmentShape(JSContext* cx, BindingIter& bi,
                                     const JSClass* cls, uint32_t numSlots,
                                     ObjectFlags objectFlags);
 
+SharedShape* CreateEnvironmentShapeForSyntheticModule(
+    JSContext* cx, const JSClass* cls, uint32_t numSlots,
+    Handle<ModuleObject*> module);
+
 SharedShape* EmptyEnvironmentShape(JSContext* cx, const JSClass* cls,
                                    uint32_t numSlots, ObjectFlags objectFlags);
 
@@ -1844,7 +1881,7 @@ template <>
 struct GCPolicy<js::ScopeKind> : public IgnoreGCPolicy<js::ScopeKind> {};
 
 template <typename T>
-struct ScopeDataGCPolicy : public NonGCPointerPolicy<T> {};
+using ScopeDataGCPolicy = NonGCPointerPolicy<T>;
 
 #define DEFINE_SCOPE_DATA_GCPOLICY(Data)              \
   template <>                                         \
@@ -1862,6 +1899,7 @@ DEFINE_SCOPE_DATA_GCPOLICY(js::GlobalScope::RuntimeData);
 DEFINE_SCOPE_DATA_GCPOLICY(js::EvalScope::RuntimeData);
 DEFINE_SCOPE_DATA_GCPOLICY(js::ModuleScope::RuntimeData);
 DEFINE_SCOPE_DATA_GCPOLICY(js::WasmFunctionScope::RuntimeData);
+DEFINE_SCOPE_DATA_GCPOLICY(js::WasmInstanceScope::RuntimeData);
 
 #undef DEFINE_SCOPE_DATA_GCPOLICY
 
