@@ -8,6 +8,7 @@
 #include "ScriptPreloader-inl.h"
 
 #include "mozilla/Unused.h"
+#include "mozilla/Try.h"
 #include "mozilla/ipc/FileDescriptor.h"
 #include "nsIFile.h"
 
@@ -33,14 +34,13 @@ Result<Ok, nsresult> AutoMemMap::init(nsIFile* file, int flags, int mode,
                                       PRFileMapProtect prot) {
   MOZ_ASSERT(!fd);
 
-  MOZ_TRY(file->OpenNSPRFileDesc(flags, mode, &fd.rwget()));
+  MOZ_TRY(file->OpenNSPRFileDesc(flags, mode, getter_Transfers(fd)));
 
   return initInternal(prot);
 }
 
 Result<Ok, nsresult> AutoMemMap::init(const FileDescriptor& file,
-                                      PRFileMapProtect prot,
-                                      size_t expectedSize) {
+                                      PRFileMapProtect prot, size_t maybeSize) {
   MOZ_ASSERT(!fd);
   if (!file.IsValid()) {
     return Err(NS_ERROR_INVALID_ARG);
@@ -48,39 +48,42 @@ Result<Ok, nsresult> AutoMemMap::init(const FileDescriptor& file,
 
   auto handle = file.ClonePlatformHandle();
 
-  fd = PR_ImportFile(PROsfd(handle.get()));
+  fd.reset(PR_ImportFile(PROsfd(handle.get())));
   if (!fd) {
     return Err(NS_ERROR_FAILURE);
   }
   Unused << handle.release();
 
-  return initInternal(prot, expectedSize);
+  return initInternal(prot, maybeSize);
 }
 
 Result<Ok, nsresult> AutoMemMap::initInternal(PRFileMapProtect prot,
-                                              size_t expectedSize) {
+                                              size_t maybeSize) {
   MOZ_ASSERT(!fileMap);
   MOZ_ASSERT(!addr);
 
-  PRFileInfo64 fileInfo;
-  MOZ_TRY(PR_GetOpenFileInfo64(fd.get(), &fileInfo));
+  if (maybeSize > 0) {
+    // Some OSes' shared memory objects can't be stat()ed, either at
+    // all (Android) or without loosening the sandbox (Mac) so just
+    // use the size.
+    size_ = maybeSize;
+  } else {
+    // But if we don't have the size, assume it's a regular file and
+    // ask for it.
+    PRFileInfo64 fileInfo;
+    MOZ_TRY(PR_GetOpenFileInfo64(fd.get(), &fileInfo));
 
-  if (fileInfo.size > UINT32_MAX) {
-    return Err(NS_ERROR_INVALID_ARG);
+    if (fileInfo.size > UINT32_MAX) {
+      return Err(NS_ERROR_INVALID_ARG);
+    }
+    size_ = fileInfo.size;
   }
 
-  fileMap = PR_CreateFileMap(fd, 0, prot);
+  fileMap = PR_CreateFileMap(fd.get(), 0, prot);
   if (!fileMap) {
     return Err(NS_ERROR_FAILURE);
   }
 
-  size_ = fileInfo.size;
-  // The memory region size passed in certain IPC messages isn't necessary on
-  // Unix-like systems, since we can always stat the file descriptor to
-  // determine it accurately. But since we have it, anyway, sanity check that
-  // it matches the size returned by the stat.
-  MOZ_ASSERT_IF(expectedSize > 0, size_ == expectedSize);
-
   addr = PR_MemMap(fileMap, 0, size_);
   if (!addr) {
     return Err(NS_ERROR_FAILURE);
@@ -89,46 +92,8 @@ Result<Ok, nsresult> AutoMemMap::initInternal(PRFileMapProtect prot,
   return Ok();
 }
 
-#ifdef XP_WIN
-
-Result<Ok, nsresult> AutoMemMap::initWithHandle(const FileDescriptor& file,
-                                                size_t size,
-                                                PRFileMapProtect prot) {
-  MOZ_ASSERT(!fd);
-  MOZ_ASSERT(!handle_);
-  if (!file.IsValid()) {
-    return Err(NS_ERROR_INVALID_ARG);
-  }
-
-  handle_ = file.ClonePlatformHandle().release();
-
-  MOZ_ASSERT(!addr);
-
-  size_ = size;
-
-  addr = MapViewOfFile(
-      handle_, prot == PR_PROT_READONLY ? FILE_MAP_READ : FILE_MAP_ALL_ACCESS,
-      0, 0, size);
-
-  return Ok();
-}
-
-FileDescriptor AutoMemMap::cloneHandle() const {
-  return FileDescriptor(handle_);
-}
-
-#else
-
-Result<Ok, nsresult> AutoMemMap::initWithHandle(const FileDescriptor& file,
-                                                size_t size,
-                                                PRFileMapProtect prot) {
-  return init(file, prot);
-}
-
 FileDescriptor AutoMemMap::cloneHandle() const { return cloneFileDescriptor(); }
 
-#endif
-
 void AutoMemMap::reset() {
   if (addr && !persistent_) {
     Unused << NS_WARN_IF(PR_MemUnmap(addr, size()) != PR_SUCCESS);
@@ -138,13 +103,7 @@ void AutoMemMap::reset() {
     Unused << NS_WARN_IF(PR_CloseFileMap(fileMap) != PR_SUCCESS);
     fileMap = nullptr;
   }
-#ifdef XP_WIN
-  if (handle_) {
-    CloseHandle(handle_);
-    handle_ = nullptr;
-  }
-#endif
-  fd.dispose();
+  fd = nullptr;
 }
 
 }  // namespace loader
