@@ -10,7 +10,7 @@
 #include "nsString.h"
 
 #include "js/experimental/JSStencil.h"
-#include "js/CompileOptions.h"  // JS::DecodeOptions
+#include "js/CompileOptions.h"  // JS::ReadOnlyDecodeOptions
 
 namespace mozilla {
 namespace scache {
@@ -20,7 +20,7 @@ class StartupCache;
 
 nsresult ReadCachedStencil(mozilla::scache::StartupCache* cache,
                            nsACString& cachePath, JSContext* cx,
-                           const JS::DecodeOptions& options,
+                           const JS::ReadOnlyDecodeOptions& options,
                            JS::Stencil** stencilOut);
 
 nsresult WriteCachedStencil(mozilla::scache::StartupCache* cache,
