@@ -52,8 +52,6 @@ static constexpr gfx::sRGBColor sColorGrey10Alpha50(
     gfx::sRGBColor::UnusualFromARGB(0x7fe9e9ed));
 static constexpr gfx::sRGBColor sColorGrey20(
     gfx::sRGBColor::UnusualFromARGB(0xffd0d0d7));
-static constexpr gfx::sRGBColor sColorGrey30(
-    gfx::sRGBColor::UnusualFromARGB(0xffb1b1b9));
 static constexpr gfx::sRGBColor sColorGrey40(
     gfx::sRGBColor::UnusualFromARGB(0xff8f8f9d));
 static constexpr gfx::sRGBColor sColorGrey40Alpha50(
@@ -76,7 +74,6 @@ static constexpr gfx::sRGBColor sColorMeterRed10(
 static constexpr gfx::sRGBColor sColorMeterRed20(
     gfx::sRGBColor::UnusualFromARGB(0xff810220));
 
-static const CSSCoord kMinimumRangeThumbSize = 20.0f;
 static const CSSCoord kMinimumDropdownArrowButtonWidth = 18.0f;
 static const CSSCoord kMinimumSpinnerButtonWidth = 18.0f;
 static const CSSCoord kMinimumSpinnerButtonHeight = 9.0f;
@@ -265,16 +262,10 @@ sRGBColor Theme::ComputeBorderColor(const ElementState& aState,
   bool isActive =
       aState.HasAllStates(ElementState::HOVER | ElementState::ACTIVE);
   bool isHovered = aState.HasState(ElementState::HOVER);
-  if (aColors.HighContrast()) {
-    return aColors.System(isDisabled ? StyleSystemColor::Graytext
-                          : (isHovered && !isActive)
-                              ? StyleSystemColor::Selecteditem
-                              : StyleSystemColor::Buttontext);
+  if (isDisabled) {
+    return aColors.System(StyleSystemColor::MozButtondisabledborder);
   }
   bool isFocused = aState.HasState(ElementState::FOCUSRING);
-  if (isDisabled) {
-    return sColorGrey40Alpha50;
-  }
   if (isFocused && aOutlineCoversBorder == OutlineCoversBorder::Yes) {
     // If we draw the outline over the border, prevent issues where the border
     // shows underneath if it snaps in the wrong direction by using a
@@ -284,14 +275,13 @@ sRGBColor Theme::ComputeBorderColor(const ElementState& aState,
     // But this looks harder to mess up.
     return sTransparent;
   }
-  bool dark = aColors.IsDark();
   if (isActive) {
-    return dark ? sColorGrey20 : sColorGrey60;
+    return aColors.System(StyleSystemColor::MozButtonactiveborder);
   }
   if (isHovered) {
-    return dark ? sColorGrey30 : sColorGrey50;
+    return aColors.System(StyleSystemColor::MozButtonhoverborder);
   }
-  return sColorGrey40;
+  return aColors.System(StyleSystemColor::Buttonborder);
 }
 
 std::pair<sRGBColor, sRGBColor> Theme::ComputeButtonColors(
@@ -316,7 +306,6 @@ std::pair<sRGBColor, sRGBColor> Theme::ComputeButtonColors(
     }
     return aColors.SystemNs(StyleSystemColor::Buttonface);
   }();
-
   const sRGBColor borderColor =
       ComputeBorderColor(aState, aColors, OutlineCoversBorder::Yes);
   return std::make_pair(sRGBColor::FromABGR(backgroundColor), borderColor);
@@ -614,6 +603,9 @@ void Theme::PaintCircleShadow(DrawTarget& aDrawTarget,
   if (dtSize.IsEmpty()) {
     return;
   }
+  if (!aDrawTarget.CanCreateSimilarDrawTarget(dtSize, SurfaceFormat::A8)) {
+    return;
+  }
   RefPtr<DrawTarget> ellipseDT = aDrawTarget.CreateSimilarDrawTargetForFilter(
       dtSize, SurfaceFormat::A8, blurFilter, blurFilter,
       sourceRectInFilterSpace, destinationPointOfSourceRect);
@@ -809,8 +801,9 @@ void Theme::PaintRange(nsIFrame* aFrame, PaintBackendData& aPaintData,
   auto tickMarks = rangeFrame->TickMarks();
   double progress = rangeFrame->GetValueAsFractionOfRange();
   auto rect = aRect;
-  LayoutDeviceRect thumbRect(0, 0, kMinimumRangeThumbSize * aDpiRatio,
-                             kMinimumRangeThumbSize * aDpiRatio);
+  const CSSCoord minThumbSize = GetMinimumRangeThumbSize();
+  LayoutDeviceRect thumbRect(0, 0, minThumbSize * aDpiRatio,
+                             minThumbSize * aDpiRatio);
   LayoutDeviceRect progressClipRect(aRect);
   LayoutDeviceRect trackClipRect(aRect);
   const LayoutDeviceCoord verticalSize = kRangeHeight * aDpiRatio;
@@ -1029,7 +1022,7 @@ void Theme::PaintButton(PaintBackendData& aPaintData,
   if (aAppearance == StyleAppearance::Toolbarbutton &&
       (!aState.HasState(ElementState::HOVER) ||
        aState.HasState(ElementState::DISABLED))) {
-    borderColor = sTransparent;
+    backgroundColor = borderColor = sTransparent;
   }
 
   ThemeDrawing::PaintRoundedRectWithRadius(aPaintData, aRect, backgroundColor,
@@ -1042,16 +1035,13 @@ void Theme::PaintButton(PaintBackendData& aPaintData,
   }
 }
 
-NS_IMETHODIMP
-Theme::DrawWidgetBackground(gfxContext* aContext, nsIFrame* aFrame,
-                            StyleAppearance aAppearance, const nsRect& aRect,
-                            const nsRect& /* aDirtyRect */,
-                            DrawOverflow aDrawOverflow) {
-  if (!DoDrawWidgetBackground(*aContext->GetDrawTarget(), aFrame, aAppearance,
-                              aRect, aDrawOverflow)) {
-    return NS_ERROR_NOT_IMPLEMENTED;
-  }
-  return NS_OK;
+void Theme::DrawWidgetBackground(gfxContext* aContext, nsIFrame* aFrame,
+                                 StyleAppearance aAppearance,
+                                 const nsRect& aRect,
+                                 const nsRect& /* aDirtyRect */,
+                                 DrawOverflow aDrawOverflow) {
+  DoDrawWidgetBackground(*aContext->GetDrawTarget(), aFrame, aAppearance, aRect,
+                         aDrawOverflow);
 }
 
 bool Theme::CreateWebRenderCommandsForWidget(
@@ -1121,7 +1111,6 @@ bool Theme::DoDrawWidgetBackground(PaintBackendData& aPaintData,
   const nscoord twipsPerPixel = pc->AppUnitsPerDevPixel();
   const auto devPxRect = ToSnappedRect(aRect, twipsPerPixel, aPaintData);
 
-  const DocumentState docState = pc->Document()->State();
   ElementState elementState = GetContentState(aFrame, aAppearance);
   // Paint the outline iff we're asked to draw overflow and we have
   // outline-style: auto.
@@ -1206,25 +1195,16 @@ bool Theme::DoDrawWidgetBackground(PaintBackendData& aPaintData,
       PaintRange(aFrame, aPaintData, devPxRect, elementState, colors, dpiRatio,
                  IsRangeHorizontal(aFrame));
       break;
-    case StyleAppearance::RangeThumb:
-      // Painted as part of StyleAppearance::Range.
-      break;
     case StyleAppearance::ProgressBar:
       PaintProgress(aFrame, aPaintData, devPxRect, elementState, colors,
                     dpiRatio,
                     /* aIsMeter = */ false);
       break;
-    case StyleAppearance::Progresschunk:
-      /* Painted as part of the progress bar */
-      break;
     case StyleAppearance::Meter:
       PaintProgress(aFrame, aPaintData, devPxRect, elementState, colors,
                     dpiRatio,
                     /* aIsMeter = */ true);
       break;
-    case StyleAppearance::Meterchunk:
-      /* Painted as part of the meter bar */
-      break;
     case StyleAppearance::ScrollbarthumbHorizontal:
     case StyleAppearance::ScrollbarthumbVertical: {
       bool isHorizontal =
@@ -1232,8 +1212,8 @@ bool Theme::DoDrawWidgetBackground(PaintBackendData& aPaintData,
       auto kind = ComputeScrollbarKind(aFrame, isHorizontal);
       return GetScrollbarDrawing().PaintScrollbarThumb(
           aPaintData, devPxRect, kind, aFrame,
-          *nsLayoutUtils::StyleForScrollbar(aFrame), elementState, docState,
-          colors, dpiRatio);
+          *nsLayoutUtils::StyleForScrollbar(aFrame), elementState, colors,
+          dpiRatio);
     }
     case StyleAppearance::ScrollbarHorizontal:
     case StyleAppearance::ScrollbarVertical: {
@@ -1241,15 +1221,14 @@ bool Theme::DoDrawWidgetBackground(PaintBackendData& aPaintData,
       auto kind = ComputeScrollbarKind(aFrame, isHorizontal);
       return GetScrollbarDrawing().PaintScrollbar(
           aPaintData, devPxRect, kind, aFrame,
-          *nsLayoutUtils::StyleForScrollbar(aFrame), elementState, docState,
-          colors, dpiRatio);
+          *nsLayoutUtils::StyleForScrollbar(aFrame), elementState, colors,
+          dpiRatio);
     }
     case StyleAppearance::Scrollcorner: {
       auto kind = ComputeScrollbarKindForScrollCorner(aFrame);
       return GetScrollbarDrawing().PaintScrollCorner(
           aPaintData, devPxRect, kind, aFrame,
-          *nsLayoutUtils::StyleForScrollbar(aFrame), docState, colors,
-          dpiRatio);
+          *nsLayoutUtils::StyleForScrollbar(aFrame), colors, dpiRatio);
     }
     case StyleAppearance::ScrollbarbuttonUp:
     case StyleAppearance::ScrollbarbuttonDown:
@@ -1267,8 +1246,8 @@ bool Theme::DoDrawWidgetBackground(PaintBackendData& aPaintData,
           auto kind = ComputeScrollbarKind(aFrame, isHorizontal);
           GetScrollbarDrawing().PaintScrollbarButton(
               aPaintData, aAppearance, devPxRect, kind, aFrame,
-              *nsLayoutUtils::StyleForScrollbar(aFrame), elementState, docState,
-              colors, dpiRatio);
+              *nsLayoutUtils::StyleForScrollbar(aFrame), elementState, colors,
+              dpiRatio);
         }
       }
       break;
@@ -1315,7 +1294,7 @@ void Theme::PaintAutoStyleOutline(nsIFrame* aFrame,
 
   LayoutDeviceRect rect(aRect);
   auto devOffset = LayoutDevicePixel::FromAppUnits(cssOffset, a2d);
-  nscoord cssRadii[8] = {0};
+  nsRectCornerRadii cssRadii;
   if (!aFrame->GetBorderRadii(cssRadii)) {
     // The goal of this code is getting a 0px inner radius, but 2px outer
     // radius.
@@ -1325,9 +1304,7 @@ void Theme::PaintAutoStyleOutline(nsIFrame* aFrame,
     auto twoDevPixels = CSSCoord(2) * aDpiRatio;
     rect.Inflate(devOffset + twoDevPixels);
     devOffset = -twoDevPixels;
-    for (auto& r : cssRadii) {
-      r = radius;
-    }
+    cssRadii = nsRectCornerRadii(radius);
   }
 
   RectCornerRadii innerRadii;
@@ -1368,9 +1345,11 @@ void Theme::PaintAutoStyleOutline(
   auto DrawRect = [&](const sRGBColor& aColor) {
     RectCornerRadii outerRadii;
     if constexpr (std::is_same_v<PaintBackendData, WebRenderBackendData>) {
-      const Float widths[4] = {strokeWidth + aOffset, strokeWidth + aOffset,
-                               strokeWidth + aOffset, strokeWidth + aOffset};
-      nsCSSBorderRenderer::ComputeOuterRadii(aInnerRadii, widths, &outerRadii);
+      const LayoutDeviceMargin widths(
+          strokeWidth + aOffset, strokeWidth + aOffset, strokeWidth + aOffset,
+          strokeWidth + aOffset);
+      nsCSSBorderRenderer::ComputeOuterRadii(
+          aInnerRadii, widths.ToUnknownMargin(), &outerRadii);
       const auto dest = wr::ToLayoutRect(rect);
       const auto side =
           wr::ToBorderSide(ToDeviceColor(aColor), StyleBorderStyle::Solid);
@@ -1383,9 +1362,10 @@ void Theme::PaintAutoStyleOutline(
                                      {sides, 4}, wrRadius);
     } else {
       const LayoutDeviceCoord halfWidth = strokeWidth * 0.5f;
-      const Float widths[4] = {halfWidth + aOffset, halfWidth + aOffset,
-                               halfWidth + aOffset, halfWidth + aOffset};
-      nsCSSBorderRenderer::ComputeOuterRadii(aInnerRadii, widths, &outerRadii);
+      const LayoutDeviceMargin widths(halfWidth + aOffset, halfWidth + aOffset,
+                                      halfWidth + aOffset, halfWidth + aOffset);
+      nsCSSBorderRenderer::ComputeOuterRadii(
+          aInnerRadii, widths.ToUnknownMargin(), &outerRadii);
       LayoutDeviceRect dest(rect);
       dest.Deflate(halfWidth);
       RefPtr<Path> path =
@@ -1540,10 +1520,6 @@ LayoutDeviceIntSize Theme::GetMinimumWidgetSize(nsPresContext* aPresContext,
 
   LayoutDeviceIntSize result;
   switch (aAppearance) {
-    case StyleAppearance::RangeThumb:
-      result.SizeTo((kMinimumRangeThumbSize * dpiRatio).Rounded(),
-                    (kMinimumRangeThumbSize * dpiRatio).Rounded());
-      break;
     case StyleAppearance::MozMenulistArrowButton:
       result.width = (kMinimumDropdownArrowButtonWidth * dpiRatio).Rounded();
       break;
@@ -1587,9 +1563,6 @@ bool Theme::WidgetAttributeChangeRequiresRepaint(StyleAppearance aAppearance,
          aAttribute == nsGkAtoms::_default || aAttribute == nsGkAtoms::open;
 }
 
-NS_IMETHODIMP
-Theme::ThemeChanged() { return NS_OK; }
-
 bool Theme::WidgetAppearanceDependsOnWindowFocus(StyleAppearance aAppearance) {
   return IsWidgetScrollbarPart(aAppearance);
 }
@@ -1608,11 +1581,8 @@ bool Theme::ThemeSupportsWidget(nsPresContext* aPresContext, nsIFrame* aFrame,
     case StyleAppearance::Textarea:
     case StyleAppearance::Textfield:
     case StyleAppearance::Range:
-    case StyleAppearance::RangeThumb:
     case StyleAppearance::ProgressBar:
-    case StyleAppearance::Progresschunk:
     case StyleAppearance::Meter:
-    case StyleAppearance::Meterchunk:
     case StyleAppearance::ScrollbarbuttonUp:
     case StyleAppearance::ScrollbarbuttonDown:
     case StyleAppearance::ScrollbarbuttonLeft:
