@@ -22,38 +22,7 @@ using namespace mozilla;
 
 #define MAX_NOTIFICATION_NAME_LEN 5000
 
-@protocol FakeNSUserNotification <NSObject>
-@property(copy) NSString* title;
-@property(copy) NSString* subtitle;
-@property(copy) NSString* informativeText;
-@property(copy) NSString* actionButtonTitle;
-@property(copy) NSDictionary* userInfo;
-@property(copy) NSDate* deliveryDate;
-@property(copy) NSTimeZone* deliveryTimeZone;
-@property(copy) NSDateComponents* deliveryRepeatInterval;
-@property(readonly) NSDate* actualDeliveryDate;
-@property(readonly, getter=isPresented) BOOL presented;
-@property(readonly, getter=isRemote) BOOL remote;
-@property(copy) NSString* soundName;
-@property BOOL hasActionButton;
-@property(readonly) NSUserNotificationActivationType activationType;
-@property(copy) NSString* otherButtonTitle;
-@property(copy) NSImage* contentImage;
-@end
-
-@protocol FakeNSUserNotificationCenter <NSObject>
-+ (id<FakeNSUserNotificationCenter>)defaultUserNotificationCenter;
-@property(assign) id<NSUserNotificationCenterDelegate> delegate;
-@property(copy) NSArray* scheduledNotifications;
-- (void)scheduleNotification:(id<FakeNSUserNotification>)notification;
-- (void)removeScheduledNotification:(id<FakeNSUserNotification>)notification;
-@property(readonly) NSArray* deliveredNotifications;
-- (void)deliverNotification:(id<FakeNSUserNotification>)notification;
-- (void)removeDeliveredNotification:(id<FakeNSUserNotification>)notification;
-- (void)removeAllDeliveredNotifications;
-- (void)_removeAllDisplayedNotifications;
-- (void)_removeDisplayedNotification:(id<FakeNSUserNotification>)notification;
-@end
+static constexpr nsLiteralString kActionSuffix = u"-moz"_ns;
 
 @interface mozNotificationCenterDelegate
     : NSObject <NSUserNotificationCenterDelegate> {
@@ -71,32 +40,27 @@ using namespace mozilla;
   return self;
 }
 
-- (void)userNotificationCenter:(id<FakeNSUserNotificationCenter>)center
-        didDeliverNotification:(id<FakeNSUserNotification>)notification {
+- (void)userNotificationCenter:(NSUserNotificationCenter*)center
+        didDeliverNotification:(NSUserNotification*)notification {
 }
 
-- (void)userNotificationCenter:(id<FakeNSUserNotificationCenter>)center
-       didActivateNotification:(id<FakeNSUserNotification>)notification {
-  unsigned long long additionalActionIndex = ULLONG_MAX;
-  if ([notification respondsToSelector:@selector(_alternateActionIndex)]) {
-    NSNumber* alternateActionIndex =
-        [(NSObject*)notification valueForKey:@"_alternateActionIndex"];
-    additionalActionIndex = [alternateActionIndex unsignedLongLongValue];
-  }
+- (void)userNotificationCenter:(NSUserNotificationCenter*)center
+       didActivateNotification:(NSUserNotification*)notification {
   mOSXNC->OnActivate([[notification userInfo] valueForKey:@"name"],
-                     notification.activationType, additionalActionIndex);
+                     notification.activationType,
+                     notification.additionalActivationAction);
 }
 
-- (BOOL)userNotificationCenter:(id<FakeNSUserNotificationCenter>)center
-     shouldPresentNotification:(id<FakeNSUserNotification>)notification {
+- (BOOL)userNotificationCenter:(NSUserNotificationCenter*)center
+     shouldPresentNotification:(NSUserNotification*)notification {
   return YES;
 }
 
 // This is an undocumented method that we need for parity with Safari.
 // Apple bug #15440664.
-- (void)userNotificationCenter:(id<FakeNSUserNotificationCenter>)center
+- (void)userNotificationCenter:(NSUserNotificationCenter*)center
     didRemoveDeliveredNotifications:(NSArray*)notifications {
-  for (id<FakeNSUserNotification> notification in notifications) {
+  for (NSUserNotification* notification in notifications) {
     NSString* name = [[notification userInfo] valueForKey:@"name"];
     mOSXNC->CloseAlertCocoaString(name);
   }
@@ -104,8 +68,8 @@ using namespace mozilla;
 
 // This is an undocumented method that we need to be notified if a user clicks
 // the close button.
-- (void)userNotificationCenter:(id<FakeNSUserNotificationCenter>)center
-               didDismissAlert:(id<FakeNSUserNotification>)notification {
+- (void)userNotificationCenter:(NSUserNotificationCenter*)center
+               didDismissAlert:(NSUserNotification*)notification {
   NSString* name = [[notification userInfo] valueForKey:@"name"];
   mOSXNC->CloseAlertCocoaString(name);
 }
@@ -114,35 +78,33 @@ using namespace mozilla;
 
 namespace mozilla {
 
-enum {
-  OSXNotificationActionDisable = 0,
-  OSXNotificationActionSettings = 1,
-};
-
 class OSXNotificationInfo final : public nsISupports {
  private:
   virtual ~OSXNotificationInfo();
 
  public:
   NS_DECL_ISUPPORTS
-  OSXNotificationInfo(NSString* name, nsIObserver* observer,
-                      const nsAString& alertCookie);
+  OSXNotificationInfo(NSString* name, nsIAlertNotification* aAlertNotification,
+                      nsIObserver* observer, const nsAString& alertCookie);
 
   NSString* mName;
+  nsCOMPtr<nsIAlertNotification> mAlertNotification;
   nsCOMPtr<nsIObserver> mObserver;
   nsString mCookie;
   RefPtr<nsICancelable> mIconRequest;
-  id<FakeNSUserNotification> mPendingNotification;
+  NSUserNotification* mPendingNotification;
 };
 
 NS_IMPL_ISUPPORTS0(OSXNotificationInfo)
 
-OSXNotificationInfo::OSXNotificationInfo(NSString* name, nsIObserver* observer,
-                                         const nsAString& alertCookie) {
+OSXNotificationInfo::OSXNotificationInfo(
+    NSString* name, nsIAlertNotification* aAlertNotification,
+    nsIObserver* observer, const nsAString& alertCookie) {
   NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
 
   NS_ASSERTION(name, "Cannot create OSXNotificationInfo without a name!");
   mName = [name retain];
+  mAlertNotification = aAlertNotification;
   mObserver = observer;
   mCookie = alertCookie;
   mPendingNotification = nil;
@@ -159,7 +121,7 @@ OSXNotificationInfo::~OSXNotificationInfo() {
   NS_OBJC_END_TRY_IGNORE_BLOCK;
 }
 
-static id<FakeNSUserNotificationCenter> GetNotificationCenter() {
+static NSUserNotificationCenter* GetNotificationCenter() {
   NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   Class c = NSClassFromString(@"NSUserNotificationCenter");
@@ -187,7 +149,7 @@ OSXNotificationCenter::~OSXNotificationCenter() {
   NS_OBJC_END_TRY_IGNORE_BLOCK;
 }
 
-NS_IMPL_ISUPPORTS(OSXNotificationCenter, nsIAlertsService, nsIAlertsIconData,
+NS_IMPL_ISUPPORTS(OSXNotificationCenter, nsIAlertsService,
                   nsIAlertsDoNotDisturb, nsIAlertNotificationImageListener)
 
 nsresult OSXNotificationCenter::Init() {
@@ -220,24 +182,9 @@ OSXNotificationCenter::ShowAlertNotification(
   return ShowAlert(alert, aAlertListener);
 }
 
-NS_IMETHODIMP
-OSXNotificationCenter::ShowPersistentNotification(
-    const nsAString& aPersistentData, nsIAlertNotification* aAlert,
-    nsIObserver* aAlertListener) {
-  return ShowAlert(aAlert, aAlertListener);
-}
-
 NS_IMETHODIMP
 OSXNotificationCenter::ShowAlert(nsIAlertNotification* aAlert,
                                  nsIObserver* aAlertListener) {
-  return ShowAlertWithIconData(aAlert, aAlertListener, 0, nullptr);
-}
-
-NS_IMETHODIMP
-OSXNotificationCenter::ShowAlertWithIconData(nsIAlertNotification* aAlert,
-                                             nsIObserver* aAlertListener,
-                                             uint32_t aIconSize,
-                                             const uint8_t* aIconData) {
   NS_OBJC_BEGIN_TRY_BLOCK_RETURN;
 
   NS_ENSURE_ARG(aAlert);
@@ -247,7 +194,7 @@ OSXNotificationCenter::ShowAlertWithIconData(nsIAlertNotification* aAlert,
   }
 
   Class unClass = NSClassFromString(@"NSUserNotification");
-  id<FakeNSUserNotification> notification = [[unClass alloc] init];
+  NSUserNotification* notification = [[unClass alloc] init];
 
   nsAutoString title;
   nsresult rv = aAlert->GetTitle(title);
@@ -279,48 +226,63 @@ OSXNotificationCenter::ShowAlertWithIconData(nsIAlertNotification* aAlert,
   bool isSilent;
   aAlert->GetSilent(&isSilent);
   notification.soundName = isSilent ? nil : NSUserNotificationDefaultSoundName;
-  notification.hasActionButton = NO;
+
+  NSMutableArray* additionalActions = [[NSMutableArray alloc] init];
+
+  nsTArray<RefPtr<nsIAlertAction>> actions;
+  MOZ_TRY(aAlert->GetActions(actions));
+
+  for (const RefPtr<nsIAlertAction>& action : actions) {
+    nsAutoString actionName;
+    MOZ_TRY(action->GetAction(actionName));
+
+    nsAutoString actionTitle;
+    MOZ_TRY(action->GetTitle(actionTitle));
+
+    // Add suffix to prevent potential collision with keywords like "settings"
+    NSString* actionNameNS =
+        nsCocoaUtils::ToNSString(actionName + kActionSuffix);
+    NSString* actionTitleNS = nsCocoaUtils::ToNSString(actionTitle);
+    NSUserNotificationAction* notificationAction =
+        [NSUserNotificationAction actionWithIdentifier:actionNameNS
+                                                 title:actionTitleNS];
+    [additionalActions addObject:notificationAction];
+  }
 
   // If this is not an application/extension alert, show additional actions
   // dealing with permissions.
   bool isActionable;
   if (bundle && NS_SUCCEEDED(aAlert->GetActionable(&isActionable)) &&
       isActionable) {
-    nsAutoString closeButtonTitle, actionButtonTitle, disableButtonTitle,
-        settingsButtonTitle;
-    bundle->GetStringFromName("closeButton.title", closeButtonTitle);
-    bundle->GetStringFromName("actionButton.label", actionButtonTitle);
+    nsAutoString disableButtonTitle;
     if (!hostPort.IsEmpty()) {
       AutoTArray<nsString, 1> formatStrings = {hostPort};
       bundle->FormatStringFromName("webActions.disableForOrigin.label",
                                    formatStrings, disableButtonTitle);
     }
+
+    nsAutoString settingsButtonTitle;
     bundle->GetStringFromName("webActions.settings.label", settingsButtonTitle);
 
-    notification.otherButtonTitle = nsCocoaUtils::ToNSString(closeButtonTitle);
+    NSString* actionNameNS = nsCocoaUtils::ToNSString(kAlertActionDisable);
+    NSString* actionTitleNS = nsCocoaUtils::ToNSString(disableButtonTitle);
+    NSUserNotificationAction* notificationAction =
+        [NSUserNotificationAction actionWithIdentifier:actionNameNS
+                                                 title:actionTitleNS];
+    [additionalActions addObject:notificationAction];
 
-    // OS X 10.8 only shows action buttons if the "Alerts" style is set in
-    // Notification Center preferences, and doesn't support the alternate
-    // action menu.
-    if ([notification respondsToSelector:@selector(set_showsButtons:)] &&
-        [notification
-            respondsToSelector:@selector(set_alwaysShowAlternateActionMenu:)] &&
-        [notification
-            respondsToSelector:@selector(set_alternateActionButtonTitles:)]) {
-      notification.hasActionButton = YES;
-      notification.actionButtonTitle =
-          nsCocoaUtils::ToNSString(actionButtonTitle);
-
-      [(NSObject*)notification setValue:@(YES) forKey:@"_showsButtons"];
-      [(NSObject*)notification setValue:@(YES)
-                                 forKey:@"_alwaysShowAlternateActionMenu"];
-      [(NSObject*)notification setValue:@[
-        nsCocoaUtils::ToNSString(disableButtonTitle),
-        nsCocoaUtils::ToNSString(settingsButtonTitle)
-      ]
-                                 forKey:@"_alternateActionButtonTitles"];
-    }
+    actionNameNS = nsCocoaUtils::ToNSString(kAlertActionSettings);
+    actionTitleNS = nsCocoaUtils::ToNSString(settingsButtonTitle);
+    notificationAction =
+        [NSUserNotificationAction actionWithIdentifier:actionNameNS
+                                                 title:actionTitleNS];
+    [additionalActions addObject:notificationAction];
   }
+
+  notification.additionalActions = additionalActions;
+  notification.hasActionButton = additionalActions.count == 0;
+  [additionalActions release];
+
   nsAutoString name;
   rv = aAlert->GetName(name);
   // Don't let an alert name be more than MAX_NOTIFICATION_NAME_LEN characters.
@@ -344,19 +306,7 @@ OSXNotificationCenter::ShowAlertWithIconData(nsIAlertNotification* aAlert,
   NS_ENSURE_SUCCESS(rv, rv);
 
   OSXNotificationInfo* osxni =
-      new OSXNotificationInfo(alertName, aAlertListener, cookie);
-
-  // Show the favicon if supported on this version of OS X.
-  if (aIconSize > 0 &&
-      [notification respondsToSelector:@selector(set_identityImage:)] &&
-      [notification
-          respondsToSelector:@selector(set_identityImageHasBorder:)]) {
-    NSData* iconData = [NSData dataWithBytes:aIconData length:aIconSize];
-    NSImage* icon = [[[NSImage alloc] initWithData:iconData] autorelease];
-
-    [(NSObject*)notification setValue:icon forKey:@"_identityImage"];
-    [(NSObject*)notification setValue:@(NO) forKey:@"_identityImageHasBorder"];
-  }
+      new OSXNotificationInfo(alertName, aAlert, aAlertListener, cookie);
 
   bool inPrivateBrowsing;
   rv = aAlert->GetInPrivateBrowsing(&inPrivateBrowsing);
@@ -400,6 +350,16 @@ OSXNotificationCenter::CloseAlert(const nsAString& aAlertName,
   NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
+NS_IMETHODIMP OSXNotificationCenter::Teardown() {
+  mPendingAlerts.Clear();
+  mActiveAlerts.Clear();
+  return NS_OK;
+}
+
+NS_IMETHODIMP OSXNotificationCenter::PbmTeardown() {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 void OSXNotificationCenter::CloseAlertCocoaString(NSString* aAlertName) {
   NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
 
@@ -408,11 +368,10 @@ void OSXNotificationCenter::CloseAlertCocoaString(NSString* aAlertName) {
   }
 
   NSArray* notifications = [GetNotificationCenter() deliveredNotifications];
-  for (id<FakeNSUserNotification> notification in notifications) {
+  for (NSUserNotification* notification in notifications) {
     NSString* name = [[notification userInfo] valueForKey:@"name"];
     if ([name isEqualToString:aAlertName]) {
       [GetNotificationCenter() removeDeliveredNotification:notification];
-      [GetNotificationCenter() _removeDisplayedNotification:notification];
       break;
     }
   }
@@ -438,7 +397,7 @@ void OSXNotificationCenter::CloseAlertCocoaString(NSString* aAlertName) {
 
 void OSXNotificationCenter::OnActivate(
     NSString* aAlertName, NSUserNotificationActivationType aActivationType,
-    unsigned long long aAdditionalActionIndex) {
+    NSUserNotificationAction* aAdditionalActivationAction) {
   NS_OBJC_BEGIN_TRY_IGNORE_BLOCK;
 
   if (!aAlertName) {
@@ -450,23 +409,34 @@ void OSXNotificationCenter::OnActivate(
     if ([aAlertName isEqualToString:osxni->mName]) {
       if (osxni->mObserver) {
         switch ((int)aActivationType) {
-          case NSUserNotificationActivationTypeAdditionalActionClicked:
-          case NSUserNotificationActivationTypeActionButtonClicked:
-            switch (aAdditionalActionIndex) {
-              case OSXNotificationActionDisable:
-                osxni->mObserver->Observe(nullptr, "alertdisablecallback",
-                                          osxni->mCookie.get());
-                break;
-              case OSXNotificationActionSettings:
-                osxni->mObserver->Observe(nullptr, "alertsettingscallback",
-                                          osxni->mCookie.get());
-                break;
-              default:
-                NS_WARNING(
-                    "Unknown NSUserNotification additional action clicked");
-                break;
+          case NSUserNotificationActivationTypeAdditionalActionClicked: {
+            MOZ_ASSERT(aAdditionalActivationAction);
+            nsAutoString actionName;
+            nsCocoaUtils::GetStringForNSString(
+                aAdditionalActivationAction.identifier, actionName);
+
+            if (actionName == kAlertActionDisable) {
+              osxni->mObserver->Observe(nullptr, "alertdisablecallback",
+                                        osxni->mCookie.get());
+              break;
             }
+            if (actionName == kAlertActionSettings) {
+              osxni->mObserver->Observe(nullptr, "alertsettingscallback",
+                                        osxni->mCookie.get());
+              break;
+            }
+
+            // Trim the suffix
+            actionName.Truncate(actionName.Length() - kActionSuffix.Length());
+
+            nsCOMPtr<nsIAlertAction> action;
+            osxni->mAlertNotification->GetAction(actionName,
+                                                 getter_AddRefs(action));
+            osxni->mObserver->Observe(action, "alertclickcallback",
+                                      osxni->mCookie.get());
             break;
+          }
+          case NSUserNotificationActivationTypeActionButtonClicked:
           default:
             osxni->mObserver->Observe(nullptr, "alertclickcallback",
                                       osxni->mCookie.get());
@@ -544,9 +514,11 @@ OSXNotificationCenter::OnImageReady(nsISupports* aUserData,
 
   NSImage* cocoaImage = nil;
   // TODO: Pass pres context / ComputedStyle here to support context paint
-  // properties
+  // properties.
+  // TODO: Do we have a reasonable size to pass around here?
   nsCocoaUtils::CreateDualRepresentationNSImageFromImageContainer(
-      image, imgIContainer::FRAME_FIRST, nullptr, nullptr, &cocoaImage);
+      image, imgIContainer::FRAME_FIRST, nullptr, NSMakeSize(0, 0),
+      &cocoaImage);
   (osxni->mPendingNotification).contentImage = cocoaImage;
   [cocoaImage release];
   ShowPendingNotification(osxni);
@@ -556,6 +528,16 @@ OSXNotificationCenter::OnImageReady(nsISupports* aUserData,
   NS_OBJC_END_TRY_BLOCK_RETURN(NS_ERROR_FAILURE);
 }
 
+NS_IMETHODIMP
+OSXNotificationCenter::GetHistory(nsTArray<nsString>& aResult) {
+  // NSUserNotificationCenter doesn't support this, blocked by the migration to
+  // UNUserNotificationCenter which has
+  // getDeliveredNotificationsWithCompletionHandler
+  // https://developer.apple.com/documentation/usernotifications/unusernotificationcenter/getdeliverednotifications(completionhandler:)?language=objc
+  // See bug 1971395.
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 // nsIAlertsDoNotDisturb
 NS_IMETHODIMP
 OSXNotificationCenter::GetManualDoNotDisturb(bool* aRetVal) {
