@@ -71,12 +71,12 @@ HeadlessWidget::HeadlessWidget()
       mVisible(false),
       mDestroyed(false),
       mAlwaysOnTop(false),
-      mTopLevel(nullptr),
       mCompositorWidget(nullptr),
       mSizeMode(nsSizeMode_Normal),
       mLastSizeMode(nsSizeMode_Normal),
       mEffectiveSizeMode(nsSizeMode_Normal),
       mRestoreBounds(0, 0, 0, 0) {
+  mWidgetType = WidgetType::Headless;
   if (!sActiveWindows) {
     sActiveWindows = new nsTArray<HeadlessWidget*>();
     ClearOnShutdown(&sActiveWindows);
@@ -117,53 +117,28 @@ void HeadlessWidget::Destroy() {
 }
 
 nsresult HeadlessWidget::Create(nsIWidget* aParent,
-                                nsNativeWidget aNativeParent,
                                 const LayoutDeviceIntRect& aRect,
                                 widget::InitData* aInitData) {
-  MOZ_ASSERT(!aNativeParent, "No native parents for headless widgets.");
-
-  BaseCreate(nullptr, aInitData);
+  BaseCreate(aParent, aInitData);
 
   mBounds = aRect;
   mRestoreBounds = aRect;
 
   mAlwaysOnTop = aInitData && aInitData->mAlwaysOnTop;
 
-  if (aParent) {
-    mTopLevel = aParent->GetTopLevelWidget();
-  } else {
-    mTopLevel = this;
-  }
-
   return NS_OK;
 }
 
-already_AddRefed<nsIWidget> HeadlessWidget::CreateChild(
-    const LayoutDeviceIntRect& aRect, widget::InitData* aInitData,
-    bool aForceUseIWidgetParent) {
-  nsCOMPtr<nsIWidget> widget = nsIWidget::CreateHeadlessWidget();
-  if (!widget) {
-    return nullptr;
-  }
-  if (NS_FAILED(widget->Create(this, nullptr, aRect, aInitData))) {
-    return nullptr;
-  }
-  return widget.forget();
-}
-
 void HeadlessWidget::GetCompositorWidgetInitData(
     mozilla::widget::CompositorWidgetInitData* aInitData) {
   *aInitData =
       mozilla::widget::HeadlessCompositorWidgetInitData(GetClientSize());
 }
 
-nsIWidget* HeadlessWidget::GetTopLevelWidget() { return mTopLevel; }
-
 void HeadlessWidget::RaiseWindow() {
-  MOZ_ASSERT(mWindowType == WindowType::TopLevel ||
-                 mWindowType == WindowType::Dialog ||
-                 mWindowType == WindowType::Sheet,
-             "Raising a non-toplevel window.");
+  MOZ_ASSERT(
+      mWindowType == WindowType::TopLevel || mWindowType == WindowType::Dialog,
+      "Raising a non-toplevel window.");
 
   // Do nothing if this is the currently active window.
   RefPtr<HeadlessWidget> activeWindow = GetActiveWindow();
@@ -171,13 +146,6 @@ void HeadlessWidget::RaiseWindow() {
     return;
   }
 
-  // Raise the window to the top of the stack.
-  nsWindowZ placement = nsWindowZTop;
-  nsCOMPtr<nsIWidget> actualBelow;
-  if (mWidgetListener)
-    mWidgetListener->ZLevelChanged(true, &placement, nullptr,
-                                   getter_AddRefs(actualBelow));
-
   // Deactivate the last active window.
   if (activeWindow && activeWindow->mWidgetListener) {
     activeWindow->mWidgetListener->WindowDeactivated();
@@ -204,7 +172,7 @@ void HeadlessWidget::Show(bool aState) {
   //     so we don't focus them by default.
   if (aState && !mAlwaysOnTop &&
       (mWindowType == WindowType::TopLevel ||
-       mWindowType == WindowType::Dialog || mWindowType == WindowType::Sheet)) {
+       mWindowType == WindowType::Dialog)) {
     RaiseWindow();
   }
 
@@ -223,7 +191,9 @@ void HeadlessWidget::SetFocus(Raise aRaise,
 
     // The toplevel only becomes active if it's currently visible; otherwise, it
     // will be activated anyway when it's shown.
-    if (topLevel->IsVisible()) topLevel->RaiseWindow();
+    if (topLevel->IsVisible()) {
+      topLevel->RaiseWindow();
+    }
   }
 }
 
@@ -260,7 +230,10 @@ void HeadlessWidget::MoveInternal(int32_t aX, int32_t aY) {
 }
 
 LayoutDeviceIntPoint HeadlessWidget::WidgetToScreenOffset() {
-  return mTopLevel->GetBounds().TopLeft();
+  if (mWindowType == WindowType::Popup) {
+    return mBounds.TopLeft();
+  }
+  return GetTopLevelWidget()->GetBounds().TopLeft();
 }
 
 WindowRenderer* HeadlessWidget::GetWindowRenderer() {
@@ -451,8 +424,8 @@ nsresult HeadlessWidget::DispatchEvent(WidgetGUIEvent* aEvent,
 nsresult HeadlessWidget::SynthesizeNativeMouseEvent(
     LayoutDeviceIntPoint aPoint, NativeMouseMessage aNativeMessage,
     MouseButton aButton, nsIWidget::Modifiers aModifierFlags,
-    nsIObserver* aObserver) {
-  AutoObserverNotifier notifier(aObserver, "mouseevent");
+    nsISynthesizedEventCallback* aCallback) {
+  AutoSynthesizedEventCallbackNotifier notifier(aCallback);
   EventMessage msg;
   switch (aNativeMessage) {
     case NativeMouseMessage::Move:
@@ -485,9 +458,8 @@ nsresult HeadlessWidget::SynthesizeNativeMouseEvent(
 nsresult HeadlessWidget::SynthesizeNativeMouseScrollEvent(
     mozilla::LayoutDeviceIntPoint aPoint, uint32_t aNativeMessage,
     double aDeltaX, double aDeltaY, double aDeltaZ, uint32_t aModifierFlags,
-    uint32_t aAdditionalFlags, nsIObserver* aObserver) {
-  AutoObserverNotifier notifier(aObserver, "mousescrollevent");
-  printf(">>> DEBUG_ME: Synth: aDeltaY=%f\n", aDeltaY);
+    uint32_t aAdditionalFlags, nsISynthesizedEventCallback* aCallback) {
+  AutoSynthesizedEventCallbackNotifier notifier(aCallback);
   // The various platforms seem to handle scrolling deltas differently,
   // but the following seems to emulate it well enough.
   WidgetWheelEvent event(true, eWheel, this);
@@ -505,8 +477,8 @@ nsresult HeadlessWidget::SynthesizeNativeMouseScrollEvent(
 nsresult HeadlessWidget::SynthesizeNativeTouchPoint(
     uint32_t aPointerId, TouchPointerState aPointerState,
     LayoutDeviceIntPoint aPoint, double aPointerPressure,
-    uint32_t aPointerOrientation, nsIObserver* aObserver) {
-  AutoObserverNotifier notifier(aObserver, "touchpoint");
+    uint32_t aPointerOrientation, nsISynthesizedEventCallback* aCallback) {
+  AutoSynthesizedEventCallbackNotifier notifier(aCallback);
 
   MOZ_ASSERT(NS_IsMainThread());
   if (aPointerState == TOUCH_HOVER) {
@@ -584,8 +556,8 @@ nsresult HeadlessWidget::SynthesizeNativeTouchPadPinch(
 nsresult HeadlessWidget::SynthesizeNativeTouchpadPan(
     TouchpadGesturePhase aEventPhase, LayoutDeviceIntPoint aPoint,
     double aDeltaX, double aDeltaY, int32_t aModifierFlags,
-    nsIObserver* aObserver) {
-  AutoObserverNotifier notifier(aObserver, "touchpadpanevent");
+    nsISynthesizedEventCallback* aCallback) {
+  AutoSynthesizedEventCallbackNotifier notifier(aCallback);
 
   MOZ_ASSERT(NS_IsMainThread());
 
