@@ -22,7 +22,7 @@ registerCleanupFunction(() => {
 const progressListeners = new Map();
 
 function loadContentWindow(windowlessBrowser, uri) {
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     let loadURIOptions = {
       triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(),
     };
@@ -46,7 +46,7 @@ function loadContentWindow(windowlessBrowser, uri) {
         progressListeners.delete(progressListener);
         contentWindow.addEventListener(
           "load",
-          event => {
+          () => {
             resolve(contentWindow);
           },
           { once: true }
@@ -65,6 +65,14 @@ function loadContentWindow(windowlessBrowser, uri) {
   });
 }
 
+add_setup(function () {
+  Services.prefs.setBoolPref("security.allow_unsafe_parent_loads", true);
+});
+
+registerCleanupFunction(function () {
+  Services.prefs.clearUserPref("security.allow_unsafe_parent_loads");
+});
+
 add_task(async function test_snapshot() {
   let windowlessBrowser = Services.appShell.createWindowlessBrowser(false);
   let contentWindow = await loadContentWindow(windowlessBrowser, HEADLESS_URL);
@@ -200,15 +208,14 @@ add_task(async function test_mouse_drag() {
   let left = rect.left;
   let top = rect.top;
 
-  let utils = contentWindow.windowUtils;
-  utils.sendMouseEvent("mousedown", left, top, 0, 1, 0, false, 0, 0);
-  utils.sendMouseEvent("mousemove", left, top, 0, 1, 0, false, 0, 0);
+  contentWindow.synthesizeMouseEvent("mousedown", left, top);
+  contentWindow.synthesizeMouseEvent("mousemove", left, top);
   // Wait for a turn of the event loop since the synthetic mouse event
   // that creates the drag service is processed during the refresh driver.
   await new Promise(r => {
     executeSoon(r);
   });
-  utils.sendMouseEvent("mouseup", left, top, 0, 1, 0, false, 0, 0);
+  contentWindow.synthesizeMouseEvent("mouseup", left, top);
 
   ok(true, "Send mouse event didn't crash");
 
