@@ -8,18 +8,23 @@
 #include "nsMemoryReporterManager.h"
 
 #include "mozilla/ClearOnShutdown.h"
+#ifdef MOZ_PHC
+#  include "mozilla/PHCManager.h"
+#endif
 #include "mozilla/Result.h"
 #include "mozilla/ResultExtensions.h"
 #include "mozilla/Services.h"
 #include "mozilla/ScopeExit.h"
 #include "mozilla/SimpleEnumerator.h"
+#include "mozilla/glean/XpcomMetrics.h"
 #include "mozilla/Telemetry.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/dom/ContentParent.h"
+#include "mozilla/dom/ContentChild.h"
 #include "mozilla/dom/ScriptSettings.h"
 #include "nsContentUtils.h"
+#include "nsGlobalWindowOuter.h"
 #include "nsIBrowserDOMWindow.h"
-#include "nsIDOMChromeWindow.h"
 #include "nsIMemoryReporter.h"
 #include "nsIWindowMediator.h"
 #include "nsImportModule.h"
@@ -38,22 +43,29 @@ using namespace mozilla;
 using mozilla::dom::AutoJSAPI;
 using mozilla::dom::ContentParent;
 
-// Do not gather data more than once a minute (ms)
-static constexpr uint32_t kTelemetryInterval = 60 * 1000;
+// Do not gather data more than once a minute (in seconds)
+static constexpr uint32_t kTelemetryIntervalS = 60;
 
-static constexpr const char* kTopicCycleCollectorBegin =
-    "cycle-collector-begin";
+// Do not create a timer for telemetry this many seconds after the previous one
+// fires.  This exists so that we don't respond to our own timer.
+static constexpr uint32_t kTelemetryCooldownS = 10;
+
+// We use a sliding window to detect a reasonable amount of activity.  If there
+// are more than kPokeWindowEvents events within kPokeWindowSeconds seconds then
+// that counts as "active".
+static constexpr unsigned kPokeWindowEvents = 10;
+static constexpr unsigned kPokeWindowSeconds = 1;
+
+static constexpr const char* kTopicShutdown = "content-child-shutdown";
 
 namespace {
 
 enum class PrevValue : uint32_t {
 #ifdef XP_WIN
-  LOW_MEMORY_EVENTS_VIRTUAL,
-  LOW_MEMORY_EVENTS_COMMIT_SPACE,
-  LOW_MEMORY_EVENTS_PHYSICAL,
+  low_memory_events_physical,
 #endif
 #if defined(XP_LINUX) && !defined(ANDROID)
-  PAGE_FAULTS_HARD,
+  page_faults_hard,
 #endif
   SIZE_,
 };
@@ -64,25 +76,48 @@ constexpr uint32_t kUninitialized = ~0;
 
 static uint32_t gPrevValues[uint32_t(PrevValue::SIZE_)];
 
-static uint32_t PrevValueIndex(Telemetry::HistogramID aId) {
-  switch (aId) {
-#ifdef XP_WIN
-    case Telemetry::LOW_MEMORY_EVENTS_VIRTUAL:
-      return uint32_t(PrevValue::LOW_MEMORY_EVENTS_VIRTUAL);
-    case Telemetry::LOW_MEMORY_EVENTS_COMMIT_SPACE:
-      return uint32_t(PrevValue::LOW_MEMORY_EVENTS_COMMIT_SPACE);
-    case Telemetry::LOW_MEMORY_EVENTS_PHYSICAL:
-      return uint32_t(PrevValue::LOW_MEMORY_EVENTS_PHYSICAL);
-#endif
-#if defined(XP_LINUX) && !defined(ANDROID)
-    case Telemetry::PAGE_FAULTS_HARD:
-      return uint32_t(PrevValue::PAGE_FAULTS_HARD);
-#endif
-    default:
-      MOZ_ASSERT_UNREACHABLE("Unexpected histogram ID");
-      return 0;
+/*
+ * Because even in "idle" processes there may be some background events,
+ * ideally there shouldn't, we use a sliding window to determine if the process
+ * is active or not.  If there are N recent calls to Poke() the browser is
+ * active.
+ *
+ * This class implements the sliding window of timestamps.
+ */
+class TimeStampWindow {
+ public:
+  void push(TimeStamp aNow) {
+    mEvents.insertBack(new Event(aNow));
+    mNumEvents++;
   }
-}
+
+  // Remove any events older than aOld.
+  void clearExpired(TimeStamp aOld) {
+    Event* e = mEvents.getFirst();
+    while (e && e->olderThan(aOld)) {
+      e->removeFrom(mEvents);
+      mNumEvents--;
+      delete e;
+      e = mEvents.getFirst();
+    }
+  }
+
+  size_t numEvents() const { return mNumEvents; }
+
+ private:
+  class Event : public LinkedListElement<Event> {
+   public:
+    explicit Event(TimeStamp aTime) : mTime(aTime) {}
+
+    bool olderThan(TimeStamp aOld) const { return mTime < aOld; }
+
+   private:
+    TimeStamp mTime;
+  };
+
+  size_t mNumEvents = 0;
+  AutoCleanLinkedList<Event> mEvents;
+};
 
 NS_IMPL_ISUPPORTS(MemoryTelemetry, nsIObserver, nsISupportsWeakReference)
 
@@ -98,7 +133,7 @@ void MemoryTelemetry::Init() {
     nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
     MOZ_RELEASE_ASSERT(obs);
 
-    obs->AddObserver(this, "content-child-shutdown", true);
+    obs->AddObserver(this, kTopicShutdown, true);
   }
 }
 
@@ -115,79 +150,94 @@ void MemoryTelemetry::Init() {
   return *sInstance;
 }
 
-nsresult MemoryTelemetry::DelayedInit() {
-  if (Telemetry::CanRecordExtended()) {
-    nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
-    MOZ_RELEASE_ASSERT(obs);
+void MemoryTelemetry::DelayedInit() {
+  mCanRun = true;
+  Poke();
+}
 
-    obs->AddObserver(this, kTopicCycleCollectorBegin, true);
+void MemoryTelemetry::Poke() {
+  // Don't do anything that might delay process startup
+  if (!mCanRun) {
+    return;
   }
 
-  GatherReports();
+  if (XRE_IsContentProcess() && !Telemetry::CanRecordReleaseData()) {
+    // All memory telemetry produced by content processes is release data, so if
+    // we're not recording release data then don't setup the timers on content
+    // processes.
+    return;
+  }
 
-  return NS_OK;
+  if (XRE_IsContentProcess()) {
+    auto& remoteType = dom::ContentChild::GetSingleton()->GetRemoteType();
+    if (remoteType == PREALLOC_REMOTE_TYPE) {
+      // Preallocated processes should stay dormant and not run this telemetry
+      // code.
+      return;
+    }
+  }
+
+  TimeStamp now = TimeStamp::Now();
+  if (mPokeWindow) {
+    mPokeWindow->clearExpired(now -
+                              TimeDuration::FromSeconds(kPokeWindowSeconds));
+  }
+
+  if (mLastRun &&
+      now - mLastRun < TimeDuration::FromSeconds(kTelemetryCooldownS)) {
+    // If we last gathered telemetry less than kTelemetryCooldownS seconds ago
+    // then Poke() does nothing.  This is to prevent our own timer waking us up.
+    // In the condition above `now - mLastRun` is how long ago we last gathered
+    // telemetry.
+    return;
+  }
+
+  // Even idle processes have some events, so we only want to create the timer
+  // if there's been several events in the last small window.
+  if (!mPokeWindow) {
+    mPokeWindow = MakeUnique<TimeStampWindow>();
+  }
+  mPokeWindow->push(now);
+  if (mPokeWindow->numEvents() < kPokeWindowEvents) {
+    return;
+  }
+  mPokeWindow = nullptr;
+
+  mLastPoke = now;
+  if (!mTimer) {
+    TimeDuration delay = TimeDuration::FromSeconds(kTelemetryIntervalS);
+    if (mLastRun) {
+      delay = std::min(delay,
+                       std::max(TimeDuration::FromSeconds(kTelemetryCooldownS),
+                                delay - (now - mLastRun)));
+    }
+    RefPtr<MemoryTelemetry> self(this);
+    auto res = NS_NewTimerWithCallback(
+        [self](nsITimer* aTimer) { self->GatherReports(); }, delay,
+        nsITimer::TYPE_ONE_SHOT_LOW_PRIORITY,
+        "MemoryTelemetry::GatherReports"_ns);
+
+    if (res.isOk()) {
+      // Errors are ignored, if there was an error then we just don't get
+      // telemetry.
+      mTimer = res.unwrap();
+    }
+  }
 }
 
 nsresult MemoryTelemetry::Shutdown() {
+  if (mTimer) {
+    mTimer->Cancel();
+  }
+
   nsCOMPtr<nsIObserverService> obs = services::GetObserverService();
   MOZ_RELEASE_ASSERT(obs);
 
-  obs->RemoveObserver(this, kTopicCycleCollectorBegin);
+  obs->RemoveObserver(this, kTopicShutdown);
 
   return NS_OK;
 }
 
-static inline void HandleMemoryReport(Telemetry::HistogramID aId,
-                                      int32_t aUnits, uint64_t aAmount,
-                                      const nsCString& aKey = VoidCString()) {
-  uint32_t val;
-  switch (aUnits) {
-    case nsIMemoryReporter::UNITS_BYTES:
-      val = uint32_t(aAmount / 1024);
-      break;
-
-    case nsIMemoryReporter::UNITS_PERCENTAGE:
-      // UNITS_PERCENTAGE amounts are 100x greater than their raw value.
-      val = uint32_t(aAmount / 100);
-      break;
-
-    case nsIMemoryReporter::UNITS_COUNT:
-      val = uint32_t(aAmount);
-      break;
-
-    case nsIMemoryReporter::UNITS_COUNT_CUMULATIVE: {
-      // If the reporter gives us a cumulative count, we'll report the
-      // difference in its value between now and our previous ping.
-
-      uint32_t idx = PrevValueIndex(aId);
-      uint32_t prev = gPrevValues[idx];
-      gPrevValues[idx] = aAmount;
-
-      if (prev == kUninitialized) {
-        // If this is the first time we're reading this reporter, store its
-        // current value but don't report it in the telemetry ping, so we
-        // ignore the effect startup had on the reporter.
-        return;
-      }
-      val = aAmount - prev;
-      break;
-    }
-
-    default:
-      MOZ_ASSERT_UNREACHABLE("Unexpected aUnits value");
-      return;
-  }
-
-  // Note: The reference equality check here should allow the compiler to
-  // optimize this case out at compile time when we weren't given a key,
-  // while IsEmpty() or IsVoid() most likely will not.
-  if (&aKey == &VoidCString()) {
-    Telemetry::Accumulate(aId, val);
-  } else {
-    Telemetry::Accumulate(aId, aKey, val);
-  }
-}
-
 nsresult MemoryTelemetry::GatherReports(
     const std::function<void()>& aCompletionCallback) {
   auto cleanup = MakeScopeExit([&]() {
@@ -196,23 +246,43 @@ nsresult MemoryTelemetry::GatherReports(
     }
   });
 
+  mLastRun = TimeStamp::Now();
+  mTimer = nullptr;
+
   RefPtr<nsMemoryReporterManager> mgr = nsMemoryReporterManager::GetOrCreate();
   MOZ_DIAGNOSTIC_ASSERT(mgr);
   NS_ENSURE_TRUE(mgr, NS_ERROR_FAILURE);
 
-#define RECORD(id, metric, units)                                       \
-  do {                                                                  \
-    int64_t amt;                                                        \
-    nsresult rv = mgr->Get##metric(&amt);                               \
-    if (NS_SUCCEEDED(rv)) {                                             \
-      HandleMemoryReport(Telemetry::id, nsIMemoryReporter::units, amt); \
-    } else if (rv != NS_ERROR_NOT_AVAILABLE) {                          \
-      NS_WARNING("Failed to retrieve memory telemetry for " #metric);   \
-    }                                                                   \
+#define RECORD_OUTER(metric, inner)                                   \
+  do {                                                                \
+    int64_t amt;                                                      \
+    nsresult rv = mgr->Get##metric(&amt);                             \
+    if (NS_SUCCEEDED(rv)) {                                           \
+      inner                                                           \
+    } else if (rv != NS_ERROR_NOT_AVAILABLE) {                        \
+      NS_WARNING("Failed to retrieve memory telemetry for " #metric); \
+    }                                                                 \
   } while (0)
+#define RECORD_COUNT(id, metric) \
+  RECORD_OUTER(metric, glean::memory::id.AccumulateSingleSample(amt);)
+#define RECORD_BYTES(id, metric) \
+  RECORD_OUTER(metric, glean::memory::id.Accumulate(amt / 1024);)
+#define RECORD_PERCENTAGE(id, metric) \
+  RECORD_OUTER(metric, glean::memory::id.AccumulateSingleSample(amt / 100);)
+#define RECORD_COUNT_CUMULATIVE(id, metric)                               \
+  RECORD_OUTER(                                                           \
+      metric, uint32_t prev = gPrevValues[uint32_t(PrevValue::id)];       \
+      gPrevValues[uint32_t(PrevValue::id)] = amt;                         \
+                                                                          \
+      /* If this is the first time we're reading this reporter, store its \
+       * current value but don't report it in the telemetry ping, so we   \
+       * ignore the effect startup had on the reporter. */                \
+      if (prev != kUninitialized) {                                       \
+        glean::memory::id.AccumulateSingleSample(amt - prev);             \
+      })
 
   // GHOST_WINDOWS is opt-out as of Firefox 55
-  RECORD(GHOST_WINDOWS, GhostWindows, UNITS_COUNT);
+  RECORD_COUNT(ghost_windows, GhostWindows);
 
   // If we're running in the parent process, collect data from all processes for
   // the MEMORY_TOTAL histogram.
@@ -243,22 +313,30 @@ nsresult MemoryTelemetry::GatherReports(
 
   // Collect cheap or main-thread only metrics synchronously, on the main
   // thread.
-  RECORD(MEMORY_JS_GC_HEAP, JSMainRuntimeGCHeap, UNITS_BYTES);
-  RECORD(MEMORY_JS_COMPARTMENTS_SYSTEM, JSMainRuntimeCompartmentsSystem,
-         UNITS_COUNT);
-  RECORD(MEMORY_JS_COMPARTMENTS_USER, JSMainRuntimeCompartmentsUser,
-         UNITS_COUNT);
-  RECORD(MEMORY_JS_REALMS_SYSTEM, JSMainRuntimeRealmsSystem, UNITS_COUNT);
-  RECORD(MEMORY_JS_REALMS_USER, JSMainRuntimeRealmsUser, UNITS_COUNT);
-  RECORD(MEMORY_IMAGES_CONTENT_USED_UNCOMPRESSED, ImagesContentUsedUncompressed,
-         UNITS_BYTES);
-  RECORD(MEMORY_STORAGE_SQLITE, StorageSQLite, UNITS_BYTES);
+  RECORD_BYTES(js_gc_heap, JSMainRuntimeGCHeap);
+  RECORD_COUNT(js_compartments_system, JSMainRuntimeCompartmentsSystem);
+  RECORD_COUNT(js_compartments_user, JSMainRuntimeCompartmentsUser);
+  RECORD_COUNT(js_realms_system, JSMainRuntimeRealmsSystem);
+  RECORD_COUNT(js_realms_user, JSMainRuntimeRealmsUser);
+  RECORD_BYTES(images_content_used_uncompressed, ImagesContentUsedUncompressed);
+  RECORD_BYTES(storage_sqlite, StorageSQLite);
 #ifdef XP_WIN
-  RECORD(LOW_MEMORY_EVENTS_PHYSICAL, LowMemoryEventsPhysical,
-         UNITS_COUNT_CUMULATIVE);
+  RECORD_COUNT_CUMULATIVE(low_memory_events_physical, LowMemoryEventsPhysical);
 #endif
 #if defined(XP_LINUX) && !defined(ANDROID)
-  RECORD(PAGE_FAULTS_HARD, PageFaultsHard, UNITS_COUNT_CUMULATIVE);
+  RECORD_COUNT_CUMULATIVE(page_faults_hard, PageFaultsHard);
+#endif
+
+#ifdef HAVE_JEMALLOC_STATS
+  jemalloc_stats_t stats;
+  jemalloc_stats(&stats);
+  glean::memory::heap_allocated.Accumulate(mgr->HeapAllocated(stats) / 1024);
+  glean::memory::heap_overhead_fraction.AccumulateSingleSample(
+      mgr->HeapOverheadFraction(stats) / 100);
+#endif
+
+#ifdef MOZ_PHC
+  ReportPHCTelemetry();
 #endif
 
   RefPtr<Runnable> completionRunnable;
@@ -270,21 +348,23 @@ nsresult MemoryTelemetry::GatherReports(
   // asynchronously, on a background thread.
   RefPtr<Runnable> runnable = NS_NewRunnableFunction(
       "MemoryTelemetry::GatherReports", [mgr, completionRunnable]() mutable {
-        Telemetry::AutoTimer<Telemetry::MEMORY_COLLECTION_TIME> autoTimer;
-        RECORD(MEMORY_VSIZE, Vsize, UNITS_BYTES);
-#if !defined(HAVE_64BIT_BUILD) || !defined(XP_WIN)
-        RECORD(MEMORY_VSIZE_MAX_CONTIGUOUS, VsizeMaxContiguous, UNITS_BYTES);
+        auto timer = glean::memory::collection_time.Measure();
+// Each WebAssembly program eats up an entire 32-bits worth of address space,
+// which makes vsize rather useless on 64-bit systems, and will cause telemetry
+// to frequently hit the max value of 1TB, so only record it in 32-bit builds.
+#if !defined(HAVE_64BIT_BUILD)
+        RECORD_BYTES(vsize, Vsize);
 #endif
-        RECORD(MEMORY_RESIDENT_FAST, ResidentFast, UNITS_BYTES);
-        RECORD(MEMORY_RESIDENT_PEAK, ResidentPeak, UNITS_BYTES);
+#if !defined(HAVE_64BIT_BUILD) || !defined(XP_WIN)
+        RECORD_BYTES(vsize_max_contiguous, VsizeMaxContiguous);
+#endif
+        RECORD_BYTES(resident_fast, ResidentFast);
+        RECORD_BYTES(resident_peak, ResidentPeak);
 // Although we can measure unique memory on MacOS we choose not to, because
 // doing so is too slow for telemetry.
 #ifndef XP_MACOSX
-        RECORD(MEMORY_UNIQUE, ResidentUnique, UNITS_BYTES);
+        RECORD_BYTES(unique, ResidentUnique);
 #endif
-        RECORD(MEMORY_HEAP_ALLOCATED, HeapAllocated, UNITS_BYTES);
-        RECORD(MEMORY_HEAP_OVERHEAD_FRACTION, HeapOverheadFraction,
-               UNITS_PERCENTAGE);
 
         if (completionRunnable) {
           NS_DispatchToMainThread(completionRunnable.forget(),
@@ -371,17 +451,22 @@ void MemoryTelemetry::GatherTotalMemory() {
 
         // Use our handle for the remote process to collect resident unique set
         // size information for that process.
+        bool success = true;
         for (const auto& info : infos) {
 #ifdef XP_MACOSX
           int64_t memory =
               nsMemoryReporterManager::PhysicalFootprint(info.mHandle);
 #else
-	  int64_t memory =
-	      nsMemoryReporterManager::ResidentUnique(info.mHandle);
+          int64_t memory =
+              nsMemoryReporterManager::ResidentUnique(info.mHandle);
 #endif
           if (memory > 0) {
             childSizes.AppendElement(memory);
             totalMemory += memory;
+          } else {
+            // We don't break out of the loop otherwise the cleanup code
+            // wouldn't run.
+            success = false;
           }
 
 #if defined(XP_WIN)
@@ -391,17 +476,22 @@ void MemoryTelemetry::GatherTotalMemory() {
 #endif
         }
 
+        Maybe<int64_t> mbTotal;
+        if (success) {
+          mbTotal = Some(totalMemory);
+        }
+
         NS_DispatchToMainThread(NS_NewRunnableFunction(
             "MemoryTelemetry::FinishGatheringTotalMemory",
-            [totalMemory, childSizes = std::move(childSizes)] {
-              MemoryTelemetry::Get().FinishGatheringTotalMemory(totalMemory,
+            [mbTotal, childSizes = std::move(childSizes)] {
+              MemoryTelemetry::Get().FinishGatheringTotalMemory(mbTotal,
                                                                 childSizes);
             }));
       }));
 }
 
 nsresult MemoryTelemetry::FinishGatheringTotalMemory(
-    int64_t aTotalMemory, const nsTArray<int64_t>& aChildSizes) {
+    Maybe<int64_t> aTotalMemory, const nsTArray<int64_t>& aChildSizes) {
   mGatheringTotalMemory = false;
 
   // Total memory usage can be difficult to measure both accurately and fast
@@ -410,8 +500,9 @@ nsresult MemoryTelemetry::FinishGatheringTotalMemory(
   // especially on MacOS where it double-counts shared memory.  For a more
   // detailed explaination see:
   // https://groups.google.com/a/mozilla.org/g/dev-platform/c/WGNOtjHdsdA
-  HandleMemoryReport(Telemetry::MEMORY_TOTAL, nsIMemoryReporter::UNITS_BYTES,
-                     aTotalMemory);
+  if (aTotalMemory) {
+    glean::memory::total.Accumulate(aTotalMemory.value() / 1024);
+  }
 
   if (aChildSizes.Length() > 1) {
     int32_t tabsCount;
@@ -447,8 +538,8 @@ nsresult MemoryTelemetry::FinishGatheringTotalMemory(
     for (auto size : aChildSizes) {
       int64_t diff = llabs(size - mean) * 100 / mean;
 
-      HandleMemoryReport(Telemetry::MEMORY_DISTRIBUTION_AMONG_CONTENT,
-                         nsIMemoryReporter::UNITS_COUNT, diff, key);
+      glean::memory::distribution_among_content.Get(key).AccumulateSingleSample(
+          diff);
     }
   }
 
@@ -472,9 +563,9 @@ nsresult MemoryTelemetry::FinishGatheringTotalMemory(
                                         getter_AddRefs(enumerator)));
 
   uint32_t total = 0;
-  for (auto& window : SimpleEnumerator<nsIDOMChromeWindow>(enumerator)) {
-    nsCOMPtr<nsIBrowserDOMWindow> browserWin;
-    MOZ_TRY(window->GetBrowserDOMWindow(getter_AddRefs(browserWin)));
+  for (const auto& window : SimpleEnumerator<nsPIDOMWindowOuter>(enumerator)) {
+    nsCOMPtr<nsIBrowserDOMWindow> browserWin =
+        nsGlobalWindowOuter::Cast(window)->GetBrowserDOMWindow();
 
     NS_ENSURE_TRUE(browserWin, Err(NS_ERROR_UNEXPECTED));
 
@@ -488,21 +579,7 @@ nsresult MemoryTelemetry::FinishGatheringTotalMemory(
 
 nsresult MemoryTelemetry::Observe(nsISupports* aSubject, const char* aTopic,
                                   const char16_t* aData) {
-  if (strcmp(aTopic, kTopicCycleCollectorBegin) == 0) {
-    auto now = TimeStamp::Now();
-    if (!mLastPoll.IsNull() &&
-        (now - mLastPoll).ToMilliseconds() < kTelemetryInterval) {
-      return NS_OK;
-    }
-
-    mLastPoll = now;
-
-    NS_DispatchToCurrentThreadQueue(
-        NewRunnableMethod<std::function<void()>>(
-            "MemoryTelemetry::GatherReports", this,
-            &MemoryTelemetry::GatherReports, nullptr),
-        EventQueuePriority::Idle);
-  } else if (strcmp(aTopic, "content-child-shutdown") == 0) {
+  if (strcmp(aTopic, kTopicShutdown) == 0) {
     if (nsCOMPtr<nsITelemetry> telemetry =
             do_GetService("@mozilla.org/base/telemetry;1")) {
       telemetry->FlushBatchedChildTelemetry();
