@@ -1,12 +1,17 @@
 #include "gtest/gtest.h"
 
 #include "mozilla/NonBlockingAsyncInputStream.h"
+#include "mozilla/SpinEventLoopUntil.h"
 #include "nsIAsyncInputStream.h"
+#include "nsIThread.h"
 #include "nsStreamUtils.h"
 #include "nsString.h"
 #include "nsStringStream.h"
 #include "Helpers.h"
 
+using mozilla::NonBlockingAsyncInputStream;
+using mozilla::SpinEventLoopUntil;
+
 TEST(TestNonBlockingAsyncInputStream, Simple)
 {
   nsCString data;
@@ -130,7 +135,9 @@ TEST(TestNonBlockingAsyncInputStream, AsyncWait_Simple)
 
   ASSERT_EQ(NS_OK, async->AsyncWait(cb, 0, 0, thread));
   ASSERT_FALSE(cb->Called());
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() { return cb->Called(); }));
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "xpcom:TEST(TestNonBlockingAsyncInputStream, AsyncWait_Simple)"_ns,
+      [&]() { return cb->Called(); }));
   ASSERT_TRUE(cb->Called());
 
   // Read works fine.
@@ -197,7 +204,9 @@ TEST(TestNonBlockingAsyncInputStream, AsyncWait_ClosureOnly_withEventTarget)
   ASSERT_EQ(NS_OK, async->Close());
   ASSERT_FALSE(cb->Called());
 
-  MOZ_ALWAYS_TRUE(SpinEventLoopUntil([&]() { return cb->Called(); }));
+  MOZ_ALWAYS_TRUE(SpinEventLoopUntil(
+      "xpcom:TEST(TestNonBlockingAsyncInputStream, AsyncWait_ClosureOnly_withEventTarget)"_ns,
+      [&]() { return cb->Called(); }));
   ASSERT_TRUE(cb->Called());
 }
 
@@ -254,6 +263,7 @@ class QIInputStream final : public nsIInputStream,
   // nsIInputStream
   NS_IMETHOD Close() override { return NS_ERROR_NOT_IMPLEMENTED; }
   NS_IMETHOD Available(uint64_t*) override { return NS_ERROR_NOT_IMPLEMENTED; }
+  NS_IMETHOD StreamStatus() override { return NS_ERROR_NOT_IMPLEMENTED; }
   NS_IMETHOD Read(char*, uint32_t, uint32_t*) override {
     return NS_ERROR_NOT_IMPLEMENTED;
   }
@@ -273,18 +283,11 @@ class QIInputStream final : public nsIInputStream,
   }
 
   // nsIIPCSerializableInputStream
-  void Serialize(mozilla::ipc::InputStreamParams&, FileDescriptorArray&, bool,
-                 uint32_t, uint32_t*, mozilla::dom::ContentChild*) override {}
-  void Serialize(mozilla::ipc::InputStreamParams&, FileDescriptorArray&, bool,
-                 uint32_t, uint32_t*,
-                 mozilla::ipc::PBackgroundChild*) override {}
-  void Serialize(mozilla::ipc::InputStreamParams&, FileDescriptorArray&, bool,
-                 uint32_t, uint32_t*, mozilla::dom::ContentParent*) override {}
-  void Serialize(mozilla::ipc::InputStreamParams&, FileDescriptorArray&, bool,
-                 uint32_t, uint32_t*,
-                 mozilla::ipc::PBackgroundParent*) override {}
-  bool Deserialize(const mozilla::ipc::InputStreamParams&,
-                   const FileDescriptorArray&) override {
+  void SerializedComplexity(uint32_t, uint32_t*, uint32_t*,
+                            uint32_t*) override {}
+  void Serialize(mozilla::ipc::InputStreamParams&, uint32_t,
+                 uint32_t*) override {}
+  bool Deserialize(const mozilla::ipc::InputStreamParams&) override {
     return false;
   }
 
