@@ -10,12 +10,8 @@
 #include "nsISupports.h"
 #include "mozilla/TimeStamp.h"
 
-#define NS_IIDLERUNNABLE_IID                         \
-  {                                                  \
-    0x688be92e, 0x7ade, 0x4fdc, {                    \
-      0x9d, 0x83, 0x74, 0xcb, 0xef, 0xf4, 0xa5, 0x2c \
-    }                                                \
-  }
+#define NS_IIDLERUNNABLE_IID \
+  {0x688be92e, 0x7ade, 0x4fdc, {0x9d, 0x83, 0x74, 0xcb, 0xef, 0xf4, 0xa5, 0x2c}}
 
 class nsIEventTarget;
 
@@ -25,7 +21,7 @@ class nsIEventTarget;
  */
 class nsIIdleRunnable : public nsISupports {
  public:
-  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IIDLERUNNABLE_IID)
+  NS_INLINE_DECL_STATIC_IID(NS_IIDLERUNNABLE_IID)
 
   /**
    * Notify the task of a point in time in the future when the task
@@ -43,6 +39,4 @@ class nsIIdleRunnable : public nsISupports {
   virtual ~nsIIdleRunnable() = default;
 };
 
-NS_DEFINE_STATIC_IID_ACCESSOR(nsIIdleRunnable, NS_IIDLERUNNABLE_IID)
-
 #endif  // nsIIdleRunnable_h__
